@@ -499,8 +499,10 @@ inline int CreateAppOutputDir(
     char *path = (char*)malloc(sizeof(char)*bufSize);
     char *outputDirectoryCopy = (char*)malloc(sizeof(char)*bufSize);
 
-    if (NULL == path)
+    if (!path || !outputDirectoryCopy)
     {
+        free(path);
+        free(outputDirectoryCopy);
         return -1;
     }
 
@@ -598,6 +600,7 @@ inline int CreateAppOutputDir(
                 {
                     pathBuf[0] = '\0';
                     free(path);
+                    free(outputDirectoryCopy);
                     return ret;
                 }
             }
@@ -607,6 +610,7 @@ inline int CreateAppOutputDir(
                 {
                     pathBuf[0]= '\0';
                     free(path);
+                    free(outputDirectoryCopy);
                     return ret;
                 }
             }
@@ -630,6 +634,7 @@ inline int CreateAppOutputDir(
     }
 
     free(path);
+    free(outputDirectoryCopy);
 #endif //#elif !defined(_WIN32)
     return ret;
 }
