@@ -31,6 +31,7 @@ enum {
     FLAG_PS_SIMD_MODE_FORCE_SIMD32 = 4,            // Force SIMD32 compilation
 };
 
+
 enum {
     // VISAPreSchedRPThreshold
     FLAG_VISA_PRE_SCHED_RP_THRESHOLD = 0,                 // 0 means use the default
@@ -83,6 +84,8 @@ typedef struct {
 #define SetSimdSpill(MODE, stats)    (stats = (stats | (BIT_CG_SIMD##MODE) |  (BIT_CG_SPILL##MODE)))
 #define SetSimdNoSpill(MODE, stats)  (stats = (stats | (BIT_CG_SIMD##MODE) & ~(BIT_CG_SPILL##MODE)))
 
+
+
 typedef enum CG_FLAG_t {
     FLAG_CG_ALL_SIMDS = 0,
     FLAG_CG_STAGE1_FAST_COMPILE = 1,
@@ -91,6 +94,8 @@ typedef enum CG_FLAG_t {
 } CG_FLAG_t;
 
 #define IsSupportedForStagedCompilation(platform, product) (true)
+#define IsSupportedForDX12StaticSampler(platform, product) (true)
+#define IsSupportedForFastestSingleCSSIMD(platform, product) (true)
 #define RequestStage2(flag, ctx_ptr) (ctx_ptr != nullptr || flag == FLAG_CG_STAGE1_FASTEST_COMPILE)
 
 #define IsStage2RestSIMDs(prev_ctx_ptr) (prev_ctx_ptr != nullptr)
@@ -186,7 +191,7 @@ typedef enum
     FCEXP_TOBE_DESIGNED                 = ( 0x1 << 0xc ),
 
     // Current default stage 1 options. *Must* be updated whenever the default changes.
-    FCEXP_DEFAULT                       = ( FCEXP_DISABLE_LVN | FCEXP_LINEARSCAN | FCEXP_DISABLE_GOPT | FCEXP_LOCAL_SCHEDULING | FCEXP_PRERA_SCHEDULING | FCEXP_NO_SPILL_COMPRESSION | FCEXP_QUICKTOKEN_ALLOC ),
+    FCEXP_DEFAULT                       = ( FCEXP_DISABLE_LVN | FCEXP_LINEARSCAN | FCEXP_DISABLE_GOPT | FCEXP_LOCAL_SCHEDULING | FCEXP_PRERA_SCHEDULING | FCEXP_QUICKTOKEN_ALLOC ),
 
     // Alias for UMD to indicate staged compilation must be disabled
     FCEXP_DISABLED                      = FCEXP_TOBE_DESIGNED
