@@ -56,6 +56,14 @@ void DivergentBarrierPass::updateFenceArgs(
         else
             return true;
     };
+    auto updateScope = [](const Value* V)
+    {
+        // If we don't know at compile-time, conservatively assume LSC_SCOPE_GPU
+        if (auto* C = dyn_cast<ConstantInt>(V))
+            return static_cast<uint32_t>(C->getValue().getZExtValue());
+        else
+            return static_cast<uint32_t>(LSC_SCOPE_GPU);
+    };
 
     // Always CommitEnable
 
@@ -66,6 +74,7 @@ void DivergentBarrierPass::updateFenceArgs(
     Args.Global                 |= update(I->getOperand(5));
     Args.L1_Invalidate          |= update(I->getOperand(6));
     Args.L1_Evict               |= update(I->getOperand(7));
+    Args.Scope                   = updateScope(I->getOperand(8));
 }
 
 CallInst* DivergentBarrierPass::insertFence(
@@ -83,6 +92,7 @@ CallInst* DivergentBarrierPass::insertFence(
         IRB.getInt1(FA.Global),
         IRB.getInt1(FA.L1_Invalidate),
         IRB.getInt1(FA.L1_Evict),
+        IRB.getInt32(FA.Scope)
     };
     return IRB.CreateCall(FenceFn, Args);
 }
@@ -390,7 +400,7 @@ void DivergentBarrierPass::generateBody(
         IRB.CreateCall(BarrierFn);
 
         auto* ThreadDoneCnt = IRB.CreateLoad(
-            IGCLLVM::getNonOpaquePtrEltTy(ThreadDoneCntPtr->getType()),
+            IRB.getInt32Ty(),
             ThreadDoneCntPtr, VALUE_NAME("thread.done.cnt"));
 
         Value* GroupSize = getGroupSize(*Wrapper);
@@ -414,7 +424,7 @@ void DivergentBarrierPass::generateBody(
     for (auto* CI : ContCalls)
     {
         InlineFunctionInfo IFI;
-        bool CanInline = IGCLLVM::InlineFunction(CI, IFI, nullptr, false);
+        bool CanInline = IGCLLVM::InlineFunction(*CI, IFI, nullptr, false);
         IGC_ASSERT_MESSAGE(CanInline, "failed to inline?");
     }
 
@@ -470,7 +480,7 @@ void DivergentBarrierPass::handleSpillFill(Function* F, SlotDepMap& depMap)
             }
             Builder.SetInsertPoint(FI);
             Instruction* LI = Builder.CreateLoad(
-                IGCLLVM::getNonOpaquePtrEltTy(Ptr->getType()), Ptr);
+                cast<AllocaInst>(Ptr)->getAllocatedType(), Ptr);
             LI->takeName(FI);
 
             // If the values spilled to this slot are all uniform, then force uniform load on fill via readFirstLane.
