@@ -11,6 +11,7 @@ SPDX-License-Identifier: MIT
 #include "common/LLVMUtils.h"
 #include "Compiler/IGCPassSupport.h"
 #include "common/LLVMWarningsPush.hpp"
+#include <llvm/ADT/BreadthFirstIterator.h>
 #include <llvm/IR/InstIterator.h>
 #include <llvm/Analysis/CFG.h>
 #include <llvm/Analysis/InstructionSimplify.h>
@@ -296,7 +297,7 @@ bool DynamicRayManagementPass::TryProceedBasedApproach(Function& F)
     // this approach assumes all traffic between private memory and RTStack happens on Proceed calls
     // will be removed once RayQuery will be overhauled to minimize shadowstack usage
 
-    if (IGC_IS_FLAG_ENABLED(DisableProceedBasedApproachForRayQueryDynamicRayManagementMechanism))
+    if (!m_CGCtx->platform.allowProceedBasedApproachForRayQueryDynamicRayManagementMechanism())
         return false;
 
     SmallVector<TraceRaySyncProceedHLIntrinsic*> allProceeds;
@@ -442,7 +443,7 @@ bool DynamicRayManagementPass::TryProceedBasedApproach(Function& F)
 
         IRB.SetInsertPoint(IP);
 
-        auto* load = IRB.CreateLoad(guard, VALUE_NAME("RQGuardValue"));
+        auto* load = IRB.CreateLoad(IRB.getInt1Ty(), guard, VALUE_NAME("RQGuardValue"));
 
         guardStoresAndLoads.push_back(load);
 
@@ -470,7 +471,7 @@ bool DynamicRayManagementPass::TryProceedBasedApproach(Function& F)
 
         IRB.SetInsertPoint(IP);
 
-        auto* cond = IRB.CreateLoad(guard, VALUE_NAME("RQGuardValue"));
+        auto* cond = IRB.CreateLoad(IRB.getInt1Ty(), guard, VALUE_NAME("RQGuardValue"));
 
         guardStoresAndLoads.push_back(cond);
 
@@ -662,7 +663,7 @@ bool DynamicRayManagementPass::AddDynamicRayManagement(Function& F)
 
     // Find the Block which dominates all AllocateRayQueryIntrinsics
     // This will be the place to put RayQueryCheck().
-    BasicBlock* dominatorBasicBlock = allocateRayQueries[0]->getParent();
+    BasicBlock* dominatorBasicBlock = commonPostDominatorForRayQueryUsers;
 
     for (AllocateRayQueryIntrinsic* allocateRayQueryIntrinsic : allocateRayQueries)
     {
@@ -873,6 +874,29 @@ bool DynamicRayManagementPass::AddDynamicRayManagement(Function& F)
     // RayQueryCheck-Release pair identification.
     RayQueryReleaseIntrinsic* rayQueryRelease = builder.CreateRayQueryReleaseIntrinsic();
 
+    // There is a possibility that the check is no longer post-dominated by the release now (because release insertion logic changes the domtrees)
+    // If that's the case, iterate over descendants of the check and find a block thats both dominated by the current check and postdominated by the current release
+    if (!m_PDT->dominates(rayQueryRelease->getParent(), rayQueryCheck->getParent()))
+    {
+        for (auto* node : llvm::breadth_first(m_DT->getNode(rayQueryCheck->getParent())))
+        {
+            auto* bb = node->getBlock();
+            if (m_PDT->dominates(rayQueryRelease->getParent(), bb))
+            {
+                for (auto& I : *bb)
+                {
+                    if (isa<AllocateRayQueryIntrinsic>(&I) || &I == bb->getTerminator() || &I == rayQueryRelease)
+                    {
+                        rayQueryCheck->moveBefore(&I);
+                        break;
+                    }
+                }
+
+                break;
+            }
+        }
+    }
+
     // Add created RayQueryCheck-Release to the list, which
     // will be used during complex control flow handling.
     m_RayQueryCheckReleasePairs.push_back(std::make_tuple(rayQueryCheck, rayQueryRelease));
