@@ -185,15 +185,14 @@ RayQueryReleaseIntrinsic* RTBuilder::CreateRayQueryReleaseIntrinsic(Value* predi
     return cast<RayQueryReleaseIntrinsic>(rayQueryRelease);
 }
 
-PreemptionDisableIntrinsic* RTBuilder::CreatePreemptionDisableIntrinsic(Value* Flag)
+PreemptionDisableIntrinsic* RTBuilder::CreatePreemptionDisableIntrinsic()
 {
     Module* M = this->GetInsertBlock()->getModule();
 
     auto* GII = CreateCall(
         GenISAIntrinsic::getDeclaration(
             M,
-            GenISAIntrinsic::GenISA_PreemptionDisable),
-        Flag ? Flag : getTrue());
+            GenISAIntrinsic::GenISA_PreemptionDisable));
 
     return cast<PreemptionDisableIntrinsic>(GII);
 }
@@ -1995,3 +1994,17 @@ void RTBuilder::commitProceduralPrimitiveHit(
 #undef STYLE
     }
 }
+
+Value* RTBuilder::getHitAddress(StackPointerVal* StackPtr, bool Committed)
+{
+    switch (getMemoryStyle())
+    {
+#define STYLE(X)           \
+    case RTMemoryStyle::X: \
+        return _getHitAddress_##X(StackPtr, VAdapt{ *this, Committed });
+#include "RayTracingMemoryStyle.h"
+#undef STYLE
+    }
+    IGC_ASSERT(0);
+    return {};
+}
