@@ -13,6 +13,7 @@ SPDX-License-Identifier: MIT
 #include "common/LLVMWarningsPush.hpp"
 #include "llvmWrapper/Support/Alignment.h"
 #include <llvm/IR/InstIterator.h>
+#include <llvmWrapper/IR/Instructions.h>
 #include "common/LLVMWarningsPop.hpp"
 
 using namespace llvm;
@@ -138,7 +139,7 @@ void TypesLegalizationPass::ResolvePhiNode( PHINode *phi ) {
     IGCLLVM::IRBuilder<> builder( phi );
     BasicBlock *block = builder.GetInsertBlock();
     builder.SetInsertPoint( &(*block->getFirstInsertionPt()) );
-    Value* newLoad = builder.CreateLoad( allocaPtr );
+    Value* newLoad = builder.CreateLoad(allocaPtr->getAllocatedType(), allocaPtr);
     phi->replaceAllUsesWith( newLoad );
     phi->eraseFromParent();
   }
@@ -235,9 +236,10 @@ TypesLegalizationPass::ResolveValue( Instruction *ip,Value *val,SmallVector<unsi
     Value* gep = CreateGEP( builder, ld->getType(), ld->getOperand( 0 ),indices );
     auto alignment = IGCLLVM::getAlignmentValue(ld);
     unsigned pointerTypeSize = ld->getType()->getScalarSizeInBits() / 8;
+    Type* gepTy = IGCLLVM::getGEPIndexedType(ld->getType(), indices);
     if ( alignment && (alignment_t)pointerTypeSize == alignment )
-      return builder.CreateAlignedLoad(ld->getType(), gep, IGCLLVM::getAlign(alignment) );
-    return builder.CreateLoad( gep );
+      return builder.CreateAlignedLoad(gepTy, gep, IGCLLVM::getAlign(alignment));
+    return builder.CreateLoad(gepTy, gep);
   }
   else if(Constant *c = dyn_cast<Constant>(val)) {
     IRBuilder<> builder( ip );
