@@ -192,6 +192,33 @@ namespace IGC
             }
         }
 
+        template<typename dataType, typename RegkeyT>
+        inline void SetCompilerOptionOpaqueHelper(RegkeyT& regkeyName, const dataType* data)
+        {
+            memcpy_s(&regkeyName.m_Value, sizeof(dataType), data, sizeof(dataType));
+        }
+
+        template<typename RegkeyT>
+        inline void SetCompilerOptionOpaqueHelper(RegkeyT& regkeyName, debugString data)
+        {
+            strcpy_s(&regkeyName.m_string, sizeof(debugString), data);
+        }
+
+        void IGC_DEBUG_API_CALL SetCompilerOptionOpaque(OptionFlag flag, void* data)
+        {
+            switch (flag)
+            {
+#define DECLARE_IGC_REGKEY(dataType, regkeyName, defaultValue, description, releaseMode) \
+            case OptionFlag::OPTION_##regkeyName: \
+                SetCompilerOptionOpaqueHelper(g_RegKeyList.regkeyName, reinterpret_cast<dataType*>(data)); \
+            break;
+#include "common/igc_regkeys.h"
+#undef DECLARE_IGC_REGKEY
+            default:
+                break;
+            }
+        }
+
         extern "C" void IGC_DEBUG_API_CALL SetCompilerOptionValue( const char* flagName, int value )
         {
             if (!flagName)
@@ -234,7 +261,7 @@ namespace IGC
             }
         }
 
-        void IGC_DEBUG_API_CALL SetDebugFlag( DebugFlag flag, bool enabled )
+        void IGC_DEBUG_API_CALL SetDebugFlag(DebugFlag flag, bool enabled)
         {
             IGC_ASSERT_EXIT_MESSAGE((0 <= static_cast<int>(flag)), "range sanity check");
             IGC_ASSERT_EXIT_MESSAGE((static_cast<int>(flag) < static_cast<int> (DebugFlag::END)), "range sanity check");
@@ -264,7 +291,7 @@ namespace IGC
             return g_debugFlags[ static_cast<int>(flag) ];
         }
 
-        void IGC_DEBUG_API_CALL SetDumpFlag( DumpType type, DumpLoc loc, bool enabled )
+        void IGC_DEBUG_API_CALL SetDumpFlag(DumpType type, DumpLoc loc, bool enabled)
         {
             IGC_ASSERT_EXIT_MESSAGE((0 <= static_cast<int>(type)), "range sanity check");
             IGC_ASSERT_EXIT_MESSAGE((static_cast<int>(type) < static_cast<int> (DumpType::END)), "range sanity check");
@@ -297,7 +324,7 @@ namespace IGC
             }
         }
 
-        void IGC_DEBUG_API_CALL SetShaderCorpusName( CorpusName name )
+        void IGC_DEBUG_API_CALL SetShaderCorpusName(CorpusName name)
         {
             g_shaderCorpusName = name;
         }
@@ -307,7 +334,7 @@ namespace IGC
             return g_shaderCorpusName.c_str();
         }
 
-        void IGC_DEBUG_API_CALL SetShaderOutputFolder( OutputFolderName name )
+        void IGC_DEBUG_API_CALL SetShaderOutputFolder(OutputFolderName name)
         {
             g_shaderOutputFolder = name;
         }
@@ -316,6 +343,21 @@ namespace IGC
             g_shaderOutputName = name;
         }
 
+        bool needMkDir()
+        {
+            return IGC_IS_FLAG_ENABLED(DumpLLVMIR)              ||
+                   IGC_IS_FLAG_ENABLED(EnableCosDump)           ||
+                   IGC_IS_FLAG_ENABLED(EnableVISAOutput)        ||
+                   IGC_IS_FLAG_ENABLED(EnableVISABinary)        ||
+                   IGC_IS_FLAG_ENABLED(EnableVISADumpCommonISA) ||
+                   GetDebugFlag(DebugFlag::DUMP_AFTER_PASSES)   ||
+                   GetDebugFlag(DebugFlag::VISA_OUTPUT)         ||
+                   GetDebugFlag(DebugFlag::VISA_BINARY)         ||
+                   GetDebugFlag(DebugFlag::VISA_DUMPCOMMONISA)  ||
+                   IGC_IS_FLAG_ENABLED(EnableCapsDump)          ||
+                   IGC_IS_FLAG_ENABLED(ShaderOverride);
+        }
+
         OutputFolderName IGC_DEBUG_API_CALL GetBaseIGCOutputFolder()
         {
             static std::mutex m;
@@ -328,18 +370,7 @@ namespace IGC
 #   if defined(_WIN64) || defined(_WIN32)
             if (!IGC_IS_FLAG_ENABLED(DumpToCurrentDir) && !IGC_IS_FLAG_ENABLED(DumpToCustomDir))
             {
-                bool needMkdir =
-                    IGC_IS_FLAG_ENABLED(DumpLLVMIR) ||
-                    IGC_IS_FLAG_ENABLED(EnableCosDump) ||
-                    IGC_IS_FLAG_ENABLED(EnableVISAOutput) ||
-                    IGC_IS_FLAG_ENABLED(EnableVISABinary) ||
-                    IGC_IS_FLAG_ENABLED(EnableVISADumpCommonISA) ||
-                    GetDebugFlag(DebugFlag::DUMP_AFTER_PASSES) ||
-                    GetDebugFlag(DebugFlag::VISA_OUTPUT) ||
-                    GetDebugFlag(DebugFlag::VISA_BINARY) ||
-                    GetDebugFlag(DebugFlag::VISA_DUMPCOMMONISA) ||
-                    IGC_IS_FLAG_ENABLED(EnableCapsDump) ||
-                    IGC_IS_FLAG_ENABLED(ShaderOverride);
+                bool needMkdir = needMkDir();
 
                 char dumpPath[256];
 
@@ -492,7 +523,7 @@ namespace IGC
                     sprintf_s(dumpPath, "%sunknownProcess_%d\\", dumpPath, _getpid());
                 }
 
-                if (GetFileAttributesA(dumpPath) != FILE_ATTRIBUTE_DIRECTORY)
+                if (GetFileAttributesA(dumpPath) != FILE_ATTRIBUTE_DIRECTORY && needMkDir())
                 {
                     _mkdir(dumpPath);
                 }
@@ -508,11 +539,8 @@ namespace IGC
 #elif defined ANDROID
 
             if (IGC_IS_FLAG_ENABLED(DumpToCurrentDir))
-
                 return "";
 
-
-
             if (!SysUtils::CreateDir(GetBaseIGCOutputFolder(), true, IGC_IS_FLAG_DISABLED(ShaderDumpPidDisable), &g_shaderOutputFolder))
 
                 g_shaderOutputFolder = "";
@@ -520,21 +548,7 @@ namespace IGC
 #elif defined __linux__
             if (!IGC_IS_FLAG_ENABLED(DumpToCurrentDir) && g_shaderOutputFolder == "" && !IGC_IS_FLAG_ENABLED(DumpToCustomDir))
             {
-                bool needMkdir = false;
-
-                if (IGC_IS_FLAG_ENABLED(DumpLLVMIR) ||
-                    IGC_IS_FLAG_ENABLED(EnableCosDump) ||
-                    IGC_IS_FLAG_ENABLED(EnableVISAOutput) ||
-                    IGC_IS_FLAG_ENABLED(EnableVISABinary) ||
-                    IGC_IS_FLAG_ENABLED(EnableVISADumpCommonISA) ||
-                    GetDebugFlag(DebugFlag::DUMP_AFTER_PASSES) ||
-                    GetDebugFlag(DebugFlag::VISA_OUTPUT) ||
-                    GetDebugFlag(DebugFlag::VISA_BINARY) ||
-                    GetDebugFlag(DebugFlag::VISA_DUMPCOMMONISA) ||
-                    IGC_IS_FLAG_ENABLED(EnableCapsDump))
-                {
-                    needMkdir = true;
-                }
+                bool needMkdir = needMkDir();
 
                 char path[MAX_PATH] = { 0 };
                 bool pidEnabled = IGC_IS_FLAG_DISABLED(ShaderDumpPidDisable);
@@ -550,7 +564,6 @@ namespace IGC
                         pidEnabled);
                 }
 
-
                 g_shaderOutputFolder = path;
             }
             else if (IGC_IS_FLAG_ENABLED(DumpToCustomDir))
@@ -594,6 +607,8 @@ namespace IGC
         extern "C" void IGC_DEBUG_API_CALL SetCompilerOptionValue(const char* flagName, int value);
         extern "C" void IGC_DEBUG_API_CALL SetCompilerOptionString(const char* flagName, debugString s);
 
+        void IGC_DEBUG_API_CALL SetCompilerOptionOpaque(OptionFlag flag, void* data);
+
 
 #endif // defined( IGC_DEBUG_VARIABLES )
     }
