@@ -10,9 +10,7 @@ SPDX-License-Identifier: MIT
 // Used by BuiltinResource.rc
 //
 #define OCL_BC_START                    120
-#define OCL_BC_32                       120
-#define OCL_BC_64                       121
-#define OCL_BC                          122
+#define OCL_BIFBC                       122
 #define OCL_BC_RS                       123
 #define OCL_BC_END                      124
 
