@@ -80,6 +80,9 @@ public:
   SPIRVWord WordCount;
   Op OpCode;
   SPIRVEntry *Scope; // A function or basic block
+
+  std::vector<SPIRVEntry*>
+      getContinuedInstructions(const Op ContinuedOpCode);
 };
 
 template<typename T>
@@ -117,6 +120,14 @@ operator>>(const SPIRVDecoder& I, std::vector<T> &V) {
   return I;
 }
 
+template <typename T>
+const SPIRVDecoder&
+operator>>(const SPIRVDecoder& I, llvm::Optional<T>& V) {
+  if (V)
+    I >> V.getValue();
+  return I;
+}
+
 #define SPIRV_DEC_DEC(Type) \
     const SPIRVDecoder& operator>>(const SPIRVDecoder& I, Type &V);
 
