@@ -224,16 +224,17 @@ private:
 
 class SPIRVTypeForwardPointer : public SPIRVEntryNoId<OpTypeForwardPointer> {
 public:
-  SPIRVTypeForwardPointer(SPIRVModule* M, SPIRVTypePointer* Pointer,
+  SPIRVTypeForwardPointer(SPIRVModule* M, SPIRVId PointerId,
       SPIRVStorageClassKind SC)
-      : SPIRVEntryNoId(M, 3), Pointer(Pointer), SC(SC) {}
+      : SPIRVEntryNoId(M, 3), PointerId(PointerId), SC(SC) {}
 
   SPIRVTypeForwardPointer()
-      : Pointer(nullptr), SC(StorageClassUniformConstant) {}
+      : PointerId(SPIRVID_INVALID), SC(StorageClassUniformConstant) {}
 
+  SPIRVId getPointerId() const { return PointerId; }
   _SPIRV_DCL_DEC
 private:
-  SPIRVTypePointer* Pointer;
+  SPIRVId PointerId;
   SPIRVStorageClassKind SC;
 };
 
@@ -265,8 +266,8 @@ protected:
     SPIRVEntry::validate();
     CompType->validate();
 #if 1
-    // Allow any vector from 1 to 16 for testing.
-    IGC_ASSERT(CompCount >= 1 && CompCount <= 16);
+    // Allow vector size to be 64 as Block2D load can have short64(d8_m32k32v2)
+    IGC_ASSERT(CompCount >= 1 && CompCount <= 64);
 #else
     IGC_ASSERT(CompCount == 2 || CompCount == 3 || CompCount == 4 ||
         CompCount == 8 || CompCount == 16);
@@ -291,7 +292,7 @@ public:
 
 protected:
   _SPIRV_DCL_DEC
-  void validate()const;
+  void validate() const override;
 private:
   SPIRVType *ElemType;                // Element Type
   SPIRVId Length;                     // Array Length
@@ -524,11 +525,11 @@ public:
   }
 
 protected:
-  void setWordCount(SPIRVWord WordCount) { MemberTypeVec.resize(WordCount - FixedWC);}
+  void setWordCount(SPIRVWord WordCount) override { MemberTypeVec.resize(WordCount - FixedWC); }
 
   _SPIRV_DCL_DEC
 
-  void validate()const {
+  void validate() const override {
     SPIRVEntry::validate();
   }
 private:
