@@ -20,6 +20,7 @@ SPDX-License-Identifier: MIT
 #include "ocl_igc_interface/igc_builtins.h"
 
 #include "cif/macros/enable.h"
+#include "OCLAPI/oclapi.h"
 
 // Interface : IGC_OCL_DEVC
 //             "IGC - OCL" Device Context
@@ -33,32 +34,32 @@ CIF_DEFINE_INTERFACE_VER(IgcOclDeviceCtx, 1){
   CIF_INHERIT_CONSTRUCTOR();
 
   template <typename PlatformInterface = PlatformTagOCL>
-  CIF::RAII::UPtr_t<PlatformInterface> GetPlatformHandle() {
+  OCL_API_CALL CIF::RAII::UPtr_t<PlatformInterface> GetPlatformHandle() {
     return CIF::RAII::RetainAndPack<PlatformInterface>( GetPlatformHandleImpl(PlatformInterface::GetVersion()) );
   }
 
   template <typename GTSystemInfoInterface = GTSystemInfoTagOCL>
-  CIF::RAII::UPtr_t<GTSystemInfoInterface> GetGTSystemInfoHandle() {
+  OCL_API_CALL CIF::RAII::UPtr_t<GTSystemInfoInterface> GetGTSystemInfoHandle() {
     return CIF::RAII::RetainAndPack<GTSystemInfoInterface>( GetGTSystemInfoHandleImpl(GTSystemInfoInterface::GetVersion()) );
   }
 
   template <typename IgcFeaturesAndWorkaroundsInterface = IgcFeaturesAndWorkaroundsTagOCL>
-  CIF::RAII::UPtr_t<IgcFeaturesAndWorkaroundsInterface> GetIgcFeaturesAndWorkaroundsHandle() {
+  OCL_API_CALL CIF::RAII::UPtr_t<IgcFeaturesAndWorkaroundsInterface> GetIgcFeaturesAndWorkaroundsHandle() {
     return CIF::RAII::RetainAndPack<IgcFeaturesAndWorkaroundsInterface>(  GetIgcFeaturesAndWorkaroundsHandleImpl(IgcFeaturesAndWorkaroundsInterface::GetVersion()) );
   }
 
   virtual void SetProfilingTimerResolution(float v);
 
   template <typename OclIgcTranslationInterface = IgcOclTranslationCtxTagOCL>
-  CIF::RAII::UPtr_t<OclIgcTranslationInterface> CreateTranslationCtx(CodeType::CodeType_t inType, CodeType::CodeType_t outType) {
+  OCL_API_CALL CIF::RAII::UPtr_t<OclIgcTranslationInterface> CreateTranslationCtx(CodeType::CodeType_t inType, CodeType::CodeType_t outType) {
     return CIF::RAII::Pack<OclIgcTranslationInterface>( CreateTranslationCtxImpl(OclIgcTranslationInterface::GetVersion(), inType, outType) );
   }
 
 protected:
-  virtual PlatformBase *GetPlatformHandleImpl(CIF::Version_t ver);
-  virtual GTSystemInfoBase *GetGTSystemInfoHandleImpl(CIF::Version_t ver);
-  virtual IgcFeaturesAndWorkaroundsBase *GetIgcFeaturesAndWorkaroundsHandleImpl(CIF::Version_t ver);
-  virtual IgcOclTranslationCtxBase *CreateTranslationCtxImpl(CIF::Version_t ver,
+  OCL_API_CALL virtual PlatformBase *GetPlatformHandleImpl(CIF::Version_t ver);
+  OCL_API_CALL virtual GTSystemInfoBase *GetGTSystemInfoHandleImpl(CIF::Version_t ver);
+  OCL_API_CALL virtual IgcFeaturesAndWorkaroundsBase *GetIgcFeaturesAndWorkaroundsHandleImpl(CIF::Version_t ver);
+  OCL_API_CALL virtual IgcOclTranslationCtxBase *CreateTranslationCtxImpl(CIF::Version_t ver,
                                                              CodeType::CodeType_t inType,
                                                              CodeType::CodeType_t outType);
 };
@@ -76,7 +77,7 @@ namespace SystemRoutineType {
 CIF_DEFINE_INTERFACE_VER_WITH_COMPATIBILITY(IgcOclDeviceCtx, 2, 1) {
     CIF_INHERIT_CONSTRUCTOR();
 
-    virtual bool GetSystemRoutine(SystemRoutineType::SystemRoutineType_t typeOfSystemRoutine,
+    OCL_API_CALL virtual bool GetSystemRoutine(SystemRoutineType::SystemRoutineType_t typeOfSystemRoutine,
                                     bool bindless,
                                     CIF::Builtins::BufferSimple *outSystemRoutineBuffer,
                                     CIF::Builtins::BufferSimple *stateSaveAreaHeaderInit);
@@ -85,19 +86,19 @@ CIF_DEFINE_INTERFACE_VER_WITH_COMPATIBILITY(IgcOclDeviceCtx, 2, 1) {
 CIF_DEFINE_INTERFACE_VER_WITH_COMPATIBILITY(IgcOclDeviceCtx, 3, 2) {
   CIF_INHERIT_CONSTRUCTOR();
 
-  virtual const char* GetIGCRevision();
+  OCL_API_CALL virtual const char* GetIGCRevision();
 };
 
 CIF_DEFINE_INTERFACE_VER_WITH_COMPATIBILITY(IgcOclDeviceCtx, 4, 3) {
     CIF_INHERIT_CONSTRUCTOR();
 
     template <typename IgcBuiltinsInterface = IgcBuiltinsLatest>
-    CIF::RAII::UPtr_t<IgcBuiltinsInterface> GetIgcBuiltinsHandle() {
+    OCL_API_CALL CIF::RAII::UPtr_t<IgcBuiltinsInterface> GetIgcBuiltinsHandle() {
         return CIF::RAII::RetainAndPack<IgcBuiltinsInterface>( GetIgcBuiltinsHandleImpl(IgcBuiltinsInterface::GetVersion()) );
     }
 
 protected:
-    virtual IgcBuiltinsBase *GetIgcBuiltinsHandleImpl(CIF::Version_t ver);
+    OCL_API_CALL virtual IgcBuiltinsBase *GetIgcBuiltinsHandleImpl(CIF::Version_t ver);
 };
 
 CIF_GENERATE_VERSIONS_LIST_AND_DECLARE_INTERFACE_DEPENDENCIES(IgcOclDeviceCtx, IGC::Platform, IGC::GTSystemInfo,
