@@ -16,11 +16,12 @@ SPDX-License-Identifier: MIT
 #include "gtsysinfo.h"
 
 #include "cif/macros/enable.h"
+#include "OCLAPI/oclapi.h"
 
 namespace IGC {
 
 CIF_DECLARE_INTERFACE_PIMPL(GTSystemInfo) : CIF::PimplBase {
-  CIF_PIMPL_DECLARE_CONSTRUCTOR() {
+  OCL_API_CALL CIF_PIMPL_DECLARE_CONSTRUCTOR() {
       CIF::SafeZeroOut(gsi);
   }
 
