@@ -15,6 +15,7 @@ SPDX-License-Identifier: MIT
 #include "cif/export/pimpl_base.h"
 
 #include "cif/macros/enable.h"
+#include "OCLAPI/oclapi.h"
 
 namespace IGC{
 
@@ -34,7 +35,7 @@ struct TranslationErrorType
 
 CIF_DECLARE_INTERFACE_PIMPL(OclTranslationOutput) : CIF::PimplBase
 {
-    CIF_PIMPL_DECLARE_CONSTRUCTOR(CodeType::CodeType_t OutputType)
+    OCL_API_CALL CIF_PIMPL_DECLARE_CONSTRUCTOR(CodeType::CodeType_t OutputType)
         : OutputType(OutputType), Error(TranslationErrorType::Unused)
     {
         BuildLog.CreateImpl();
@@ -42,37 +43,37 @@ CIF_DECLARE_INTERFACE_PIMPL(OclTranslationOutput) : CIF::PimplBase
         DebugData.CreateImpl();
     }
 
-    bool Successful() const
+    OCL_API_CALL bool Successful() const
     {
         return Error == TranslationErrorType::Success;
     }
 
-    bool HasWarnings() const
+    OCL_API_CALL bool HasWarnings() const
     {
         return false;
     }
 
-    CIF::Builtins::BufferBase * GetBuildLog(CIF::Version_t bufferVersion)
+    OCL_API_CALL CIF::Builtins::BufferBase * GetBuildLog(CIF::Version_t bufferVersion)
     {
         return BuildLog.GetVersion(bufferVersion);
     }
 
-    CIF::Builtins::BufferBase * GetOutput(CIF::Version_t bufferVersion)
+    OCL_API_CALL CIF::Builtins::BufferBase * GetOutput(CIF::Version_t bufferVersion)
     {
         return Output.GetVersion(bufferVersion);
     }
 
-    CIF::Builtins::BufferBase * GetDebugData(CIF::Version_t bufferVersion)
+    OCL_API_CALL CIF::Builtins::BufferBase * GetDebugData(CIF::Version_t bufferVersion)
     {
         return DebugData.GetVersion(bufferVersion);
     }
 
-    CodeType::CodeType_t GetOutputType() const
+    OCL_API_CALL CodeType::CodeType_t GetOutputType() const
     {
         return OutputType;
     }
 
-    bool SetError(TranslationErrorType::ErrorCode_t e, const char * errString = nullptr)
+    OCL_API_CALL bool SetError(TranslationErrorType::ErrorCode_t e, const char * errString = nullptr)
     {
         this->Error = e;
 
@@ -84,12 +85,12 @@ CIF_DECLARE_INTERFACE_PIMPL(OclTranslationOutput) : CIF::PimplBase
         return BuildLog->PushBackRawBytes(errString, len + 1);
     }
 
-    bool AddWarning(const std::string & warnString)
+    OCL_API_CALL bool AddWarning(const std::string & warnString)
     {
         return AddWarning(warnString.c_str(), warnString.size());
     }
 
-    bool AddWarning(const char * warn, size_t warnLength)
+    OCL_API_CALL bool AddWarning(const char * warn, size_t warnLength)
     {
         if((warn == nullptr) || (warnLength == 0)){
             return true;
@@ -117,13 +118,13 @@ CIF_DECLARE_INTERFACE_PIMPL(OclTranslationOutput) : CIF::PimplBase
     }
 
     template<typename T>
-    bool SetSuccessfulAndCloneOutput(const T * data, size_t size)
+    OCL_API_CALL bool SetSuccessfulAndCloneOutput(const T * data, size_t size)
     {
         this->Error = TranslationErrorType::Success;
         return Output->PushBackRawBytes(data, size);
     }
 
-    bool CloneDebugData(const char * data, size_t size)
+    OCL_API_CALL bool CloneDebugData(const char * data, size_t size)
     {
         return DebugData->PushBackRawBytes(data, size);
     }
