@@ -16,10 +16,12 @@ SPDX-License-Identifier: MIT
 
 #include "cif/macros/enable.h"
 
+#include "OCLAPI/oclapi.h"
+
 namespace IGC {
 
 CIF_DECLARE_INTERFACE_PIMPL(Platform) : CIF::PimplBase {
-  CIF_PIMPL_DECLARE_CONSTRUCTOR()
+  OCL_API_CALL CIF_PIMPL_DECLARE_CONSTRUCTOR()
   {
       memset(&p, 0, sizeof(p));
   }
