@@ -12,6 +12,7 @@ SPDX-License-Identifier: MIT
 #include "cif/common/id.h"
 
 #include "ocl_igc_interface/platform.h"
+#include "OCLAPI/oclapi.h"
 
 #define COPY_VAL(INTEFACE_VAL_NAME, SRC_VAL_NAME)                              \
   dst.Set##INTEFACE_VAL_NAME(src.SRC_VAL_NAME)
@@ -21,7 +22,7 @@ SPDX-License-Identifier: MIT
 namespace IGC {
 namespace PlatformHelper {
 template <CIF::Version_t Ver, typename SrcStructT>
-inline void PopulateInterfaceWith(IGC::Platform<Ver> &dst,
+OCL_API_CALL inline void PopulateInterfaceWith(IGC::Platform<Ver> &dst,
                                    const SrcStructT &src) {
   COPY_VAL_E(ProductFamily);
   COPY_VAL_E(PCHProductFamily);
@@ -36,7 +37,7 @@ inline void PopulateInterfaceWith(IGC::Platform<Ver> &dst,
 }
 
 template <typename SrcStructT>
-inline void PopulateInterfaceWith(IGC::Platform<2>& dst,
+OCL_API_CALL inline void PopulateInterfaceWith(IGC::Platform<2>& dst,
                                   const SrcStructT& src) {
   PopulateInterfaceWith<1>(dst, src);
   // Below COPY_VALs are not valid. NEO will populate *BlockID values
@@ -49,7 +50,7 @@ inline void PopulateInterfaceWith(IGC::Platform<2>& dst,
 
 namespace GtSysInfoHelper {
 template <CIF::Version_t Ver, typename SrcStructT>
-inline void PopulateInterfaceWith(IGC::GTSystemInfo<Ver> &dst,
+OCL_API_CALL inline void PopulateInterfaceWith(IGC::GTSystemInfo<Ver> &dst,
                                   const SrcStructT &src) {
   COPY_VAL_EXACT(EUCount);
   COPY_VAL_EXACT(ThreadCount);
@@ -81,18 +82,25 @@ inline void PopulateInterfaceWith(IGC::GTSystemInfo<Ver> &dst,
 }
 
 template <typename SrcStructT>
-inline void PopulateInterfaceWith(IGC::GTSystemInfo<3>& dst,
+OCL_API_CALL inline void PopulateInterfaceWith(IGC::GTSystemInfo<3>& dst,
                                   const SrcStructT& src) {
   PopulateInterfaceWith<1>(dst, src);
   COPY_VAL_EXACT(MaxDualSubSlicesSupported);
   COPY_VAL_EXACT(DualSubSliceCount);
 }
 
+template <typename SrcStructT>
+OCL_API_CALL inline void PopulateInterfaceWith(IGC::GTSystemInfo<4>& dst,
+                                  const SrcStructT& src) {
+  PopulateInterfaceWith<SrcStructT>(static_cast<IGC::GTSystemInfo<3> &>(dst), src);
+  COPY_VAL_EXACT(SLMSizeInKb);
+}
+
 }
 
 namespace IgcPlatformFeaturesHelper {
 template <CIF::Version_t Ver, typename SrcStructT>
-inline void PopulateInterfaceWith(IGC::IgcFeaturesAndWorkarounds<Ver> &dst,
+OCL_API_CALL inline void PopulateInterfaceWith(IGC::IgcFeaturesAndWorkarounds<Ver> &dst,
                                   const SrcStructT &src) {
   COPY_VAL_EXACT(FtrDesktop);
   COPY_VAL_EXACT(FtrChannelSwizzlingXOREnabled);
