@@ -46,13 +46,21 @@ namespace IGC
 
         void visitLoadInst(llvm::LoadInst& I);
         void visitStoreInst(llvm::StoreInst& I);
+        void visitCallInst(llvm::CallInst& I);
+        void visit2DBlockReadCallInst(llvm::CallInst& I, llvm::StringRef unmangledName);
+        void visit2DBlockWriteCallInst(llvm::CallInst& I, llvm::StringRef unmangledName);
+        void visitPrefetchCallInst(llvm::CallInst& I);
+        void visit1DBlockReadCallInst(llvm::CallInst& I);
+        void visit1DBlockWriteCallInst(llvm::CallInst& I);
+        void visit1DBlockPrefetchCallInst(llvm::CallInst& I);
+        void visitOCL1DBlockPrefetchCallInst(llvm::CallInst& I, llvm::SmallVectorImpl<llvm::StringRef>& Matches);
 
     private:
         llvm::Module* m_Module = nullptr;
         CodeGenContext* m_pCtx = nullptr;
         ModuleMetaData* m_Metadata = nullptr;
-        bool m_updateModuleMetadata = false;
         bool m_changed = false;
+        llvm::DenseSet<llvm::Function*> m_BuiltinsToRemove;
 
         void handleInstructionsDecorations();
         void handleGlobalVariablesDecorations();
@@ -60,6 +68,11 @@ namespace IGC
         void handleHostAccessIntel(llvm::GlobalVariable& globalVariable, llvm::MDNode* node);
         template<typename T>
         void handleCacheControlINTEL(llvm::Instruction& I, llvm::SmallPtrSetImpl<llvm::MDNode*>& MDNodes);
-        llvm::DenseMap<uint64_t, llvm::SmallPtrSet<llvm::MDNode*, 4>> parseSPIRVDecorationsFromMD(llvm::Value* V);
+        template<typename T>
+        void handleCacheControlINTELFor2DBlockIO(llvm::CallInst& I, llvm::SmallPtrSetImpl<llvm::MDNode*>& MDNodes, llvm::StringRef unmangledName);
+        void handleCacheControlINTELForPrefetch(llvm::CallInst& I, llvm::SmallPtrSetImpl<llvm::MDNode*>& MDNodes);
+        template<typename T>
+        void handleCacheControlINTELFor1DBlockIO(llvm::CallInst& I, llvm::SmallPtrSetImpl<llvm::MDNode*>& MDNodes);
+        void handleCacheControlINTELForOCL1DBlockPrefetch(llvm::CallInst& I, llvm::SmallPtrSetImpl<llvm::MDNode*>& MDNodes, llvm::SmallVectorImpl<llvm::StringRef>& Matches);
     };
 }
