@@ -138,6 +138,7 @@ namespace IGC
         void MarkAsPayloadLiveOut(CVariable* var);
         void Compile(bool hasSymbolTable, GenXFunctionGroupAnalysis*& pFGA);
         std::string GetShaderName();
+        const vISA::KernelCostInfo* kci;
 
         CEncoder();
         void SetProgram(CShader* program);
@@ -332,7 +333,7 @@ namespace IGC
             AtomicOp subOp, ResourceDescriptor* resource,
             CVariable* pU, CVariable* pV, CVariable* pR,
             CVariable* pSrc0, CVariable* pSrc1, CVariable* pSrcDst,
-            unsigned elemSize, LSC_ADDR_SIZE addr_size);
+            unsigned elemSize, LSC_ADDR_SIZE addr_size, LSC_CACHE_OPTS cacheOpts);
 
 
         void LSC_Typed2dBlock(
@@ -360,6 +361,7 @@ namespace IGC
         void Scatter4A64(CVariable* src, CVariable* offset);
         void BoolToInt(CVariable* dst, CVariable* src);
         void Copy(CVariable* dst, CVariable* src);
+        void CopyWithImplicitConversion(CVariable* dst, CVariable* src);
         void SubroutineCall(CVariable* flag, llvm::Function* F);
         void SubroutineRet(CVariable* flag, llvm::Function* F);
         void StackCall(CVariable* flag, llvm::Function* F, unsigned char argSize, unsigned char retSize);
@@ -782,6 +784,8 @@ namespace IGC
                                        GenXFunctionGroupAnalysis *fga,
                                        uint32_t gtpinScratchUse);
 
+    const vISA::KernelCostInfo* createKernelCostInfo(VISAKernel &pMainKenrel);
+
     protected:
         // encoder states
         SEncoderState m_encoderState = {};
