@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -43,6 +43,7 @@ namespace IGC {
         CName() : CName(nullptr) { }
 
         CName(const CName &) = default;
+        CName& operator=(const CName&) = default;
 
         CName(const std::string &arg)
 #ifdef IGC_MAP_LLVM_NAMES_TO_VISA
@@ -204,6 +205,9 @@ namespace IGC {
         {
         }
 
+        ~CVariable() = default;
+        CVariable& operator=(const CVariable&) = delete;
+
         e_alignment GetAlign() const
         {
             IGC_ASSERT_MESSAGE(!m_isImmediate, "Calling GetAlign() on an immediate returns undefined result");
