@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -9,6 +9,7 @@ SPDX-License-Identifier: MIT
 #pragma once
 
 #include "Compiler/CISACodeGen/WIAnalysis.hpp"
+#include "Compiler/CISACodeGen/ResourceLoopAnalysis.h"
 #include "Compiler/CISACodeGen/PatternMatchPass.hpp"
 #include "Compiler/MetaDataUtilsWrapper.h"
 #include "Compiler/CISACodeGen/PayloadMapping.hpp"
@@ -34,7 +35,8 @@ namespace IGC {
     class CoalescingEngine : public llvm::FunctionPass, public llvm::InstVisitor<CoalescingEngine>
     {
         //TODO: this is fixed for now, but once we have pressure heuristic, could be relaxed
-        static const int MaxTupleSize = 12;
+        static const int MaxTouchedTuples = 8;
+        static const int MaxTupleSize = MaxTouchedTuples + 4;
 
     public:
         static char ID; // Pass identification, replacement for typeid
@@ -146,10 +148,6 @@ namespace IGC {
                 parent.setPointer(this);
             }
 
-            ~ElementNode()
-            {
-            }
-
             ElementNode* getLeader();
 
             // Fields:
@@ -327,7 +325,7 @@ namespace IGC {
             else {
                 uint numUsers = 0;
                 {
-                    llvm::SmallPtrSet<llvm::User*, 8> touchedUsers;
+                    llvm::SmallPtrSet<llvm::User*, MaxTouchedTuples> touchedUsers;
                     for (llvm::Value::user_iterator i = val->user_begin(), e = val->user_end(); i != e; ++i) {
                         llvm::User* user = *i;
                         if (llvm::isa<llvm::Instruction>(user)) {
@@ -388,8 +386,8 @@ namespace IGC {
         void PrepareTuple(
             const uint numOperands,
             llvm::Instruction* tupleGeneratingInstruction,
-            llvm::SmallPtrSet<CCTuple*, 8> & touchedTuplesSet,
-            llvm::SmallVector<CCTuple*, 8> & touchedTuples,
+            llvm::SmallPtrSet<CCTuple*, MaxTouchedTuples> & touchedTuplesSet,
+            llvm::SmallVector<CCTuple*, MaxTouchedTuples> & touchedTuples,
             bool& isAnyNodeAnchored,
             bool& isAnyNodeCoalescable);
 
@@ -712,7 +710,7 @@ namespace IGC {
             CCTuple* m_ccTuple;
             CoalescingEngine* m_CE;
             int m_index;
-            llvm::SmallPtrSet<llvm::Value*, 8> m_valuesForIsolation;
+            llvm::SmallPtrSet<llvm::Value*, MaxTouchedTuples> m_valuesForIsolation;
 
         public:
             ProcessInterferencesElementFunctor(
@@ -732,7 +730,7 @@ namespace IGC {
 
             }
 
-            llvm::SmallPtrSet<llvm::Value*, 8> & GetComputedValuesForIsolation()
+            llvm::SmallPtrSet<llvm::Value*, MaxTouchedTuples> & GetComputedValuesForIsolation()
             {
                 return m_valuesForIsolation;
             }
@@ -961,6 +959,7 @@ namespace IGC {
         LiveVars* LV;
         WIAnalysis* WIA;
         DeSSA* m_DeSSA;
+        ResourceLoopAnalysis* RLA;
         CodeGenPatternMatch* CG;
         llvm::DenseMap<llvm::Value*, ElementNode*> ValueNodeMap;
         llvm::DenseMap<ElementNode*, CCTuple*> NodeCCTupleMap;
