@@ -148,7 +148,7 @@ namespace IGC {
         struct Candidate {
             typedef llvm::SmallVector<llvm::Instruction*, 16> InstrVec;
 
-            Candidate(InstrVec Instructions, BasicBlock* TgtBB, LoopSinkWorthiness Worthiness, llvm::Instruction* UndoPos)
+            Candidate(const InstrVec& Instructions, BasicBlock* TgtBB, LoopSinkWorthiness Worthiness, llvm::Instruction* UndoPos)
                 : Instructions(Instructions), TgtBB(TgtBB), Worthiness(Worthiness), UndoPos(UndoPos) {}
 
             Candidate(llvm::Instruction* Instruction, BasicBlock* TgtBB, LoopSinkWorthiness Worthiness, llvm::Instruction* UndoPos)
@@ -157,6 +157,8 @@ namespace IGC {
             Candidate(const Candidate& Other)
                 : Instructions(Other.Instructions), TgtBB(Other.TgtBB), Worthiness(Other.Worthiness), UndoPos(Other.UndoPos) {}
 
+            Candidate& operator=(const Candidate&) = delete;
+
             InstrVec Instructions;
 
             BasicBlock *TgtBB;
