@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2022 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -11,6 +11,7 @@ SPDX-License-Identifier: MIT
 #include "common/LLVMUtils.h"
 #include "Compiler/CISACodeGen/ConstantCoalescing.hpp"
 #include "Compiler/CISACodeGen/CISACodeGen.h"
+#include "Compiler/CISACodeGen/EmitVISAPass.hpp"
 #include "Compiler/CodeGenContextWrapper.hpp"
 #include "Compiler/IGCPassSupport.h"
 #include "common/IGCIRBuilder.h"
@@ -105,20 +106,6 @@ void ConstantCoalescing::ProcessFunction(Function* function)
     std::vector<BufChunk*> indcb_owloads;
     std::vector<BufChunk*> indcb_gathers;
 
-    for (df_iterator<DomTreeNode*> dom_it = df_begin(dom_tree.getRootNode()),
-        dom_end = df_end(dom_tree.getRootNode()); dom_it != dom_end; ++dom_it)
-    {
-        BasicBlock* cur_blk = dom_it->getBlock();
-        FindAllDirectCB(cur_blk, dircb_owloads);
-
-        while (!dircb_owloads.empty())
-        {
-            BufChunk* top_chunk = dircb_owloads.back();
-            dircb_owloads.pop_back();
-            delete top_chunk;
-        }
-    }
-
     for (df_iterator<DomTreeNode*> dom_it = df_begin(dom_tree.getRootNode()),
         dom_end = df_end(dom_tree.getRootNode()); dom_it != dom_end; ++dom_it)
     {
@@ -400,22 +387,6 @@ void ConstantCoalescing::ProcessBlock(
     std::vector<BufChunk*> & indcb_owloads,
     std::vector<BufChunk*> & indcb_gathers)
 {
-    // checking for stores for future load merging check
-    bool bbHasStores = false;
-    for (BasicBlock::iterator BBI_st = blk->begin(), BBE_st = blk->end();
-        BBI_st != BBE_st; ++BBI_st)
-    {
-        if (isa<StoreInst>(BBI_st) ||
-            isa<StoreRawIntrinsic>(BBI_st))
-        {
-            bbHasStores = true;
-            break;
-        }
-    }
-
-    bool isCPS = false;
-
-    bool skipLdrawOpt = isCPS || bbHasStores;
     // get work-item analysis, need to update uniformness information
     for (BasicBlock::iterator BBI = blk->begin(), BBE = blk->end();
          BBI != BBE; ++BBI)
@@ -432,14 +403,6 @@ void ConstantCoalescing::ProcessBlock(
             BufferType bufType = DecodeAS4GFXResource(
                 ldRaw->getResourceValue()->getType()->getPointerAddressSpace(), directIdx, bufId);
 
-            if ((bufType != BINDLESS_CONSTANT_BUFFER)
-                && (bufType != BINDLESS_TEXTURE)
-                && (bufType != SSH_BINDLESS_CONSTANT_BUFFER)
-                && (bufType != BINDLESS)
-                )
-            {
-                continue;
-            }
 
             uint offsetInBytes = 0;
             Value* baseOffsetInBytes = nullptr;
@@ -462,25 +425,19 @@ void ConstantCoalescing::ProcessBlock(
                 uint addrSpace = ldRaw->getResourceValue()->getType()->getPointerAddressSpace();
                 if (wiAns->isUniform(ldRaw))
                 {
-                    if (!skipLdrawOpt || (bufType != BINDLESS))
-                    {
-                        MergeUniformLoad(
-                            ldRaw,
-                            ldRaw->getResourceValue(),
-                            addrSpace,
-                            baseOffsetInBytes,
-                            offsetInBytes,
-                            maxEltPlus,
-                            Extension,
-                            baseOffsetInBytes ? indcb_owloads : dircb_owloads);
-                    }
+                    MergeUniformLoad(
+                        ldRaw,
+                        ldRaw->getResourceValue(),
+                        addrSpace,
+                        baseOffsetInBytes,
+                        offsetInBytes,
+                        maxEltPlus,
+                        Extension,
+                        baseOffsetInBytes ? indcb_owloads : dircb_owloads);
                 }
-                else if (bufType == BINDLESS_CONSTANT_BUFFER
-                         || bufType == SSH_BINDLESS_CONSTANT_BUFFER
-                        || bufType == BINDLESS
-                         )
+                else if (IsUntypedBuffer(bufType))
                 {
-                    if (UsesTypedConstantBuffer(m_ctx, bufType) && (bufType != BINDLESS))
+                    if (UsesTypedConstantBuffer(m_ctx, bufType))
                     {
                         ScatterToSampler(
                             ldRaw,
@@ -492,7 +449,8 @@ void ConstantCoalescing::ProcessBlock(
                     }
                     else if (IGC_IS_FLAG_DISABLED(DisableConstantCoalescingOfStatefulNonUniformLoads))
                     {
-                        if (bufType != BINDLESS)
+                        // TODO: remove this condition
+                        if (!IsWritableBuffer(bufType))
                         {
                             MergeScatterLoad(
                                 ldRaw,
@@ -506,7 +464,7 @@ void ConstantCoalescing::ProcessBlock(
                         }
                     }
                 }
-                else if (bufType == BINDLESS_TEXTURE && IGC_IS_FLAG_ENABLED(EnableTextureLoadCoalescing))
+                else if (IsTypedBuffer(bufType) && IGC_IS_FLAG_ENABLED(EnableTextureLoadCoalescing))
                 {
                     MergeScatterLoad(
                         ldRaw,
@@ -537,7 +495,8 @@ void ConstantCoalescing::ProcessBlock(
         }
 
         // stateless path: use int2ptr, and support vector-loads
-        if (LI->getPointerAddressSpace() == ADDRESS_SPACE_CONSTANT)
+        uint addrSpace = LI->getPointerAddressSpace();
+        if (addrSpace == ADDRESS_SPACE_CONSTANT)
         {
             // another limit: load has to be dword-aligned
             if (IGCLLVM::getAlignmentValue(LI) % 4)
@@ -559,13 +518,13 @@ void ConstantCoalescing::ProcessBlock(
                     if (wiAns->isUniform(LI))
                     {   // uniform
                         if (elt_idxv)
-                            MergeUniformLoad(LI, buf_idxv, 0, elt_idxv, offsetInBytes, maxEltPlus, Extension, indcb_owloads);
+                            MergeUniformLoad(LI, buf_idxv, addrSpace, elt_idxv, offsetInBytes, maxEltPlus, Extension, indcb_owloads);
                         else
-                            MergeUniformLoad(LI, buf_idxv, 0, nullptr, offsetInBytes, maxEltPlus, Extension, dircb_owloads);
+                            MergeUniformLoad(LI, buf_idxv, addrSpace, nullptr, offsetInBytes, maxEltPlus, Extension, dircb_owloads);
                     }
                     else
                     {   // not uniform
-                        MergeScatterLoad(LI, buf_idxv, 0, elt_idxv, offsetInBytes, maxEltPlus, Extension, indcb_gathers);
+                        MergeScatterLoad(LI, buf_idxv, addrSpace, elt_idxv, offsetInBytes, maxEltPlus, Extension, indcb_gathers);
                     }
                 }
             }
@@ -578,7 +537,6 @@ void ConstantCoalescing::ProcessBlock(
         BufferType bufType = BUFFER_TYPE_UNKNOWN;
         if (IsReadOnlyLoadDirectCB(LI, bufId, elt_ptrv, bufType))
         {
-            uint addrSpace = LI->getPointerAddressSpace();
             uint maxEltPlus = 1;
             if (!isProfitableLoad(LI, maxEltPlus))
                 continue;
@@ -646,144 +604,6 @@ bool sortFunction(BufChunk* buf1, BufChunk* buf2)
         (buf1->addrSpace == buf2->addrSpace && buf1->chunkStart < buf2->chunkStart));
 }
 
-// find all direct CB, check which ones should be merged together, and move the smallest indexed one to the first used location.
-void ConstantCoalescing::FindAllDirectCB(
-    BasicBlock* blk,
-    std::vector<BufChunk*>& dircb_owloads)
-{
-    int loadOrder = 0;
-    // get work-item analysis, need to update uniformness information
-    for (BasicBlock::iterator BBI = blk->begin(), BBE = blk->end();
-        BBI != BBE; ++BBI)
-    {
-        // skip dead instructions
-        if (BBI->use_empty())
-        {
-            continue;
-        }
-        // bindless case
-        if (LdRawIntrinsic * ldRaw = dyn_cast<LdRawIntrinsic>(BBI))
-        {
-            continue;
-        }
-        LoadInst* inst = dyn_cast<LoadInst>(BBI);
-        // skip load on struct or array type
-        if (!inst || inst->getType()->isAggregateType() ||
-            inst->getPointerAddressSpace() == ADDRESS_SPACE_CONSTANT)
-        {
-            continue;
-        }
-
-        const alignment_t alignment = GetAlignment(inst);
-        Type* loadType = inst->getType();
-        Type* elemType = loadType->getScalarType();
-        // right now, only work on load with dword element-type
-        if (elemType->getPrimitiveSizeInBits() != SIZE_DWORD * 8 || loadType->isVectorTy() || alignment == 0)
-        {
-            continue;
-        }
-
-        // skip stateless path
-        uint bufId = 0;
-        Value* elt_ptrv = nullptr;
-        BufferType bufType = BUFFER_TYPE_UNKNOWN;
-        bool is_cbload = IsReadOnlyLoadDirectCB(inst, bufId, elt_ptrv, bufType);
-        if (is_cbload)
-        {
-            uint addrSpace = inst->getPointerAddressSpace();
-            uint maxEltPlus = 1;
-            const uint scalarSizeInBytes = inst->getType()->getScalarSizeInBits() / 8;
-            uint offsetInBytes = 0;
-
-            if (isa<ConstantPointerNull>(elt_ptrv))
-            {
-            }
-            else if (isa<IntToPtrInst>(elt_ptrv))
-            {
-                Value* elt_idxv = cast<Instruction>(elt_ptrv)->getOperand(0);
-                ConstantInt* offsetConstant = dyn_cast<ConstantInt>(elt_idxv);
-                if (offsetConstant)
-                {   // direct access
-                    offsetInBytes = (uint)offsetConstant->getZExtValue();
-                    if ((int32_t)offsetInBytes < 0)
-                    {
-                        continue;
-                    }
-                }
-                else
-                {
-                    continue;
-                }
-            }
-            const uint eltid = offsetInBytes / scalarSizeInBytes;
-
-            BufChunk* cov_chunk = nullptr;
-
-            cov_chunk = new BufChunk();
-            cov_chunk->bufIdxV = nullptr;
-            cov_chunk->addrSpace = addrSpace;
-            cov_chunk->baseIdxV = nullptr;
-            cov_chunk->elementSize = scalarSizeInBytes;
-            cov_chunk->chunkStart = eltid;
-            cov_chunk->chunkSize = maxEltPlus;
-            cov_chunk->chunkIO = inst;
-            cov_chunk->loadOrder = loadOrder++;
-            //const uint chunkAlignment = std::max<uint>(alignment, 4);
-            //cov_chunk->chunkIO = CreateChunkLoad(inst, cov_chunk, eltid, chunkAlignment);
-            dircb_owloads.push_back(cov_chunk);
-        }  // end of if gfx cbload handling
-    } // loop over inst in block
-
-    if (dircb_owloads.size() <= 1)
-    {
-        return;
-    }
-
-    std::sort(dircb_owloads.begin(), dircb_owloads.end(), sortFunction);
-    std::vector<BufChunk*>::iterator iter = dircb_owloads.begin();
-    BufChunk* firstBufInChunk = *iter;
-    uint firstCBLoadEle = (*iter)->loadOrder;
-    llvm::Instruction* firstCBLoad = (*iter)->chunkIO;
-    llvm::Instruction* MoveToLocation = (*iter)->chunkIO;
-    iter++;
-    while (iter != dircb_owloads.end())
-    {
-        const uint scalarSizeInBytes = (*iter)->elementSize;
-        const uint eltid = (*iter)->chunkStart;
-        uint chunkSize = (*iter)->chunkStart + scalarSizeInBytes - firstBufInChunk->chunkStart;
-        static_assert(MAX_VECTOR_NUM_ELEMENTS >= SIZE_OWORD, "Code below may need an update");
-        if (firstBufInChunk->addrSpace == (*iter)->addrSpace &&
-            profitableChunkSize(chunkSize, scalarSizeInBytes) &&
-            ((scalarSizeInBytes * eltid) % 4) == 0)
-        {
-            if ((*iter)->loadOrder < firstCBLoadEle)
-            {
-                firstCBLoadEle = (*iter)->loadOrder;
-                MoveToLocation = (*iter)->chunkIO;
-            }
-            iter++;
-            continue;
-        }
-        else
-        {
-            // move the CB with the smallest index to before the 1st used CB with the same to-be-merged CBs.
-            if (firstCBLoad != MoveToLocation)
-            {
-                if (Instruction * temp = dyn_cast<Instruction>(firstCBLoad->getOperand(0)))
-                {
-                    firstCBLoad->moveBefore(MoveToLocation);
-                    temp->moveBefore(firstCBLoad);
-                }
-            }
-            firstCBLoadEle = (*iter)->loadOrder;
-            firstBufInChunk = *iter;
-            MoveToLocation = (*iter)->chunkIO;
-            firstCBLoad = (*iter)->chunkIO;
-            iter++;
-        }
-    }
-}
-
 bool ConstantCoalescing::profitableChunkSize(
     uint32_t ub, uint32_t lb, uint32_t eltSizeInBytes)
 {
@@ -843,7 +663,9 @@ void ConstantCoalescing::MergeScatterLoad(Instruction* load,
         BufChunk* cur_chunk = *rit;
         if (CompareBufferBase(cur_chunk->bufIdxV, cur_chunk->addrSpace, bufIdxV, addrSpace) &&
             cur_chunk->baseIdxV == eltIdxV &&
-            cur_chunk->chunkIO->getType()->getScalarType() == load->getType()->getScalarType())
+            cur_chunk->chunkIO->getType()->getScalarType() == load->getType()->getScalarType() &&
+            CompareMetadata(cur_chunk->chunkIO, load) &&
+            !CheckForAliasingWrites(addrSpace, cur_chunk->chunkIO, load))
         {
             uint lb = std::min(eltid, cur_chunk->chunkStart);
             uint ub = std::max(eltid + maxEltPlus, cur_chunk->chunkStart + cur_chunk->chunkSize);
@@ -1061,7 +883,7 @@ void ConstantCoalescing::CombineTwoLoads(
                 eac, PointerType::get(vty, addrSpace), "twoScalar", load0);
             m_TT->RegisterNewValueAndAssignID(ptrcast);
             wiAns->incUpdateDepend(ptrcast, WIAnalysis::RANDOM);
-            cov_chunk->chunkIO = irBuilder->CreateLoad(ptrcast, false);
+            cov_chunk->chunkIO = irBuilder->CreateLoad(vty, ptrcast, false);
             cast<LoadInst>(cov_chunk->chunkIO)->setAlignment(getAlign(4)); // \todo, more precise
             wiAns->incUpdateDepend(cov_chunk->chunkIO, WIAnalysis::RANDOM);
         }
@@ -1069,7 +891,7 @@ void ConstantCoalescing::CombineTwoLoads(
         {
             IGC_ASSERT(isa<IntToPtrInst>(addr_ptr));
             addr_ptr->mutateType(PointerType::get(vty, addrSpace));
-            cov_chunk->chunkIO = irBuilder->CreateLoad(addr_ptr, false);
+            cov_chunk->chunkIO = irBuilder->CreateLoad(cov_chunk->chunkIO->getType(), addr_ptr, false);
             cast<LoadInst>(cov_chunk->chunkIO)->setAlignment(getAlign(4));  // \todo, more precise
             wiAns->incUpdateDepend(cov_chunk->chunkIO, WIAnalysis::RANDOM);
             // modify the address calculation if the chunk-start is changed
@@ -1140,6 +962,8 @@ void ConstantCoalescing::CombineTwoLoads(
             irBuilder->getFalse()
         };
         cov_chunk->chunkIO = irBuilder->CreateCall(ldRawFn, args, ldRaw0->getName());
+        IGC_ASSERT(CompareMetadata(ldRaw0, ldRaw1));
+        CopyMetadata(cov_chunk->chunkIO, ldRaw0);
         wiAns->incUpdateDepend(cov_chunk->chunkIO, wiAns->whichDepend(ldRaw0));
     }
 
@@ -1210,21 +1034,22 @@ void ConstantCoalescing::MergeUniformLoad(Instruction* load,
         return;
     }
 
-    const Type* LoadEltTy;
-    if (!load->getType()->isPointerTy())
-        LoadEltTy = load->getType()->getScalarType();
+    const Type* loadDataTy = load->getType();
+    const Type* loadEltTy;
+    if (!loadDataTy->getScalarType()->isPointerTy())
+        loadEltTy = loadDataTy->getScalarType();
     else
     {
-        BufferType buffType = DecodeBufferType(load->getType()->getPointerAddressSpace());
+        BufferType buffType = DecodeBufferType(loadDataTy->getPointerAddressSpace());
         if (IsBindless(buffType)
             || buffType == STATELESS_A32
             || buffType == SSH_BINDLESS_CONSTANT_BUFFER
             )
-            LoadEltTy = irBuilder->getInt32Ty();
+            loadEltTy = irBuilder->getInt32Ty();
         else
             return;
     }
-    const uint scalarSizeInBytes = (const uint)(LoadEltTy->getPrimitiveSizeInBits() / 8);
+    const uint scalarSizeInBytes = (const uint)(loadEltTy->getPrimitiveSizeInBits() / 8);
 
     IGC_ASSERT(isPowerOf2_64(alignment));
     IGC_ASSERT(0 != scalarSizeInBytes);
@@ -1242,7 +1067,9 @@ void ConstantCoalescing::MergeUniformLoad(Instruction* load,
     {
         if (CompareBufferBase(cur_chunk->bufIdxV, cur_chunk->addrSpace, bufIdxV, addrSpace) &&
             cur_chunk->baseIdxV == eltIdxV &&
-            cur_chunk->chunkIO->getType()->getScalarType() == LoadEltTy)
+            cur_chunk->chunkIO->getType()->getScalarType() == loadEltTy &&
+            CompareMetadata(cur_chunk->chunkIO, load) &&
+            !CheckForAliasingWrites(addrSpace, cur_chunk->chunkIO, load))
         {
             uint lb = std::min(eltid, cur_chunk->chunkStart);
             uint ub = std::max(eltid + maxEltPlus, cur_chunk->chunkStart + cur_chunk->chunkSize);
@@ -1263,6 +1090,33 @@ void ConstantCoalescing::MergeUniformLoad(Instruction* load,
         cov_chunk = *I;
     }
 
+    // Stateless access needs OOB checks to avoid page faults.
+    bool needsOutOfBoundsChecks =
+        IGC_IS_FLAG_DISABLED(DisableConstantCoalescingOutOfBoundsCheck) &&
+        (addrSpace == ADDRESS_SPACE_CONSTANT ||
+         addrSpace == ADDRESS_SPACE_GLOBAL);
+
+    // Lambda rounds the chunk size up to the next value supported by data port
+    // loads or LSC transposed load.
+    // Legacy data port supports:
+    // - block loads of 1, 2, 4 or 8 OWords
+    // - scattered loads of 1, 2 and 4 DWords/QWords
+    // - (byte aligned) 1, 2 and 4 Bytes
+    // LSC supports:
+    // - transposed loads of 1, 2, 3, 4, 8, 16, 32 and 64 DWords/QWords
+    // - (byte aligned) loads of 1, 2 and 4 Bytes
+    auto RoundChunkSize = [this, scalarSizeInBytes, load](const uint32_t chunkSize)
+    {
+        bool supportsVec3Load = scalarSizeInBytes >= 4 && EmitPass::shouldGenerateLSCQuery(*m_ctx, load) == Tristate::True;
+        uint32_t validChunkSize = (supportsVec3Load && chunkSize == 3) ?
+            3 : iSTD::RoundPower2((DWORD)chunkSize);
+        return validChunkSize;
+    };
+    const uint32_t loadNumElements = loadDataTy->isVectorTy() ?
+        int_cast<uint32_t>(cast<IGCLLVM::FixedVectorType>(loadDataTy)->getNumElements()) : 1;
+    // VectorPreProcess pass legalizes loaded data size.
+    IGC_ASSERT(loadNumElements == RoundChunkSize(loadNumElements));
+    IGC_ASSERT(loadNumElements >= RoundChunkSize(maxEltPlus));
     if (!cov_chunk)
     {
         if (isDwordAligned)
@@ -1273,7 +1127,7 @@ void ConstantCoalescing::MergeUniformLoad(Instruction* load,
             cov_chunk->baseIdxV = eltIdxV;
             cov_chunk->elementSize = scalarSizeInBytes;
             cov_chunk->chunkStart = eltid;
-            cov_chunk->chunkSize = iSTD::RoundPower2((DWORD)maxEltPlus);
+            cov_chunk->chunkSize = RoundChunkSize(maxEltPlus);
             const alignment_t chunkAlignment = std::max<alignment_t>(alignment, 4);
             cov_chunk->chunkIO = CreateChunkLoad(load, cov_chunk, eltid, chunkAlignment, Extension);
             chunk_vec.push_back(cov_chunk);
@@ -1287,16 +1141,10 @@ void ConstantCoalescing::MergeUniformLoad(Instruction* load,
 
         // Calculate load start and size adjustments
         uint start_adj = cov_chunk->chunkStart - lb;
-        // Gen only has 1, 2, 4 or 8 oword loads round up
-        uint size_adj = iSTD::RoundPower2((DWORD)(ub - lb)) - cov_chunk->chunkSize;
+        uint size_adj = RoundChunkSize(ub - lb) - cov_chunk->chunkSize;
 
         // Out of bounds check
-        bool isStatelessLoad = false;
-        if (auto* LI = dyn_cast<LoadInst>(load))
-        {
-            isStatelessLoad = (LI->getPointerAddressSpace() == ADDRESS_SPACE_CONSTANT);
-        }
-        if (isStatelessLoad && IGC_IS_FLAG_DISABLED(DisableConstantCoalescingOutOfBoundsCheck))
+        if (needsOutOfBoundsChecks)
         {
             uint furthest_access = size_adj + lb + cov_chunk->chunkSize;
             if (furthest_access > ub)
@@ -1748,8 +1596,9 @@ Instruction* ConstantCoalescing::CreateChunkLoad(
         // Update debug location
         ptr->setDebugLoc(irBuilder->getCurrentDebugLocation());
         wiAns->incUpdateDepend(ptr, wiAns->whichDepend(seedi));
-        chunkLoad = irBuilder->CreateLoad(ptr);
+        chunkLoad = irBuilder->CreateLoad(vty, ptr);
         chunkLoad->setAlignment(getAlign(alignment));
+        CopyMetadata(chunkLoad, seedi);
         chunk->chunkIO = chunkLoad;
     }
     else
@@ -1788,6 +1637,7 @@ Instruction* ConstantCoalescing::CreateChunkLoad(
             GenISAIntrinsic::GenISA_ldrawvector_indexed,
             types);
         chunk->chunkIO = irBuilder->CreateCall(ldRawFn, arguments);
+        CopyMetadata(chunk->chunkIO, ldRaw);
     }
 
     wiAns->incUpdateDepend(chunk->chunkIO, wiAns->whichDepend(seedi));
@@ -2447,7 +2297,7 @@ Instruction* ConstantCoalescing::CreateSamplerLoad(
         cast<PointerType>(resourcePtr->getType()) :
         PointerType::get(irBuilder->getFloatTy(), addrSpace);
 
-    Type* types[] = { IGCLLVM::FixedVectorType::get(irBuilder->getFloatTy(), 4), resourceType };
+    Type* types[] = { IGCLLVM::FixedVectorType::get(irBuilder->getFloatTy(), 4), resourceType, resourceType };
     Function* l = GenISAIntrinsic::getDeclaration(curFunc->getParent(),
         llvm::GenISAIntrinsic::GenISA_ldptr,
         types);
@@ -2457,6 +2307,7 @@ Instruction* ConstantCoalescing::CreateSamplerLoad(
         irBuilder->getInt32(0),
         irBuilder->getInt32(0),
         irBuilder->getInt32(0),
+        UndefValue::get(resourceType),
         resourcePtr ? resourcePtr : ConstantPointerNull::get(resourceType),
         irBuilder->getInt32(0),
         irBuilder->getInt32(0),
@@ -2597,4 +2448,155 @@ void ConstantCoalescing::ReplaceLoadWithSamplerLoad(
     loadToReplace->replaceAllUsesWith(result);
 }
 
+// Check if any of the paths from the `dominatingChunk` to `mergeCandidate` has
+// aliasing memory store or atomic operations.
+bool ConstantCoalescing::CheckForAliasingWrites(
+    uint32_t addrSpace,
+    Instruction* dominatingChunk,
+    Instruction* mergeCandidate)
+{
+    BufferType accessType = DecodeBufferType(addrSpace);
+    // Only writable resources need to be checked.
+    if (accessType == STATELESS_READONLY ||
+        accessType == CONSTANT_BUFFER ||
+        accessType == BINDLESS_CONSTANT_BUFFER ||
+        accessType == SSH_BINDLESS_CONSTANT_BUFFER)
+    {
+        return false;
+    }
+
+    const DominatorTree& DT = getAnalysis<DominatorTreeWrapperPass>().getDomTree();
+    const bool statefulResourcesNotAliased =
+        m_ctx->getModuleMetaData()->statefulResourcesNotAliased;
+    BasicBlock* chunkBB = dominatingChunk->getParent();
+    BasicBlock* candidateBB = mergeCandidate->getParent();
+
+    // Lambda checks if an instruction may write to memory that potentially
+    // aliases the loaded memory.
+    auto IsAliasedWrite = [accessType, statefulResourcesNotAliased](
+        Instruction* inst)
+    {
+        // Treat memory fences as aliasing writes as they may change results
+        // of subsequent loads.
+        auto* intr = dyn_cast<GenIntrinsicInst>(inst);
+        if (intr &&
+            (intr->getIntrinsicID() == GenISAIntrinsic::GenISA_memoryfence ||
+             intr->getIntrinsicID() == GenISAIntrinsic::GenISA_LSCFence))
+        {
+            return true;
+        }
+        if (!inst->mayWriteToMemory())
+        {
+            return false;
+        }
+        Value* ptr = GetBufferOperand(inst);
+        if (ptr == nullptr || !ptr->getType()->isPointerTy())
+        {
+            return false;
+        }
+        BufferType writeAccessType = DecodeBufferType(
+            ptr->getType()->getPointerAddressSpace());
+        if (writeAccessType != STATELESS &&
+            writeAccessType != UAV &&
+            writeAccessType != BINDLESS &&
+            writeAccessType != SSH_BINDLESS)
+        {
+            return false;
+        }
+        if (statefulResourcesNotAliased &&
+            writeAccessType != accessType)
+        {
+            // Note: the following improvements are possible:
+            // - check BTI index for direct access to UAV/CONSTANT_BUFFER
+            // - consider checking the "unique_id" part of the addressspace
+            //   for bindless access
+            return false;
+        }
+        // By default assume that the write access may alias the load
+        return true;
+    };
+
+    // Lambda checks if a range of instructions has an instruction that may
+    // write to memory that potentially aliases the loaded memory.
+    auto HasAliasedWrite = [&IsAliasedWrite](
+        auto beginIt,
+        auto endIt)
+    {
+        for (auto II = beginIt, EI = endIt; II != EI; ++II)
+        {
+            if (IsAliasedWrite(&*II))
+            {
+                return true;
+            }
+        }
+        return false;
+    };
+
+    bool sameBB = chunkBB == candidateBB;
+    // First check the merge candidate BB
+    if (HasAliasedWrite(
+        sameBB ? dominatingChunk->getIterator() : candidateBB->begin(),
+        mergeCandidate->getIterator()))
+    {
+        return true;
+    }
+    if (!sameBB)
+    {
+        // Check the dominating BB
+        if (HasAliasedWrite(dominatingChunk->getIterator(), chunkBB->end()))
+        {
+            return true;
+        }
+        // Check all potentially reachable BBs starting from predecessors of the
+        // candidate merge BB and going up to to the dominating chunk BB.
+        IGC_ASSERT(DT.dominates(chunkBB, candidateBB));
+        SmallSet<BasicBlock*, 16> seen;
+        SmallVector<BasicBlock*, 16> worklist(pred_begin(candidateBB), pred_end(candidateBB));
+        seen.insert(chunkBB);
+        seen.insert(candidateBB);
+        while (!worklist.empty())
+        {
+            BasicBlock* bb = worklist.pop_back_val();
+            if (seen.count(bb) > 0)
+            {
+                continue;
+            }
+            if (HasAliasedWrite(bb->begin(), bb->end()))
+            {
+                return true;
+            }
+            seen.insert(bb);
+            worklist.insert(worklist.end(), pred_begin(bb), pred_end(bb));
+        }
+    }
+    return false;
+}
+
+// Checks if instructions are decorated with the same or compatible metadata.
+// Returns true if metadata is compatible.
+// Note: currently only LSC cache control metadata is checked.
+bool ConstantCoalescing::CompareMetadata(
+    Instruction* instA,
+    Instruction* instB) const
+{
+    const MDNode* nodeA = instA->getMetadata("lsc.cache.ctrl");
+    const MDNode* nodeB = instB->getMetadata("lsc.cache.ctrl");
+    IGC_ASSERT(nodeA == nullptr || nodeA->isUniqued());
+    IGC_ASSERT(nodeB == nullptr || nodeB->isUniqued());
+    return nodeA == nodeB;
+}
+
+// Copy known metadata.
+// Note: currently only LSC cache control metadata is copied.
+void ConstantCoalescing::CopyMetadata(Instruction* dst, Instruction* src)
+{
+    MDNode* nodeDst = dst->getMetadata("lsc.cache.ctrl");
+    MDNode* nodeSrc = src->getMetadata("lsc.cache.ctrl");
+    if (nodeDst != nodeSrc)
+    {
+        IGC_ASSERT(nodeDst == nullptr);
+        dst->setMetadata("lsc.cache.ctrl", nodeSrc);
+    }
+}
+
 char IGC::ConstantCoalescing::ID = 0;
