@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -13,6 +13,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/DebugInfo/ScalarVISAModule.h"
 #include "Compiler/CodeGenContextWrapper.hpp"
 #include "Compiler/IGCPassSupport.h"
+#include "Compiler/MetaDataApi/PurgeMetaDataUtils.hpp"
 #include "Compiler/MetaDataUtilsWrapper.h"
 #include "common/igc_regkeys.hpp"
 #include "common/LLVMWarningsPush.hpp"
@@ -30,6 +31,7 @@ SPDX-License-Identifier: MIT
 #include "llvm/IR/InstVisitor.h"
 #include "llvm/Transforms/IPO.h"
 #include "llvm/Transforms/IPO/Inliner.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/Transforms/Utils/Cloning.h"
 #include "llvm/IR/DebugInfo.h"
 #include "llvm/IR/DIBuilder.h"
@@ -79,7 +81,7 @@ static inline void CloneFuncMetadata(IGCMD::MetaDataUtils* pM,
 {
     using namespace IGC::IGCMD;
     auto Info = pM->getFunctionsInfoItem(F);
-    auto NewInfo = FunctionInfoMetaDataHandle(FunctionInfoMetaData::get());
+    auto NewInfo = FunctionInfoMetaDataHandle(new FunctionInfoMetaData());
 
     // Copy function type info.
     if (Info->isTypeHasValue())
@@ -136,6 +138,73 @@ inline Function* getCallerFunc(Value* user)
     return caller;
 }
 
+void GenXCodeGenModule::detectUnpromotableFunctions(Module* pM)
+{
+    auto pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+    SmallSet<Function*, 32> tempFuncs;
+
+    // Find functions that have uses of "localSLM" globals
+    for (auto gi = pM->global_begin(), ge = pM->global_end(); gi != ge; gi++)
+    {
+        GlobalVariable* GV = dyn_cast<GlobalVariable>(gi);
+        if (GV && GV->hasSection() && GV->getSection().equals("localSLM"))
+        {
+            for (auto user : GV->users())
+            {
+                if (Instruction* U = dyn_cast<Instruction>(user))
+                {
+                    Function* pF = U->getParent()->getParent();
+                    if (!isEntryFunc(pMdUtils, pF))
+                        tempFuncs.insert(pF);
+                }
+            }
+        }
+    }
+
+    // Find functions that uses instructions that can't be handled for indirect call
+    for (auto& F : *pM)
+    {
+        // Only look at non-entry functions
+        if (F.isDeclaration() || isEntryFunc(pMdUtils, &F) || tempFuncs.count(&F) != 0)
+            continue;
+
+        for (inst_iterator I = inst_begin(F), E = inst_end(F); I != E; ++I)
+        {
+            if (auto* GII = dyn_cast<GenIntrinsicInst>(&*I))
+            {
+                // Can't make indirect if threadgroupbarrier intrinsic is set
+                if (GII->getIntrinsicID() == GenISAIntrinsic::GenISA_threadgroupbarrier)
+                {
+                    tempFuncs.insert(&F);
+                    break;
+                }
+            }
+        }
+    }
+
+    // Recursively add callers, as the whole chain of calls cannot be promoted
+    std::function<void(Function*)> AddCallerRecursive = [&](Function* F)
+    {
+        if (isEntryFunc(pMdUtils, F))
+            return;
+
+        m_UnpromotableFuncs.insert(F);
+        for (auto user : F->users())
+        {
+            if (Instruction* I = dyn_cast<Instruction>(user))
+            {
+                Function* parentF = I->getParent()->getParent();
+                AddCallerRecursive(parentF);
+            }
+        }
+    };
+
+    for (auto F : tempFuncs)
+    {
+        AddCallerRecursive(F);
+    }
+}
+
 void GenXCodeGenModule::processFunction(Function& F)
 {
     // See what FunctionGroups this Function is called from.
@@ -172,7 +241,7 @@ void GenXCodeGenModule::processFunction(Function& F)
     auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
     auto pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
 
-    auto CanMakeIndirectFunc = [&](llvm::Function* F)->bool
+    auto CanMakeIndirectFunc = [&](llvm::Function* F)
     {
         // Don't convert subroutines, builtins, or invoke_simd_target
         if (F->hasFnAttribute("visaStackCall") == false ||
@@ -184,11 +253,11 @@ void GenXCodeGenModule::processFunction(Function& F)
         // have the same SIMD sizes, otherwise we cannot make it an indirect call
         int simd_size = 0;
         unsigned CallersPerSIMD[3] = { 0, 0, 0 };
-        for (auto iter : CallerFGs)
+        for (const auto &iter : CallerFGs)
         {
             Function* callerKernel = iter.first->getHead();
             auto funcInfoMD = pMdUtils->getFunctionsInfoItem(callerKernel);
-            int sz = funcInfoMD->getSubGroupSize()->getSIMD_size();
+            int sz = funcInfoMD->getSubGroupSize()->getSIMDSize();
             if (sz != 0) {
                 if (simd_size == 0)
                     simd_size = sz;
@@ -199,6 +268,7 @@ void GenXCodeGenModule::processFunction(Function& F)
             if (IGC_IS_FLAG_DISABLED(EnableSIMDVariantCompilation) && simd_size != sz)
                 return false;
         }
+
         if (IGC_IS_FLAG_ENABLED(EnableSIMDVariantCompilation))
         {
             // For MultiSIMD compile, check profitability.
@@ -220,17 +290,12 @@ void GenXCodeGenModule::processFunction(Function& F)
             return false;
         }
 
-        // Can't make indirect if threadgroupbarrier intrinsic is set
-        for (inst_iterator I = inst_begin(F), E = inst_end(F); I != E; ++I)
+        // Detect if function is part of the unpromotable set
+        if (m_UnpromotableFuncs.count(F) != 0)
         {
-            if (auto* GII = dyn_cast<GenIntrinsicInst>(&*I))
-            {
-                if (GII->getIntrinsicID() == GenISAIntrinsic::GenISA_threadgroupbarrier)
-                {
-                    return false;
-                }
-            }
+            return false;
         }
+
         return true;
     };
 
@@ -243,6 +308,32 @@ void GenXCodeGenModule::processFunction(Function& F)
         auto IFG = FGA->getOrCreateIndirectCallGroup(F.getParent());
         if (IFG)
         {
+            // If EnableSIMDVariantCompilation=0, we only compile a single variant of the dummy kernel.
+            // By default, getOrCreateIndirectCallGroup will create a dummy kernel with the lowest allowed SIMD size,
+            // but if only a single variant exists, and we determine that the caller has a required subgroup size
+            // different from the default size, we need to change the default subgroup size here to match the caller.
+            if (IGC_IS_FLAG_DISABLED(EnableSIMDVariantCompilation))
+            {
+                int req_subgroup = 0;
+                for (const auto &FG : CallerFGs)
+                {
+                    auto FHead = FG.first->getHead();
+                    auto subGrpSz = pMdUtils->getFunctionsInfoItem(FHead)->getSubGroupSize();
+                    if (subGrpSz->isSIMDSizeHasValue())
+                    {
+                        // We can assume all callers have the same subgroup size requirement, otherwise
+                        // CanMakeIndirectFunc will return false when EnableSIMDVariantCompilation=0
+                        req_subgroup = subGrpSz->getSIMDSize();
+                        break;
+                    }
+                }
+                if (req_subgroup)
+                {
+                    auto defaultKernel = IFG->getHead();
+                    pMdUtils->getFunctionsInfoItem(defaultKernel)->getSubGroupSize()->setSIMDSize(req_subgroup);
+                    pMdUtils->save(pCtx->getModule()->getContext());
+                }
+            }
             if (IGC_IS_FLAG_ENABLED(PrintStackCallDebugInfo))
             {
                 dbgs() << "Make Indirect: " << F.getName().str() << "\n";
@@ -255,7 +346,7 @@ void GenXCodeGenModule::processFunction(Function& F)
     }
 
     bool FirstPair = true;
-    for (auto FGPair : CallerFGs)
+    for (const auto &FGPair : CallerFGs)
     {
         if (FirstPair)
         {
@@ -484,6 +575,12 @@ bool GenXCodeGenModule::runOnModule(Module& M)
     // Add all indirect functions to the default kernel group
     FGA->addIndirectFuncsToKernelGroup(&M);
 
+    // If FunctionCloningThreshold is enabled, detect functions that cannot be promoted to indirect
+    if (m_FunctionCloningThreshold > 0)
+    {
+        detectUnpromotableFunctions(&M);
+    }
+
     for (auto I = SCCVec.rbegin(), IE = SCCVec.rend(); I != IE; ++I)
     {
         std::vector<CallGraphNode*>* SCCNodes = (*I);
@@ -640,18 +737,16 @@ static inline int getDefaultSIMDSize(CodeGenContext* ctx)
     }
 
     // By default, use the minimum allowed SIMD for the HW platform
-    int defaultSz = numLanes(ctx->platform.getMinDispatchMode());
+    SIMDMode simdMode = ctx->platform.getMinDispatchMode();
+    int defaultSz = numLanes(simdMode);
 
-    // Library Compilation requires functions to be exported externally, so we should always compile to the default size.
-    // Otherwise, use the sub_group_size set for the dummy kernel (in InsertDummyKernelForSymbolTable pass) if available.
-    if (ctx->getModuleMetaData()->compOpt.IsLibraryCompilation == false)
+    if (ctx->getModuleMetaData()->compOpt.IsLibraryCompilation)
     {
-        Function* defaultDummyKernel = IGC::getIntelSymbolTableVoidProgram(ctx->getModule());
-        if (defaultDummyKernel)
+        unsigned sz = ctx->getModuleMetaData()->compOpt.LibraryCompileSIMDSize;
+        if (sz > 0)
         {
-            auto subGrpSz = ctx->getMetaDataUtils()->getFunctionsInfoItem(defaultDummyKernel)->getSubGroupSize();
-            if (subGrpSz->isSIMD_sizeHasValue())
-                defaultSz = subGrpSz->getSIMD_size();
+            IGC_ASSERT(sz == 8 || sz == 16 || sz == 32);
+            defaultSz = sz;
         }
     }
     return defaultSz;
@@ -691,7 +786,7 @@ FunctionGroup* GenXFunctionGroupAnalysis::getOrCreateIndirectCallGroup(Module* p
 
         // Set spirv calling convention and kernel metadata
         pNewFunc->setCallingConv(llvm::CallingConv::SPIR_KERNEL);
-        IGCMD::FunctionInfoMetaDataHandle fHandle = IGCMD::FunctionInfoMetaDataHandle(IGCMD::FunctionInfoMetaData::get());
+        IGCMD::FunctionInfoMetaDataHandle fHandle = IGCMD::FunctionInfoMetaDataHandle(new IGCMD::FunctionInfoMetaData());
         FunctionMetaData* funcMD = &pModMD->FuncMD[pNewFunc];
         funcMD->functionType = IGC::FunctionTypeMD::KernelFunction;
         fHandle->setType(FunctionTypeMD::KernelFunction);
@@ -699,7 +794,7 @@ FunctionGroup* GenXFunctionGroupAnalysis::getOrCreateIndirectCallGroup(Module* p
         defaultKernel = pNewFunc;
     }
     // Set the requested sub_group_size value for this kernel
-    pMdUtil->getFunctionsInfoItem(defaultKernel)->getSubGroupSize()->setSIMD_size(SimdSize);
+    pMdUtil->getFunctionsInfoItem(defaultKernel)->getSubGroupSize()->setSIMDSize(SimdSize);
     pMdUtil->save(pModule->getContext());
 
     auto FG = getGroup(defaultKernel);
@@ -891,7 +986,7 @@ void GenXFunctionGroupAnalysis::CloneFunctionGroupForMultiSIMDCompile(llvm::Modu
                 {
                     auto FG = getGroup(callerF);
                     auto funcInfoMD = pMdUtils->getFunctionsInfoItem(FG->getHead());
-                    int sz = funcInfoMD->getSubGroupSize()->getSIMD_size();
+                    int sz = funcInfoMD->getSubGroupSize()->getSIMDSize();
                     if (sz != 0)
                     {
                         IGC_ASSERT(sz == 8 || sz == 16 || sz == 32);
@@ -921,7 +1016,7 @@ void GenXFunctionGroupAnalysis::CloneFunctionGroupForMultiSIMDCompile(llvm::Modu
                         auto pNewICG = getOrCreateIndirectCallGroup(pModule, simdsz);
                         addToFunctionGroup(FCloned, pNewICG, FCloned);
 
-                        for (auto iter : UsersMap)
+                        for (const auto &iter : UsersMap)
                         {
                             if (iter.second == simdsz)
                             {
@@ -939,7 +1034,7 @@ void GenXFunctionGroupAnalysis::CloneFunctionGroupForMultiSIMDCompile(llvm::Modu
         }
     }
 
-    auto hasIndirectCaller = [this](Function* F, FunctionGroup* ICG)->bool
+    auto hasIndirectCaller = [this](Function* F, FunctionGroup* ICG)
     {
         for (auto U : F->users()) {
             if (Instruction* I = dyn_cast<Instruction>(U)) {
@@ -1192,7 +1287,8 @@ namespace {
     /// \brief Custom inliner for subroutines.
     class SubroutineInliner : public LegacyInlinerBase, public llvm::InstVisitor<SubroutineInliner>
     {
-        EstimateFunctionSize* FSA;
+        EstimateFunctionSize* FSA = nullptr;
+        MetaDataUtilsWrapper* MDUW = nullptr;
     public:
         static char ID; // Pass identification, replacement for typeid
 
@@ -1211,7 +1307,9 @@ namespace {
 
         using llvm::Pass::doFinalization;
         bool doFinalization(CallGraph& CG) override {
-            return removeDeadFunctions(CG);
+            bool Changed = removeDeadFunctions(CG);
+            Changed |= purgeMetaDataUtils(CG.getModule(), MDUW);
+            return Changed;
         }
     };
 
@@ -1220,6 +1318,7 @@ namespace {
 IGC_INITIALIZE_PASS_BEGIN(SubroutineInliner, "SubroutineInliner", "SubroutineInliner", false, false)
 IGC_INITIALIZE_PASS_DEPENDENCY(EstimateFunctionSize)
 IGC_INITIALIZE_PASS_DEPENDENCY(CallGraphWrapperPass)
+IGC_INITIALIZE_PASS_DEPENDENCY(MetaDataUtilsWrapper)
 IGC_INITIALIZE_PASS_END(SubroutineInliner, "SubroutineInliner", "SubroutineInliner", false, false)
 
 char SubroutineInliner::ID = 0;
@@ -1228,6 +1327,7 @@ void SubroutineInliner::getAnalysisUsage(AnalysisUsage& AU) const
 {
     AU.addRequired<EstimateFunctionSize>();
     AU.addRequired<CodeGenContextWrapper>();
+    AU.addRequired<MetaDataUtilsWrapper>();
     LegacyInlinerBase::getAnalysisUsage(AU);
 }
 
@@ -1272,14 +1372,14 @@ void SubroutineInliner::visitMemCpyInst(MemCpyInst& I)
         if (origSrc->getType()->getPointerAddressSpace() != Src->getType()->getPointerAddressSpace())
         {
             Value* SrcCast = BitCastInst::Create(Instruction::BitCast, origSrc,
-                PointerType::get(IGCLLVM::getNonOpaquePtrEltTy(Src->getType()), origSrc->getType()->getPointerAddressSpace()),
+                IGCLLVM::getWithSamePointeeType(dyn_cast<PointerType>(Src->getType()), origSrc->getType()->getPointerAddressSpace()),
                 "", &I);
             I.replaceUsesOfWith(Src, SrcCast);
         }
         if (origDst->getType()->getPointerAddressSpace() != Dst->getType()->getPointerAddressSpace())
         {
             Value* DstCast = BitCastInst::Create(Instruction::BitCast, origDst,
-                PointerType::get(IGCLLVM::getNonOpaquePtrEltTy(Dst->getType()), origDst->getType()->getPointerAddressSpace()),
+                IGCLLVM::getWithSamePointeeType(dyn_cast<PointerType>(Dst->getType()), origDst->getType()->getPointerAddressSpace()),
                 "", &I);
             I.replaceUsesOfWith(Dst, DstCast);
         }
@@ -1308,6 +1408,7 @@ void SubroutineInliner::verifyAddrSpaceMismatch(CallGraphSCC& SCC)
 bool SubroutineInliner::runOnSCC(CallGraphSCC& SCC)
 {
     FSA = &getAnalysis<EstimateFunctionSize>();
+    MDUW = &getAnalysis<MetaDataUtilsWrapper>();
     bool changed = LegacyInlinerBase::runOnSCC(SCC);
     if (changed) verifyAddrSpaceMismatch(SCC);
 
@@ -1341,8 +1442,7 @@ InlineCost SubroutineInliner::getInlineCost(IGCLLVM::CallSiteRef CS)
         if (pCtx->m_enableSubroutine == false)
             return llvm::InlineCost::getAlways("Disabled subroutines/stackcalls");
 
-        if (pCtx->type == ShaderType::OPENCL_SHADER &&
-            Callee->hasFnAttribute(llvm::Attribute::NoInline))
+        if (Callee->hasFnAttribute(llvm::Attribute::NoInline))
             return llvm::InlineCost::getNever("Per NoInline function attribute");
 
         if (Callee->hasFnAttribute("KMPLOCK"))
