@@ -113,7 +113,9 @@ void IGC::HalfPromotion::handleGenIntrinsic(llvm::GenIntrinsicInst& I)
     GenISAIntrinsic::ID id = I.getIntrinsicID();
     if (id == GenISAIntrinsic::GenISA_WaveAll ||
         id == GenISAIntrinsic::GenISA_WavePrefix ||
-        id == GenISAIntrinsic::GenISA_WaveClustered)
+        id == GenISAIntrinsic::GenISA_WaveClustered ||
+        id == GenISAIntrinsic::GenISA_WaveInterleave ||
+        id == GenISAIntrinsic::GenISA_WaveClusteredInterleave)
     {
         Module* M = I.getParent()->getParent()->getParent();
         llvm::IGCIRBuilder<> builder(&I);
