@@ -48,6 +48,7 @@ namespace IGC
         std::vector<const char*> m_VISAAsmToLink;
         // Functions that are forced to be direct calls.
         std::unordered_set<std::string> m_DirectCallFunctions;
+        SComputeShaderWalkOrder m_walkOrderStruct;
 
         OpenCLProgramContext(
             const COCLBTILayout& btiLayout,
@@ -167,6 +168,7 @@ namespace IGC
     public:
         SOpenCLProgramInfo* m_programInfo;
         SOpenCLKernelInfo m_kernelInfo;
+        SOpenCLKernelCostExpInfo m_kernelCostexpInfo;
 
         unsigned int m_perWIStatelessPrivateMemSize;
 
@@ -261,6 +263,9 @@ namespace IGC
         std::string getKernelArgTypeQualifier(const FunctionMetaData& funcMD, uint argIndex) const;
         std::string getKernelArgAddressQualifier(const FunctionMetaData& funcMD, uint argIndex) const;
         std::string getKernelArgAccessQualifier(const FunctionMetaData& funcMD, uint argIndex) const;
+        // Helper function to get SIMD size specified in intel_reqd_sub_group_size attribute
+        uint32_t getReqdSubGroupSize(llvm::Function& F, IGC::IGCMD::MetaDataUtils* MDUtils) const;
+        uint32_t getMaxPressure(llvm::Function& F, IGC::IGCMD::MetaDataUtils* MDUtils) const;
     };
 
     void CodeGen(OpenCLProgramContext* ctx);
