@@ -204,14 +204,9 @@ void InternalOptions::parseOptions(const char* internalOpts)
         }
     }
 
-    if (internalOptions.hasArg(OPT_allow_zebin_common))
-    {
-        EnableZEBinary = true;
-    }
-
     if (internalOptions.hasArg(OPT_disable_zebin_common))
     {
-        EnableZEBinary = false;
+        DisableZEBinary = true;
     }
 
     if (internalOptions.hasArg(OPT_exclude_ir_from_zebin_common))
@@ -414,6 +409,20 @@ void InternalOptions::parseOptions(const char* internalOpts)
     {
         EnableBufferBoundsChecking = true;
     }
+
+    if (const llvm::opt::Arg* arg = internalOptions.getLastArg(OPT_minimum_valid_address_checking_common))
+    {
+        llvm::StringRef valStr = arg->getValue();
+        int val = 0;
+        if (valStr.getAsInteger(16, val) || val < 0)
+        {
+            IGC_ASSERT_MESSAGE(false, "-cl-minimum-valid-address-checking: invalid value, ignored!");
+        }
+        else
+        {
+            MinimumValidAddress = val;
+        }
+    }
 }
 
 void Options::parseOptions(const char* opts)
@@ -589,14 +598,14 @@ void Options::parseOptions(const char* opts)
         EnableFP64GenConvEmu = true;
     }
 
-    if (apiOptions.hasArg(OPT_enable_zebin_common))
+    if (const llvm::opt::Arg* arg = apiOptions.getLastArg(OPT_Xfinalizer))
     {
-        EnableZEBinary = true;
+        Xfinalizer = true;
+        XfinalizerOption = arg->getValue();
     }
 
-    if (apiOptions.hasArg(OPT_disable_zebin_common))
-    {
-        EnableZEBinary = false;
+    if (apiOptions.hasArg(OPT_static_profile_guided_trimming_common)) {
+        StaticProfileGuidedTrimming = true;
     }
 }
 } // namespace IGC
