@@ -47,7 +47,7 @@ void RuntimeValueLegalizationPass::getAnalysisUsage(llvm::AnalysisUsage& AU) con
 ////////////////////////////////////////////////////////////////////////////
 // @brief RuntimeValue comaprator function
 static std::function<bool(const std::pair<uint32_t, uint32_t>&, const std::pair<uint32_t, uint32_t>&)>
-RuntimeValueComparator = [](const std::pair<uint32_t, uint32_t>& lhs, const std::pair<uint32_t, uint32_t>& rhs) -> bool
+RuntimeValueComparator = [](const std::pair<uint32_t, uint32_t>& lhs, const std::pair<uint32_t, uint32_t>& rhs)
 {
     return (lhs.first < rhs.first) || ((lhs.first == rhs.first) && (lhs.second > rhs.second));
 };
@@ -144,7 +144,7 @@ static void GetDisjointRegions(
     // ->{ {6, 6},  RuntimeValue* }
     // ->{ {8, 19}, RuntimeValue* }
     //   { {8, 8},  RuntimeValue* }
-    for (auto it : runtimeValueCalls)
+    for (const auto& it : runtimeValueCalls)
     {
         std::pair<uint32_t, uint32_t> range = it.first;
         if (disjointRegions.empty() ||
@@ -286,7 +286,7 @@ bool RuntimeValueLegalizationPass::runOnModule(llvm::Module& module)
         GetAccessedRegions(accessedRegions, runtimeValueCalls, dataGRFAlignmentInDwords);
 
         // Loop through all RuntimeValue calls
-        for (auto it : runtimeValueCalls)
+        for (const auto& it : runtimeValueCalls)
         {
             llvm::CallInst* callToResolve = llvm::cast<llvm::CallInst>(it.second);
 
