@@ -14,6 +14,7 @@ SPDX-License-Identifier: MIT
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/Analysis/LoopInfo.h>
 #include <llvm/Analysis/ScalarEvolution.h>
+#include <llvm/Analysis/ValueTracking.h>
 #include <llvm/IR/DataLayout.h>
 #include <llvm/IR/Instructions.h>
 #include <llvm/IR/BasicBlock.h>
@@ -65,10 +66,10 @@ namespace
         struct StoreInfo {
             PatternType  ty;  // Pattern type if this store is part of
                               // a pattern.
-            int  ix;          // ix into m_storeInsts vector
-            SymExpr* SE;      // Store's address
-            Constant* Val;    // Store's value if it is constant.
-            int offset;       // offset relative to a particular SymExpr. Its meaning
+            int  ix{};          // ix into m_storeInsts vector
+            SymExpr* SE = nullptr;      // Store's address
+            Constant* Val = nullptr;    // Store's value if it is constant.
+            int offset{};       // offset relative to a particular SymExpr. Its meaning
                               // is based on the context in which it is used.
 
             StoreInfo() : ty(PATTERN_UNDEFINED) {}
@@ -90,11 +91,11 @@ namespace
         }
 
     private:
-        CodeGenContext* m_Ctx;
-        Function* m_F;
-        LoopInfo* m_LI;
-        DominatorTree* m_DT;
-        const DataLayout* m_DL;
+        CodeGenContext* m_Ctx = nullptr;
+        Function* m_F = nullptr;
+        LoopInfo* m_LI = nullptr;
+        DominatorTree* m_DT = nullptr;
+        const DataLayout* m_DL = nullptr;
         SymbolicEvaluation m_SymEval;
 
         // All SLM loads
@@ -143,17 +144,25 @@ void SymbolicEvaluation::dump_symbols()
         Vals[VSI->ID] = V;
     }
 
-    dbgs() << "\nSymbols used\n";
+    dbgs() << "\nSymbols used\n\n";
     for (int i = 0; i < m_nextValueID; ++i) {
         const Value* V = Vals[i];
-        // V should not be nullptr, but check it
-        // for safety.
+        // V should not be nullptr, but check it for safety.
         IGC_ASSERT(nullptr != V);
         if (V) {
-            dbgs() << "\n    V"
-                << i
-                << ":    "
-                << *V;
+            ValueSymInfo* VSI = getSymInfo(V);
+            SymCastInfo SCI = (SymCastInfo)VSI->CastInfo;
+            dbgs() << "  V" << i;
+            switch (SCI) {
+            case SYMCAST_ZEXT:
+                dbgs() << " [zext] :    " << *V << "\n";
+                break;
+            case SYMCAST_SEXT:
+                dbgs() << " [sext] :    " << *V << "\n";
+                break;
+            default:
+                dbgs() << " :    " << *V << "\n";
+            }
         }
     }
     dbgs() << "\n\n";
@@ -299,6 +308,11 @@ void SymbolicEvaluation::dump(const SymProd* P)
     dbgs() << "\n";
 }
 
+void SymbolicEvaluation::dump(SymProd* P)
+{
+    dump((const SymProd*)P);
+}
+
 void SymbolicEvaluation::dump(const SymTerm* T)
 {
     dbgs() << "\n";
@@ -307,6 +321,12 @@ void SymbolicEvaluation::dump(const SymTerm* T)
     dbgs() << "\n";
 }
 
+void SymbolicEvaluation::dump(SymTerm* T)
+{
+    dump((const SymTerm*)T);
+}
+
+
 void SymbolicEvaluation::dump(const SymExpr* SE)
 {
     dbgs() << "\n";
@@ -315,6 +335,11 @@ void SymbolicEvaluation::dump(const SymExpr* SE)
     dbgs() << "\n";
 }
 
+void SymbolicEvaluation::dump(SymExpr* SE)
+{
+    dump((const SymExpr*)SE);
+}
+
 void SymbolicEvaluation::dump(const Value* V)
 {
     dbgs() << "\n";
@@ -323,6 +348,10 @@ void SymbolicEvaluation::dump(const Value* V)
     dbgs() << "\n";
 }
 
+void SymbolicEvaluation::dump(Value* V)
+{
+    dump((const Value*)V);
+}
 
 #endif
 
@@ -350,7 +379,7 @@ SymExpr* SymbolicEvaluation::getSymExpr(const Value* V)
     if (expr == nullptr)
     {
         // Create a SymExpr (it is a constant)
-        SymExpr* E = new (m_symEvaAllocator) SymExpr();
+        SymExpr* E = new (m_symExprAllocator.Allocate()) SymExpr();
         E->ConstTerm = coeff;
         setSymInfo(V, E);
 
@@ -370,12 +399,28 @@ void SymbolicEvaluation::getSymExprOrConstant(const Value* V, SymExpr*& S, int64
         return;
     }
 
-    if (const ConstantInt * CI = dyn_cast<const ConstantInt>(V))
-    {
-        C = CI->getSExtValue();
-        return;
+    bool isEvaluable = true;
+    // Expect scalar type. Stop evaluation if not scalar type.
+    if (isa<Constant>(V)) {
+        const ConstantInt* CI = dyn_cast<const ConstantInt>(V);
+        if (CI && V->getType()->isSingleValueType() && !V->getType()->isVectorTy())
+        {
+            C = CI->getSExtValue();
+            return;
+        }
+        isEvaluable = false;
     }
 
+    // Used for nomalizing shift amount.
+    //   For example, i64, its type mask is 03F(63 = 64 - 1).
+    auto getTypeMask = [](const Type* Ty) -> uint32_t {
+        // For simplicity, only handle type whose size is power of 2.
+        uint32_t nbits = Ty->getScalarSizeInBits();
+        if (nbits > 0 && isPowerOf2_32(nbits))
+            return (nbits - 1);
+        return 0;
+    };
+
     // Instructions/Operators handled for now:
     //   GEP
     //   bitcast (inttoptr, ptrtoint, etc)
@@ -383,7 +428,15 @@ void SymbolicEvaluation::getSymExprOrConstant(const Value* V, SymExpr*& S, int64
     SymExpr* S0;
     SymExpr* S1;
     int64_t  C0, C1;
-    if (const Operator* Op = dyn_cast<Operator>(V))
+
+    auto BinOp = dyn_cast<OverflowingBinaryOperator>(V);
+    const Operator* Op = dyn_cast<Operator>(V);
+    const bool canOverflow = (considerOverflow() && BinOp &&
+        !(BinOp->hasNoSignedWrap() || BinOp->hasNoUnsignedWrap()));
+    // Skip if Op can overflow.
+    //   Hope we don't need to handle cases if considerOverflow() is true;
+    //   otherwise, the code needs more tuning.
+    if (Op && isEvaluable && !canOverflow && !exceedMaxValues())
     {
         unsigned opc = Op->getOpcode();
         switch (opc) {
@@ -401,7 +454,7 @@ void SymbolicEvaluation::getSymExprOrConstant(const Value* V, SymExpr*& S, int64
                     return;
                 }
             }
-            else  if (IGCLLVM::getNonOpaquePtrEltTy(V0->getType())->isSized() &&
+            else  if (GEPOp->getSourceElementType()->isSized() &&
                 !V0->getType()->isVectorTy() &&
                 m_DL != nullptr)
             {
@@ -515,6 +568,32 @@ void SymbolicEvaluation::getSymExprOrConstant(const Value* V, SymExpr*& S, int64
             }
             break;
         }
+        case Instruction::Or:
+        {
+            // Check if it is actually an add.
+            //
+            //   %mul = shl nuw nsw i64 %v, 1
+            //   %add = or i64 %mul, 1
+            //     -->  %add = add %mul, 1
+            const Value* V0 = Op->getOperand(0);
+            const Value* V1 = Op->getOperand(1);
+            getSymExprOrConstant(V0, S0, C0);
+            getSymExprOrConstant(V1, S1, C1);
+            if (!S0 && !S1) {
+                C = C0 | C1;
+                return;
+            }
+
+            // Case: 'or V0  Const' or 'or const  V1'
+            if ((S0 && !S1) || (!S0 && S1)) {
+                const Value* tV = (S0 ? V0 : V1);
+                const uint64_t tC = (uint64_t)(S0 ? C1 : C0);
+                uint32_t bits = (uint32_t)tV->getType()->getPrimitiveSizeInBits();
+                if (bits != 0 && MaskedValueIsZero(tV, APInt(bits, tC), *m_DL))
+                    S = add(S0 ? S0 : S1, tC);
+            }
+            break;
+        }
         case Instruction::Mul:
         {
             const Value* V0 = Op->getOperand(0);
@@ -538,7 +617,34 @@ void SymbolicEvaluation::getSymExprOrConstant(const Value* V, SymExpr*& S, int64
 
             break;
         }
+        case Instruction::Shl:
+        {
+            // shl is a mul
+            //     shl a,  b, 2
+            //  -> mul a, b, (1 << 2)
+            const Value* V0 = Op->getOperand(0);
+            const Value* V1 = Op->getOperand(1);
+            getSymExprOrConstant(V0, S0, C0);
+            getSymExprOrConstant(V1, S1, C1);
 
+            uint32_t shtAmtMask = getTypeMask(V->getType());
+            if (shtAmtMask == 0) // sanity
+                break;
+
+            if (!S1) {
+                C1 = (C1 & shtAmtMask);
+            }
+
+            if (!S0 && !S1) {
+                C = (C0 << C1);
+                return;
+            }
+            if (!S1) {
+                uint64_t tC = (1ull << C1);
+                S = mul(S0, tC);
+            }
+            break;
+        }
         case Instruction::BitCast:
         case Instruction::IntToPtr:
         case Instruction::PtrToInt:
@@ -551,15 +657,49 @@ void SymbolicEvaluation::getSymExprOrConstant(const Value* V, SymExpr*& S, int64
             }
             break;
         }
+        case Instruction::SExt:
+        case Instruction::ZExt:
+        {
+            SymCastInfo SCI = (opc == Instruction::SExt ? SYMCAST_SEXT : SYMCAST_ZEXT);
+            Value* V0 = Op->getOperand(0);
+            ValueSymInfo* VSI = getSymInfo(V0);
+            if (!VSI) {
+                // first time or it is a constant
+                getSymExprOrConstant(V0, S, C);
+                if (!S) {
+                    // By default, sext is used. If zext, recalculate C.
+                    if (SCI == SYMCAST_ZEXT) {
+                        uint32_t bits = (uint32_t)V0->getType()->getPrimitiveSizeInBits();
+                        uint64_t bitMask = maskTrailingOnes<uint64_t>(bits);
+                        C = (C & bitMask);
+                    }
+                    return;
+                }
+                VSI = getSymInfo(V0);
+                IGC_ASSERT(VSI);
+            }
+
+            // If V0 has been sext/zext before and this sext/zext isn't
+            // the same as before, bail out so V (input) is inevaluable.
+            if (VSI->CastInfo == SYMCAST_NOCAST || VSI->CastInfo == SCI) {
+                S = VSI->symExpr;
+                if (VSI->CastInfo == SYMCAST_NOCAST)
+                    VSI->CastInfo = SCI;
+            }
+            break;
+        }
+        default:
+            break;
         }
     }
 
-    // Unevaluable for V. Its symExpr is itself.
+    // Inevaluable for V. Its symExpr is itself. If V is a evaluable constant,
+    // the function should have returned already and won't reach this place.
     if (S == nullptr)
     {
-        SymProd* P = new (m_symEvaAllocator) SymProd();
-        SymTerm* T = new (m_symEvaAllocator) SymTerm();
-        SymExpr* E = new (m_symEvaAllocator) SymExpr();
+        SymProd* P = new (m_symProdAllocator.Allocate()) SymProd();
+        SymTerm* T = new (m_symTermAllocator.Allocate()) SymTerm();
+        SymExpr* E = new (m_symExprAllocator.Allocate()) SymExpr();
         P->Prod.push_back(V);
         T->Term = P;
         E->SymTerms.push_back(T);
@@ -612,7 +752,7 @@ int SymbolicEvaluation::cmp(const SymProd* T0, const SymProd* T1)
 SymExpr* SymbolicEvaluation::add(
     SymExpr* S0, SymExpr* S1, bool negateS1)
 {
-    SymExpr* Expr = new (m_symEvaAllocator) SymExpr();
+    SymExpr* Expr = new (m_symExprAllocator.Allocate()) SymExpr();
 
     int i0 = 0, i1 = 0;
     const int e0 = S0->SymTerms.size();
@@ -649,7 +789,7 @@ SymExpr* SymbolicEvaluation::add(
         }
 
         if (T != nullptr) {
-            SymTerm* newT = new (m_symEvaAllocator) SymTerm();
+            SymTerm* newT = new (m_symTermAllocator.Allocate()) SymTerm();
             Expr->SymTerms.push_back(newT);
             copy(newT, T);
             newT->Coeff = Coeff;
@@ -658,14 +798,14 @@ SymExpr* SymbolicEvaluation::add(
 
     while (i0 < e0)
     {
-        SymTerm* newT = new (m_symEvaAllocator) SymTerm();
+        SymTerm* newT = new (m_symTermAllocator.Allocate()) SymTerm();
         Expr->SymTerms.push_back(newT);
         copy(newT, S0->SymTerms[i0]);
         ++i0;
     }
     while (i1 < e1)
     {
-        SymTerm* newT = new (m_symEvaAllocator) SymTerm();
+        SymTerm* newT = new (m_symTermAllocator.Allocate()) SymTerm();
         Expr->SymTerms.push_back(newT);
         copy(newT, S1->SymTerms[i1]);
         if (negateS1) {
@@ -681,7 +821,7 @@ SymExpr* SymbolicEvaluation::add(
 
 SymExpr* SymbolicEvaluation::add(SymExpr* S, int64_t C)
 {
-    SymExpr* Expr = new (m_symEvaAllocator) SymExpr();
+    SymExpr* Expr = new (m_symExprAllocator.Allocate()) SymExpr();
     copy(Expr, S);
     Expr->ConstTerm += C;
     return Expr;
@@ -689,12 +829,12 @@ SymExpr* SymbolicEvaluation::add(SymExpr* S, int64_t C)
 
 SymExpr* SymbolicEvaluation::mul(SymExpr* S, int64_t C)
 {
-    SymExpr* Expr = new (m_symEvaAllocator) SymExpr();
+    SymExpr* Expr = new (m_symExprAllocator.Allocate()) SymExpr();
     if (C != 0)
     {
         for (int i = 0, e = S->SymTerms.size(); i < e; ++i)
         {
-            SymTerm* newT = new (m_symEvaAllocator) SymTerm();
+            SymTerm* newT = new (m_symTermAllocator.Allocate()) SymTerm();
             copy(newT, S->SymTerms[i]);
             newT->Coeff *= C;
             Expr->SymTerms.push_back(newT);
