@@ -86,10 +86,13 @@ namespace IGC {
                 }
             }
             // Figure out the intrinsic operands for texture & sampler
+            // & paired texture
+            llvm::Value* pPairedTextureValue = nullptr;
             llvm::Value* pTextureValue = nullptr;
             llvm::Value*pSamplerValue = nullptr;
             IGC::getTextureAndSamplerOperands(
                 pIntr,
+                pPairedTextureValue,
                 pTextureValue,
                 pSamplerValue);
 
@@ -103,6 +106,16 @@ namespace IGC {
                 }
             }
 
+            if (pPairedTextureValue && pPairedTextureValue->getType()->isPointerTy() &&
+                !m_WIAnalysis->isUniform(pPairedTextureValue))
+            {
+                // Check assumptions for texture:
+                if (m_forceUniformTexture || IsAssumedUniform(pPairedTextureValue))
+                {
+                    MakeUniformResourceOperand(pPairedTextureValue, &CI);
+                }
+            }
+
             if (pSamplerValue && pSamplerValue->getType()->isPointerTy() &&
                 !m_WIAnalysis->isUniform(pSamplerValue))
             {
