@@ -17,6 +17,7 @@ SPDX-License-Identifier: MIT
 #include <set>
 #include "Probe/Assertion.h"
 #include "llvmWrapper/IR/BasicBlock.h"
+#include "llvmWrapper/IR/Function.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -354,9 +355,7 @@ BasicBlock* Layout::getLastReturnBlock(Function& Func)
     // otherwise, return the last BB that has no succ;
     //     or nullptr if every BB has Succ (infinite looping)
     BasicBlock* noRetAndNoSucc = nullptr;  // for func that never returns
-    Function::BasicBlockListType& bblist = Func.getBasicBlockList();
-    for (Function::BasicBlockListType::reverse_iterator RI = bblist.rbegin(),
-        RE = bblist.rend();  RI != RE; ++RI)
+    for (auto RI = IGCLLVM::rbegin(&Func), RE = IGCLLVM::rend(&Func); RI != RE; ++RI)
     {
         BasicBlock* bb = &*RI;
         if (succ_begin(bb) == succ_end(bb))
@@ -546,10 +545,7 @@ void Layout::LayoutBlocks(Function& func, LoopInfo& LI)
                     {
                         // loop-header is not moved yet, so should be at the end
                         // use splice
-                        llvm::Function::BasicBlockListType& BBList = func.getBasicBlockList();
-                        BBList.splice(insp->getIterator(), BBList,
-                            LoopStart->getIterator(),
-                            hd->getIterator());
+                        IGCLLVM::splice(&func, insp->getIterator(), &func, LoopStart->getIterator(), hd->getIterator());
                         hd->moveBefore(LoopStart);
                     }
                     InsPos[PaHd] = hd;
@@ -570,8 +566,6 @@ void Layout::LayoutBlocks(Function& func, LoopInfo& LI)
     moveAtomicWrites2Loop(func, LI, false);
 
     // if function has a single exit, then the last block must be an exit
-    // comment this out due to infinite loop example in OCL
-    // IGC_ASSERT(PDT.getRootNode()->getBlock() == 0x0 || PDT.getRootNode()->getBlock() == &(func.getBasicBlockList().back()));
     // fix the loop-exit pattern, put break-blocks into the loop
     for (llvm::Function::iterator blkIter = func.begin(), blkEnd = func.end();
         blkIter != blkEnd; ++blkIter)
