@@ -1,28 +1,10 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
 #include "Compiler/CISACodeGen/messageEncoding.hpp"
 #include "Compiler/CISACodeGen/helper.h"
@@ -786,6 +768,56 @@ namespace IGC
         return messageDesc.Value;
     }
 
+    uint URBFence()
+    {
+        URBFenceGen12 messageDescriptor;
+        memset(&messageDescriptor, 0, sizeof(messageDescriptor));
+
+        messageDescriptor.DW0.All.URBOpcode = EU_URB_OPCODE_FENCE;
+        messageDescriptor.DW0.All.ResponseLength = 1;
+        messageDescriptor.DW0.All.MessageLength = 1;
+
+        return messageDescriptor.DW0.Value;
+    }
+
+    uint EOTGateway(const EU_GW_FENCE_PORTS fencePorts)
+    {
+        EOTMessageDescriptorGen12 messageDescriptor;
+        memset(&messageDescriptor, 0, sizeof(messageDescriptor));
+
+        // The only one subfunction is available so far.
+        messageDescriptor.DW0.All.EOTSubfunction = 0;
+        messageDescriptor.DW0.All.FenceDataPorts = fencePorts;
+        messageDescriptor.DW0.All.ResponseLength = 0;
+        messageDescriptor.DW0.All.MessageLength = 1;
+
+        return messageDescriptor.DW0.Value;
+    }
+
+    uint BindlessThreadDispatch(
+        const uint   messageLength,
+        const uint   SIMDMode,
+        const bool   IsTraceMessage,
+        const bool   IsRayQueryMessage)
+    {
+        // TODO: move this to vISA
+        IGC_ASSERT(messageLength > 0);
+        IGC_ASSERT(messageLength < 16);
+
+        TraceRayBTDMessageDescriptorGen12 messageDescriptor;
+        memset(&messageDescriptor, 0, sizeof(messageDescriptor));
+
+        messageDescriptor.DW0.All.SIMDMode = SIMDMode;
+        messageDescriptor.DW0.All.MessageType = IsTraceMessage ? 0x0 : 0x1;
+        // 'HeaderPresent' Must be programmed to 0
+        messageDescriptor.DW0.All.HeaderPresent = 0;
+        messageDescriptor.DW0.All.MessageLength = messageLength;
+        // isRayQueryMessage can be set only for TraceRayMessage
+        IGC_ASSERT(IsTraceMessage || !IsRayQueryMessage);
+        messageDescriptor.DW0.All.ResponseLength = IsRayQueryMessage ? 1 : 0;
+
+        return messageDescriptor.DW0.Value;
+    }
 
 
 } // namespace IGC
