@@ -23,8 +23,8 @@ using namespace llvm;
 using namespace IGC;
 
 // Register pass to igc-opt
-#define PASS_FLAG "igc-ConverMSAAPayloadTo16Bit"
-#define PASS_DESCRIPTION "Comvert normal MSAA intrinsics to 16 bit payload intrinsics"
+#define PASS_FLAG "igc-ConvertMSAAPayloadTo16Bit"
+#define PASS_DESCRIPTION "Convert normal MSAA intrinsics to 16 bit payload intrinsics"
 #define PASS_CFG_ONLY false
 #define PASS_ANALYSIS true
 IGC_INITIALIZE_PASS_BEGIN(ConvertMSAAPayloadTo16Bit, PASS_FLAG, PASS_DESCRIPTION, PASS_CFG_ONLY, PASS_ANALYSIS)
@@ -151,14 +151,15 @@ void ConvertMSAAPayloadTo16Bit::visitCallInst(CallInst& I)
                 uint32_t newLdmcsNumOfElements = (uint32_t)cast<IGCLLVM::FixedVectorType>(new_mcs_call->getType())->getNumElements();
 
                 // vec of 16bit ints to vec of 32bit ints
-                Type* newLdmcsVecType = IGCLLVM::FixedVectorType::get(m_builder->getInt32Ty(), newLdmcsNumOfElements);
-                Value* ldmcsExtendedToInt32 = m_builder->CreateSExt(new_mcs_call, newLdmcsVecType);
+                Type* newLdmcsVecType = IGCLLVM::FixedVectorType::get(m_builder->getInt32Ty(), newLdmcsNumOfElements / 2);
+                Value* ldmcsExtendedToInt32 = m_builder->CreateBitCast(new_mcs_call, newLdmcsVecType);
+                uint32_t ldmcsExtendedToInt32NumOfElements = (uint32_t)cast<IGCLLVM::FixedVectorType>(ldmcsExtendedToInt32->getType())->getNumElements();
 
                 // if ldmcs has fewer elements than new ldmcs, extend vector
                 Value* ldmcsInt32CorrectlySized;
-                if (newLdmcsNumOfElements != ldmcsNumOfElements)
+                if (ldmcsExtendedToInt32NumOfElements != ldmcsNumOfElements)
                 {
-                    IGC_ASSERT(newLdmcsNumOfElements * 2 >= ldmcsNumOfElements);
+                    IGC_ASSERT(ldmcsExtendedToInt32NumOfElements * 2 >= ldmcsNumOfElements);
                     SmallVector<uint32_t, 4> maskVals;
                     for (uint i = 0; i < ldmcsNumOfElements; i++)
                     {
@@ -189,10 +190,8 @@ void ConvertMSAAPayloadTo16Bit::visitCallInst(CallInst& I)
 bool ConvertMSAAPayloadTo16Bit::runOnFunction(Function& F)
 {
     m_pCtxWrapper = &getAnalysis<CodeGenContextWrapper>();
-    CodeGenContext* cgCtx = m_pCtxWrapper->getCodeGenContext();
     IRBuilder<> builder(F.getContext());
     m_builder = &builder;
     visit(F);
-    DumpLLVMIR(cgCtx, "AfterMSAA16bitPayload");
     return true;
 }
