@@ -9,6 +9,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/DebugInfo/ScalarVISAModule.h"
 #include "Compiler/DebugInfo/Utils.h"
 #include "Compiler/Optimizer/OpenCLPasses/KernelArgs/KernelArgs.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryToSLM.hpp"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
 #include "common/debug/Debug.hpp"
@@ -111,15 +112,11 @@ unsigned ScalarVisaModule::getPrivateBaseReg() const
 
 int ScalarVisaModule::getPTOReg() const {
     IGC_ASSERT_MESSAGE(hasPTO(), "PTO instruction required");
-    Value* pValPTO = dyn_cast_or_null<Value>(m_perThreadOffset);
-    IGC_ASSERT_MESSAGE(pValPTO, "pValPTO error");
-    // At this point we expect only a register
-    CVariable* pVarPTO = GetSymbol(m_perThreadOffset, pValPTO);
 
-    IGC_ASSERT_MESSAGE(pVarPTO, "Per Thread Offset variable does not exist");
-    IGC_ASSERT_MESSAGE(pVarPTO->GetVarType() == EVARTYPE_GENERAL, "Unexpected VISA register type!");
+    IGC_ASSERT_MESSAGE(m_perThreadOffset, "Per Thread Offset variable does not exist");
+    IGC_ASSERT_MESSAGE(m_perThreadOffset->GetVarType() == EVARTYPE_GENERAL, "Unexpected VISA register type!");
 
-    int regPTO = getDeclarationID(pVarPTO, false);
+    int regPTO = getDeclarationID(m_perThreadOffset, false);
     return regPTO;
 }
 int ScalarVisaModule::getFPReg() const {
@@ -136,7 +133,9 @@ llvm::StringRef ScalarVisaModule::GetVISAFuncName() const
     return getFunction()->getName();
 }
 uint64_t ScalarVisaModule::getFPOffset() const {
-    return EmitPass::getFPOffset();
+    auto funcMDItr = m_pShader->m_ModuleMetadata->FuncMD.find(getFunction());
+    IGC_ASSERT(funcMDItr != m_pShader->m_ModuleMetadata->FuncMD.end());
+    return funcMDItr->second.prevFPOffset;
 }
 
 bool ScalarVisaModule::usesSlot1ScratchSpill() const {
@@ -347,7 +346,7 @@ const Argument* ScalarVisaModule::GetTracedArgument(const Value* pVal, bool isAd
 VISAVariableLocation
 ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
 {
-    const Value* pVal = nullptr;
+    Value* pVal = nullptr;
     MDNode* pNode = nullptr;
     bool isDbgDclInst = false;
     if (const DbgDeclareInst * pDbgAddrInst = dyn_cast<DbgDeclareInst>(pInst))
@@ -459,6 +458,9 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
                         index = m_pShader->m_pBtiLayout->GetTextureIndex(index);
                         IGC_ASSERT_MESSAGE(index < TEXTURE_REGISTER_NUM, "Bad texture index");
                         return VISAVariableLocation(TEXTURE_REGISTER_BEGIN + index, this);
+                    case BindlessUAVResourceType:
+                        IGC_ASSERT_MESSAGE(index < TEXTURE_REGISTER_NUM, "Bad texture index");
+                        return VISAVariableLocation(TEXTURE_REGISTER_BEGIN + index, this);
                     default:
                         IGC_ASSERT_MESSAGE(0, "Unknown texture resource");
                         return VISAVariableLocation(this);
@@ -468,9 +470,7 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
         }
     }
 
-    Value* pValue = const_cast<Value*>(pVal);
-
-    Type* pType = pValue->getType();
+    Type* pType = pVal->getType();
 
     if (isDbgDclInst)
     {
@@ -483,49 +483,30 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
         pType = IGCLLVM::getNonOpaquePtrEltTy(pType);
     }
 
-    bool isInSurface = false;
     bool isGlobalAddrSpace = false;
-    unsigned int surfaceReg = 0;
     if (pType->isPointerTy())
     {
         unsigned int addrSpace = pType->getPointerAddressSpace();
-        if (addrSpace == ADDRESS_SPACE_LOCAL)
-        {
-            isInSurface = true;
-            surfaceReg = TEXTURE_REGISTER_BEGIN + LOCAL_SURFACE_BTI;
-        }
         if (addrSpace == ADDRESS_SPACE_GLOBAL)
         {
             isGlobalAddrSpace = true;
         }
     }
 
-    if (pVal->getType()->isPointerTy())
+    // SLM global variable
+    if (isa<GlobalVariable>(pVal))
     {
-        unsigned int addrSpace = pVal->getType()->getPointerAddressSpace();
-        if (addrSpace == ADDRESS_SPACE_LOCAL)
-        {
-            isInSurface = true;
-            surfaceReg = TEXTURE_REGISTER_BEGIN + LOCAL_SURFACE_BTI;
-        }
-    }
-
-    if (isa<GlobalVariable>(pValue))
-    {
-        unsigned int offset = m_pShader->GetGlobalMappingValue(pValue);
-        if (isInSurface)
-        {
-            return VISAVariableLocation(surfaceReg, offset, false, isDbgDclInst, 0, false, this);
-        }
-        return VISAVariableLocation(offset, false, isDbgDclInst, 0, false, false, this);
+        unsigned int offset = m_pShader->GetSLMMappingValue(pVal);
+        offset |= VALID_LOCAL_HIGH_BITS;
+        return VISAVariableLocation(offset, true, this);
     }
 
     // At this point we expect only a register
     CVariable* pVar = nullptr;
-    auto globalSubCVar = m_pShader->GetGlobalCVar(pValue);
+    auto globalSubCVar = m_pShader->GetGlobalCVar(pVal);
 
     if (!globalSubCVar) {
-        pVar = m_pShader->GetDebugInfoData().getMapping(*pInst->getFunction(), pValue);
+        pVar = m_pShader->GetDebugInfoData().getMapping(*pInst->getFunction(), pVal);
         if (!pVar)
         {
             return VISAVariableLocation(this);
@@ -556,12 +537,6 @@ ScalarVisaModule::GetVariableLocation(const llvm::Instruction* pInst) const
         {
             vectorNumElements = 1;
         }
-
-        if (isInSurface)
-        {
-            return VISAVariableLocation(surfaceReg, reg, true /*isRegister*/, isDbgDclInst,
-                                        vectorNumElements, !pVar->IsUniform(), this);
-        }
         VISAVariableLocation genReg(reg, true /*isRegister*/, isDbgDclInst, vectorNumElements,
                                     !pVar->IsUniform(), isGlobalAddrSpace, this);
         // SIMD32 locations can't into one register. See VISAVariableLocation::m_locationSecondReg field description for more information
@@ -672,6 +647,7 @@ int ScalarVisaModule::getDeclarationID(CVariable* pVar, bool isSecondSimd32Instr
     int varId = isSecondSimd32Instruction ? 1 : 0;
     if (isSecondSimd32Instruction) {
         if (!((GetSIMDSize() == 32 && pVar->visaGenVariable[1] && !pVar->IsUniform()))) {
+            IGC_ASSERT_MESSAGE(0, "Cannot get 2nd variable in SIMD32 (?) mode");
             return -1; // Cannot get 2nd variable in SIMD32 (?) mode
         }
     }
