@@ -116,64 +116,5 @@ namespace Utils {
         }
         return nullptr;
     }
-
-    unsigned int GetSpecialDebugVariableHash(const std::string& name)
-    {
-        // Calculate hash index for OCL special debug variables:
-        // 0 for __ocl_dbg_gid0
-        // 1 for __ocl_dbg_gid1
-        // 2 for __ocl_dbg_gid2
-        // 3 for __ocl_dbg_lid0
-        // 4 for __ocl_dbg_lid1
-        // 5 for __ocl_dbg_lid2
-        // 6 for __ocl_dbg_grid0
-        // 7 for __ocl_dbg_grid1
-        // 8 for __ocl_dbg_grid2
-
-        unsigned int idx = 0;
-
-        // 0 for ocl_dbg_gid*, so at position size-4 check only 'l' for lid and 'r' for grid
-        size_t significantCharPos = name.size() - 4;
-        if (name.at(significantCharPos) == 'l')
-            idx = 3;
-        else if (name.at(significantCharPos) == 'r')
-            idx = 6;
-
-        idx += std::strtoul(name.substr(name.size() - 1).data(), nullptr, 0);
-
-        return idx;
-    }
-
-    bool IsSpecialDebugVariable(const std::string& name)
-    {
-        // Check for OCL special debug variable such as:
-        //   "__ocl_dbg_gid0"
-        //   "__ocl_dbg_gid1"
-        //   "__ocl_dbg_gid2"
-        // Assuming all OCL debug special variables starts with "__ocl_dbg" prefix.
-        return (name.find("__ocl_dbg", 0) == 0);
-    }
-
-    std::string GetSpecialVariableMetaName(const llvm::Instruction* inst)
-    {
-        // Check for OCL special debug variable such as in metadata
-        //   "preserve__ocl_dbg_gid0"
-        //   "preserve__ocl_dbg_gid1"
-        //   "preserve__ocl_dbg_gid2"
-        std::string names[__OCL_DBG_VARIABLES] =
-        {
-            "__ocl_dbg_gid0", "__ocl_dbg_gid1", "__ocl_dbg_gid2",
-            "__ocl_dbg_lid0", "__ocl_dbg_lid1", "__ocl_dbg_lid2",
-            "__ocl_dbg_grid0", "__ocl_dbg_grid1", "__ocl_dbg_grid2"
-        };
-
-        for (int i = 0; i < __OCL_DBG_VARIABLES; i++)
-        {
-            if (inst->getMetadata(names[i]))
-                return names[i];
-        }
-
-        return "";
-    }
 } // namespace Utils
 } // namespace IGC
