@@ -47,6 +47,7 @@ void initializeHoistFMulInLoopPassPass(llvm::PassRegistry&);
 void initializeHandleFRemInstructionsPass(llvm::PassRegistry&);
 void initializeDeSSAPass(llvm::PassRegistry&);
 void initializeDetectCSWalkOrderPass(llvm::PassRegistry&);
+void initializeSelectCSWalkOrderPass(llvm::PassRegistry&);
 void initializeDeviceEnqueueFuncsAnalysisPass(llvm::PassRegistry&);
 void initializeDeviceEnqueueFuncsResolutionPass(llvm::PassRegistry&);
 void initializeDisableLoopUnrollOnRetryPass(llvm::PassRegistry&);
@@ -193,6 +194,7 @@ void initializeTraceRayInlineLoweringPassPass(llvm::PassRegistry&);
 void initializeRayTracingAddressSpaceAAWrapperPassPass(llvm::PassRegistry&);
 void initializeRayTracingMemDSEPassPass(llvm::PassRegistry&);
 void initializeOverrideTMaxPassPass(llvm::PassRegistry&);
+void initializeAtomicPullSWWalkWrapperLoopImplementationPass(llvm::PassRegistry&);
 void initializeTranslateToProgrammableOffsetsPassPass(llvm::PassRegistry&);
 void initializeDynamicRayManagementPassPass(llvm::PassRegistry&);
 void initializeVectorBitCastOptPass(llvm::PassRegistry&);
@@ -263,3 +265,6 @@ void initializeBufferBoundsCheckingPatcherPass(llvm::PassRegistry&);
 void initializeBfloatFuncsResolutionPass(llvm::PassRegistry &);
 void initializeBarrierControlFlowOptimizationPass(llvm::PassRegistry&);
 void initializeMinimumValidAddressCheckingPass(llvm::PassRegistry&);
+void initializeLoopCountAnalysisPass(llvm::PassRegistry&);
+void initializeCollectLoopCountPass(llvm::PassRegistry&);
+void initializeRemoveLoopDependencyPass(llvm::PassRegistry&);
