@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -41,8 +41,8 @@ namespace IGC
 
         const llvm::DataLayout* m_DL;
         std::vector<llvm::Instruction*> m_instructionsToRemove;
-        llvm::IRBuilder<>* m_builder;
-        IGC::CodeGenContext* m_ctx;
+        llvm::IRBuilder<>* m_builder = nullptr;
+        IGC::CodeGenContext* m_ctx = nullptr;
     public:
 
         static char ID;
@@ -81,6 +81,7 @@ namespace IGC
         void visitBitCastInst(llvm::BitCastInst& I);
         void visitBasicBlock(llvm::BasicBlock& BB);
         void visitTruncInst(llvm::TruncInst&);
+        void visitUnaryInstruction(llvm::UnaryInstruction &I);
         void visitBinaryOperator(llvm::BinaryOperator& I);
         void visitAddrSpaceCastInst(llvm::AddrSpaceCastInst&);
 
@@ -91,7 +92,7 @@ namespace IGC
         llvm::Type* LegalAllocaType(llvm::Type* type) const;
         llvm::Type* LegalStructAllocaType(llvm::Type* type) const;
 
-        void RecursivelyChangePointerType(llvm::Instruction* oldPtr, llvm::Instruction* newPtr);
+        void RecursivelyChangePointerType(llvm::Instruction* oldPtr, llvm::Type* Ty, llvm::Instruction* newPtr);
         void PromoteFp16ToFp32OnGenSampleCall(llvm::CallInst& I);
         void PromoteInsertElement(llvm::Value* I, llvm::Value* newVec);
 
