@@ -146,10 +146,11 @@ TypeLegalizer::getTypeLegalizeAction(Type* Ty) const {
     // Floating point types always need native support; otherwise, they needs
     // software emulation.
     if (Ty->isFloatingPointTy()) {
-        // Only support 'half', 'float', and 'double'.
+        // Only support 'half', 'float', 'bfloat' and 'double'.
         if (Ty->isHalfTy()) return Legal;
         if (Ty->isFloatTy()) return Legal;
         if (Ty->isDoubleTy()) return Legal;
+        if (IGCLLVM::isBFloatTy(Ty)) return Legal;
 
         return SoftenFloat;
     }
@@ -218,8 +219,7 @@ TypeSeq* TypeLegalizer::getPromotedTypeSeq(Type* Ty, bool legalizeToScalar) {
     IGC_ASSERT(Ty->isIntOrIntVectorTy());
     IGC_ASSERT(getTypeLegalizeAction(Ty) == Promote);
 
-    TypeMapTy::iterator TMI; bool New = false;
-    std::tie(TMI, New) = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
+    auto [TMI, New] = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
     if (!New) {
         IGC_ASSERT(TMI->second.size() == 1);
         return &TMI->second;
@@ -249,8 +249,7 @@ TypeSeq* TypeLegalizer::getSoftenedTypeSeq(Type* Ty) {
     IGC_ASSERT(Ty->isFloatingPointTy());
     IGC_ASSERT(getTypeLegalizeAction(Ty) == SoftenFloat);
 
-    TypeMapTy::iterator TMI; bool New = false;
-    std::tie(TMI, New) = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
+    auto [TMI, New] = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
     if (!New) {
         IGC_ASSERT(TMI->second.size() == 1);
         return &TMI->second;
@@ -278,8 +277,7 @@ TypeSeq* TypeLegalizer::getExpandedTypeSeq(Type* Ty) {
     IGC_ASSERT(Ty->isIntegerTy());
     IGC_ASSERT(getTypeLegalizeAction(Ty) == Expand);
 
-    TypeMapTy::iterator TMI; bool New = false;
-    std::tie(TMI, New) = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
+    auto [TMI, New] = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
     if (!New)
         return &TMI->second;
 
@@ -302,8 +300,7 @@ TypeSeq* TypeLegalizer::getScalarizedTypeSeq(Type* Ty) {
     IGC_ASSERT(Ty->isVectorTy());
     IGC_ASSERT(getTypeLegalizeAction(Ty) == Scalarize);
 
-    TypeMapTy::iterator TMI; bool New = false;
-    std::tie(TMI, New) = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
+    auto [TMI, New] = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
     if (!New) {
       IGC_ASSERT(TMI->second.size() ==
                  cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements());
@@ -323,8 +320,7 @@ TypeSeq* TypeLegalizer::getElementizedTypeSeq(Type* Ty) {
     IGC_ASSERT(Ty->isAggregateType());
     IGC_ASSERT(getTypeLegalizeAction(Ty) == Elementize);
 
-    TypeMapTy::iterator TMI; bool New = false;
-    std::tie(TMI, New) = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
+    auto [TMI, New] = TypeMap.insert(std::make_pair(Ty, TypeSeq()));
     if (!New)
         return &TMI->second;
 
@@ -357,8 +353,7 @@ TypeLegalizer::getLegalizedValues(Value* V, bool isSigned) {
     if (C != nullptr)
         ValueMap.erase(V);
 
-    ValueMapTy::iterator VMI; bool New = false;
-    std::tie(VMI, New) = ValueMap.insert(std::make_pair(V, ValueSeq()));
+    auto [VMI, New] = ValueMap.insert(std::make_pair(V, ValueSeq()));
     if (!New)
         return std::make_pair(&VMI->second, Act);
 
@@ -403,8 +398,7 @@ TypeLegalizer::getLegalizedValues(Value* V, bool isSigned) {
 void
 TypeLegalizer::setLegalizedValues(Value* OVal,
     ArrayRef<Value*> LegalizedVals) {
-    ValueMapTy::iterator VMI; bool New = false;
-    std::tie(VMI, New) = ValueMap.insert(std::make_pair(OVal, ValueSeq()));
+    auto [VMI, New] = ValueMap.insert(std::make_pair(OVal, ValueSeq()));
     IGC_ASSERT(New);
 
     for (auto* V : LegalizedVals)
@@ -521,10 +515,8 @@ bool TypeLegalizer::legalizeArguments(Function& F) {
     // - llvm.val.trim.<OriginalTy>(...)
     for (auto AI = F.arg_begin(), AE = F.arg_end(); AI != AE; ++AI) {
         Argument* Arg = &(*AI);
-        TypeSeq* TySeq = nullptr;
-        LegalizeAction Act = Legal;
         Type* Ty = Arg->getType();
-        std::tie(TySeq, Act) = getLegalizedTypes(Ty);
+        auto [TySeq, Act] = getLegalizedTypes(Ty);
 
         switch (Act) {
         case Legal:
@@ -582,8 +574,7 @@ bool TypeLegalizer::preparePHIs(Function& F) {
             if (!PN) break;
 
             Type* Ty = PN->getType();
-            TypeSeq* TySeq; LegalizeAction Act;
-            std::tie(TySeq, Act) = getLegalizedTypes(Ty);
+            auto [TySeq, Act] = getLegalizedTypes(Ty);
             switch (Act) {
             case Legal:
                 // Skip legal PHI node.
