@@ -26,17 +26,14 @@ namespace IGC
     /// This pass imports built-in functions from source module to destination module.
     class BIImport : public llvm::ModulePass
     {
-    protected:
-        // Type used to hold a vector of Functions and augment it during traversal.
-        typedef std::vector<llvm::Function*>       TFunctionsVec;
-
     public:
         // Pass identification, replacement for typeid.
         static char ID;
 
+
+
         /// @brief Constructor
-        BIImport(std::unique_ptr<llvm::Module> pGenericModule = nullptr,
-            std::unique_ptr<llvm::Module> pSizeModule = nullptr);
+        BIImport();
 
         /// @brief analyses used
         virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const override
@@ -59,6 +56,9 @@ namespace IGC
         static void supportOldManglingSchemes(llvm::Module& M);
         static std::unique_ptr<llvm::Module> Construct(llvm::Module& M, CLElfLib::CElfReader* pElfReader, bool hasSizet);
     protected:
+
+        // Type used to hold a vector of Functions and augment it during traversal.
+        typedef std::vector<llvm::Function*>       TFunctionsVec;
         /// @brief Get all the functions called by given function.
         /// @param [IN] pFunc The given function.
         /// @param [OUT] calledFuncs The list of all functions called by pFunc.
@@ -75,8 +75,6 @@ namespace IGC
         /// @brief  Search through all builtin modules for the specified function.
         /// @param  funcName - name of func to search for.
         static llvm::Function* GetBuiltinFunction(llvm::StringRef funcName, llvm::Module* GenericModule);
-        llvm::Function* GetBuiltinFunction2(llvm::StringRef funcName) const;
-
         /// @brief  Read elf Header file that is constructed by Build Packager and write to a DenseMap.
         static void WriteElfHeaderToMap(llvm::DenseMap<llvm::StringRef, int>& Map, char* pData, size_t dataSize);
 
@@ -85,17 +83,11 @@ namespace IGC
 
         /// @brief Fix invalid bitcasts generated by older Clang
         void fixInvalidBitcasts(llvm::Module& M);
-
-    protected:
-        /// Builtin module - contains the source function definition to import
-        std::unique_ptr<llvm::Module> m_GenericModule;
-        std::unique_ptr<llvm::Module> m_SizeModule;
     };
 
 } // namespace IGC
 
-extern "C" llvm::ModulePass* createBuiltInImportPass(
-    std::unique_ptr<llvm::Module> pGenericModule, std::unique_ptr<llvm::Module> pSizeModule);
+extern "C" llvm::ModulePass* createBuiltInImportPass();
 
 namespace IGC
 {
