@@ -395,9 +395,9 @@ bool GatingSimilarSamples::runOnFunction(llvm::Function& F)
 
     if (IGC_GET_FLAG_VALUE(DisableGatingSimilarSamples))
         return false;
-    if (F.getBasicBlockList().size() != 1)
+    if (F.size() != 1)
         return false;
-    BB = &*F.getBasicBlockList().begin();
+    BB = &*F.begin();
 
     if (!checkAndSaveSimilarSampleInsts())
         return false;
