@@ -310,6 +310,8 @@ KernelArg::ArgType KernelArg::calcArgType(const ImplicitArg& arg) const
         return KernelArg::ArgType::IMPLICIT_SAMPLER_NORMALIZED;
     case ImplicitArg::SAMPLER_SNAP_WA:
         return KernelArg::ArgType::IMPLICIT_SAMPLER_SNAP_WA;
+    case ImplicitArg::INLINE_SAMPLER:
+        return KernelArg::ArgType::IMPLICIT_INLINE_SAMPLER;
     case ImplicitArg::FLAT_IMAGE_BASEOFFSET:
         return KernelArg::ArgType::IMPLICIT_FLAT_IMAGE_BASEOFFSET;
     case ImplicitArg::FLAT_IMAGE_HEIGHT:
@@ -817,6 +819,7 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::IMPLICIT_SAMPLER_ADDRESS,
             KernelArg::ArgType::IMPLICIT_SAMPLER_NORMALIZED,
             KernelArg::ArgType::IMPLICIT_SAMPLER_SNAP_WA,
+            KernelArg::ArgType::IMPLICIT_INLINE_SAMPLER,
             KernelArg::ArgType::IMPLICIT_FLAT_IMAGE_BASEOFFSET,
             KernelArg::ArgType::IMPLICIT_FLAT_IMAGE_HEIGHT,
             KernelArg::ArgType::IMPLICIT_FLAT_IMAGE_WIDTH,
@@ -943,6 +946,7 @@ KernelArgsOrder::KernelArgsOrder(InputType layout)
             KernelArg::ArgType::IMPLICIT_SAMPLER_ADDRESS,
             KernelArg::ArgType::IMPLICIT_SAMPLER_NORMALIZED,
             KernelArg::ArgType::IMPLICIT_SAMPLER_SNAP_WA,
+            KernelArg::ArgType::IMPLICIT_INLINE_SAMPLER,
             KernelArg::ArgType::IMPLICIT_FLAT_IMAGE_BASEOFFSET,
             KernelArg::ArgType::IMPLICIT_FLAT_IMAGE_HEIGHT,
             KernelArg::ArgType::IMPLICIT_FLAT_IMAGE_WIDTH,
