@@ -344,7 +344,7 @@ namespace IGC
                             VALUE_NAME(pAI->getName() + ".totalOffset"));
 
                     Value* cast = ConstantExpr::getAddrSpaceCast(slmVar, pAI->getType());
-                    Value* ptr = builder.CreateGEP(cast, totalOffset);
+                    Value* ptr = builder.CreateGEP(pAI->getAllocatedType(), cast, totalOffset);
 
                     pAI->replaceAllUsesWith(ptr);
                     pAI->eraseFromParent();
@@ -352,7 +352,7 @@ namespace IGC
                     // Add new SLM variable offset to MD.
                     LocalOffsetMD localOffset;
                     localOffset.m_Var = slmVar;
-                    localOffset.m_Offset = useAsPointerOnly(slmVar) ? (offset & 0xFFFF) : ((offset & 0xFFFF) | VALID_LOCAL_HIGH_BITS);
+                    localOffset.m_Offset = useAsPointerOnly(slmVar) ? (offset & LOW_BITS_MASK) : ((offset & LOW_BITS_MASK) | VALID_LOCAL_HIGH_BITS);
                     ModuleMD->FuncMD[F].localOffsets.push_back(localOffset);
 
                     // Update total SLM usage MD.
