@@ -21,10 +21,10 @@ namespace IGC
     // It is convenient to represent the null pointer as the zero
     // bit-pattern. However, SLM address 0 is legal, and we want to be able
     // to use it.
-    // To go around this, we use the fact only the low 16 bits ("low nibble")
-    // of SLM addresses are significant, and set all valid pointers to have
-    // a non-zero high nibble.
-    static inline const unsigned int VALID_LOCAL_HIGH_BITS = 0x10000000;
+    // To go around this, we set all valid pointers to have a non-zero high
+    // nibble.
+    constexpr unsigned int VALID_LOCAL_HIGH_BITS = 0x10000000;
+    constexpr unsigned int LOW_BITS_MASK = VALID_LOCAL_HIGH_BITS - 1;
 
     // Experimental pass to move private memory allocations to SLM where it's
     // profitable. The pass is able to handle Compute and OpenCL shader types.
