@@ -184,8 +184,8 @@ Description:
 Returns a pointer to the data structure which the RT hardware operates on.
 The RT Stack address is computed as:
     syncStackSize = sizeof(HitInfo)*2 + (sizeof(Ray) + sizeof(TravStack))*RTDispatchGlobals.maxBVHLevels;
-    syncBase = RTDispatchGlobals.rtMemBasePtr - (DSSID * NUM_SIMD_LANES_PER_DSS + StackID + 1)*syncStackSize; */
-/* Where DSSID is an index which uniquely identifies the DSS in the machine (across tiles), and StackID is compute as below:
+    syncBase = RTDispatchGlobals.rtMemBasePtr - (DSSID * NUM_SIMD_LANES_PER_DSS + StackID + 1)*syncStackSize;
+Where DSSID is an index which uniquely identifies the DSS in the machine (across tiles), and StackID is compute as below:
     With fused EUs (e.g. in DG2) :
       StackID[10:0] (msb to lsb) = (EUID[3:0]<<7) | (THREAD_ID[2:0]<<4) | SIMD_LANE_ID[3:0]
 
@@ -377,14 +377,14 @@ void ResolveOCLRaytracingBuiltins::handleInitRayQuery(llvm::CallInst& callInst)
     unsigned numArgs = IGCLLVM::getNumArgOperands(&callInst);
     IGC_ASSERT(numArgs == 5);
 
-    auto* allocaType = IGCLLVM::getNonOpaquePtrEltTy(callInst.getType());
+    auto* allocaType = IGCLLVM::getTypeByName(callInst.getModule(), "struct.intel_ray_query_opaque_t");
     auto* alloca = m_builder->CreateAlloca(allocaType);
 
     m_builder->SetInsertPoint(&callInst);
 
     auto storeToAlloca = [&](unsigned argIndex)
     {
-        auto ptr = m_builder->CreateGEP(alloca, { m_builder->getInt32(0), m_builder->getInt32(argIndex) });
+        auto ptr = m_builder->CreateGEP(alloca->getAllocatedType(), alloca, { m_builder->getInt32(0), m_builder->getInt32(argIndex) });
         auto arg = callInst.getOperand(argIndex);
         m_builder->CreateStore(arg, ptr);
     };
@@ -411,10 +411,12 @@ void ResolveOCLRaytracingBuiltins::handleUpdateRayQuery(llvm::CallInst& callInst
     IGC_ASSERT(numArgs == 6);
 
     Value* rayQuery = callInst.getOperand(0);
+    StructType* rayQueryTy = IGCLLVM::getTypeByName(callInst.getModule(), "struct.intel_ray_query_opaque_t");
+
 
     for (unsigned argIndex = 1; argIndex < numArgs; argIndex++)
     {
-        auto* ptr = m_builder->CreateGEP(rayQuery, { m_builder->getInt32(0), m_builder->getInt32(argIndex - 1) });
+        auto* ptr = m_builder->CreateGEP(rayQueryTy, rayQuery, { m_builder->getInt32(0), m_builder->getInt32(argIndex - 1) });
         Value* arg = callInst.getOperand(argIndex);
         m_builder->CreateStore(arg, ptr);
     }
@@ -426,7 +428,7 @@ void ResolveOCLRaytracingBuiltins::handleUpdateRayQuery(llvm::CallInst& callInst
 Handler for the following builtins:
 rtfence_t        __builtin_IB_intel_query_rt_fence(intel_ray_query_t);
 rtglobals_t      __builtin_IB_intel_query_rt_globals(intel_ray_query_t);
-global void*  __builtin_IB_intel_query_rt_stack(intel_ray_query_t);
+global RTStack*  __builtin_IB_intel_query_rt_stack(intel_ray_query_t);
 TraceRayCtrl     __builtin_IB_intel_query_ctrl(intel_ray_query_t);
 uint             __builtin_IB_intel_query_bvh_level(intel_ray_query_t);
 
@@ -455,9 +457,10 @@ void ResolveOCLRaytracingBuiltins::handleQuery(llvm::CallInst& callInst) {
 
     IGC_ASSERT(IGCLLVM::getNumArgOperands(&callInst) == 1);
     Value* rayQuery = callInst.getArgOperand(0);
+    StructType* rayQueryTy = IGCLLVM::getTypeByName(callInst.getModule(), "struct.intel_ray_query_opaque_t");
 
     unsigned argIndex = builtinToArgIndex.at(callInst.getCalledFunction()->getName().str());
-    auto* ptr = m_builder->CreateGEP(rayQuery, { m_builder->getInt32(0), m_builder->getInt32(argIndex) });
+    auto* ptr = m_builder->CreateGEP(rayQueryTy, rayQuery, { m_builder->getInt32(0), m_builder->getInt32(argIndex) });
     auto* queriedValue = m_builder->CreateLoad(cast<llvm::GetElementPtrInst>(ptr)->getResultElementType(), ptr);
 
     callInst.replaceAllUsesWith(queriedValue);
