@@ -9,6 +9,7 @@ SPDX-License-Identifier: MIT
 #pragma once
 
 #include "Compiler/MetaDataUtilsWrapper.h"
+#include "Compiler/CodeGenContextWrapper.hpp"
 
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/Pass.h>
@@ -19,14 +20,22 @@ namespace IGC {
 ///
 class StackOverflowDetectionPass : public llvm::ModulePass {
 public:
+  enum class Mode {
+    Initialize,
+    AnalyzeAndCleanup,
+    RemoveDummyCalls,
+  };
+
   /// @brief  Pass identification.
   static char ID;
 
   static constexpr const char* STACK_OVERFLOW_INIT_BUILTIN_NAME = "__stackoverflow_init";
   static constexpr const char* STACK_OVERFLOW_DETECTION_BUILTIN_NAME = "__stackoverflow_detection";
 
+  Mode mode = Mode::Initialize;
+
   StackOverflowDetectionPass();
-  ~StackOverflowDetectionPass() {}
+  StackOverflowDetectionPass(Mode mode_);
 
   virtual llvm::StringRef getPassName() const override {
     return "StackOverflowDetectionPass";
@@ -35,8 +44,12 @@ public:
   virtual bool runOnModule(llvm::Module &M) override;
 
   virtual void getAnalysisUsage(llvm::AnalysisUsage &AU) const override {
+    AU.addRequired<CodeGenContextWrapper>();
     AU.addRequired<MetaDataUtilsWrapper>();
   }
+
+  bool removeDummyCalls(llvm::Module &M);
+  bool removeCallsAndFunctionsIfNoStackCallsOrVLA(llvm::Module &M, IGCMD::MetaDataUtils *pMdUtils, ModuleMetaData *pModMD);
 };
 
 } // namespace IGC
