@@ -152,6 +152,12 @@ bool StatelessToStateful::runOnFunction(llvm::Function& F)
     MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
 
+    if (modMD->compOpt.OptDisable)
+    {
+        IGC_ASSERT_MESSAGE(0, "StatelessToStateful should be disabled for -O0!");
+        return false;
+    }
+
     // skip non-entry functions
     if (!isEntryFunc(pMdUtils, &F))
     {
@@ -804,6 +810,7 @@ void StatelessToStateful::promote()
                 IGC::EncodeAS4GFXResource(
                     *UndefValue::get(Type::getInt32Ty(m_Module->getContext())),
                     IGC::BINDLESS);
+            setPointerSizeTo32bit(statefullAddrspace, m_Module);
         }
         else
         {
@@ -812,7 +819,7 @@ void StatelessToStateful::promote()
             // If the support for dynamic BTIs allocation is disabled, then BTIs are pre-assigned
             // in ResourceAllocator pass for all resources independently whether they are
             // accessed through stateful addressing model or not.
-            if (ctx->platform.supportDynamicBTIsAllocation())
+            if (ctx->platform.supportDynamicBTIsAllocation() && ctx->enableZEBinary())
             {
                 argAlloc->type = ResourceTypeEnum::UAVResourceType;
                 argAlloc->indexType = resAllocMD->uavsNumType + bufferPos;
