@@ -60,15 +60,14 @@ namespace IGC
         void mediaBlockRead(llvm::CallInst& CI);
         void mediaBlockWrite(llvm::CallInst& CI);
 
-        void simdBlockRead(llvm::CallInst& CI);
-        void simdBlockWrite(llvm::CallInst& CI);
+        void simdBlockRead(llvm::CallInst& CI, bool hasCacheControls = false);
+        void simdBlockWrite(llvm::CallInst& CI, bool hasCacheControls = false);
 
         void pushMediaBlockArgs(llvm::SmallVector<llvm::Value*, 5> & args, llvm::CallInst& CI);
 
         void CheckMediaBlockInstError(llvm::GenIntrinsicInst* inst, bool isRead);
 
         void subGroupArithmetic(llvm::CallInst& CI, WaveOps op, GroupOpType groupType);
-        void subGroup2DBlockOperation(llvm::CallInst& CI, llvm::StringRef funcName, bool isRead);
 
         static const llvm::StringRef SUB_GROUP_BARRIER;
         static const llvm::StringRef GET_MAX_SUB_GROUP_SIZE;
@@ -215,9 +214,6 @@ namespace IGC
         static const llvm::StringRef SUB_GROUP_SCAN;
         static const llvm::StringRef SUB_GROUP_CLUSTERED_REDUCE;
 
-        static const llvm::StringRef SUBGROUP_BLOCK_READ;
-        static const llvm::StringRef SUBGROUP_BLOCK_WRITE;
-
     private:
         /// @brief  Container for instructions to be deleted after visiting a function.
         llvm::SmallVector<llvm::Instruction*, 16>  m_instsToDelete;
@@ -229,9 +225,9 @@ namespace IGC
         static const std::array<std::pair<std::string, WaveOps>, 13> m_spvOpToWaveOpMap;
 
         /// @brief  Indicates if the pass changed the processed function
-        bool m_changed;
+        bool m_changed{};
 
-        CodeGenContext* m_pCtx;
+        CodeGenContext* m_pCtx = nullptr;
 
         /// @brief examine metadata for intel_reqd_sub_group_size
         int32_t GetSIMDSize(llvm::Function* F);
