@@ -234,8 +234,29 @@ namespace packed
 // align struct member offset to be divisible by the size of the member. It implicates
 // inserting an additional padding member in some cases.
 static_assert(sizeof(packed::implicit_args) == sizeof(implicit_args), "Implicit args struct is not properly aligned!");
+static_assert(offsetof(packed::implicit_args, struct_size) == offsetof(implicit_args, struct_size), "Implicit args struct is not properly aligned for struct_size member!");
+static_assert(offsetof(packed::implicit_args, struct_version) == offsetof(implicit_args, struct_version), "Implicit args struct is not properly aligned for struct_version member!");
+static_assert(offsetof(packed::implicit_args, num_work_dim) == offsetof(implicit_args, num_work_dim), "Implicit args struct is not properly aligned for num_work_dim member!");
+static_assert(offsetof(packed::implicit_args, simd_width) == offsetof(implicit_args, simd_width), "Implicit args struct is not properly aligned for simd_width member!");
+static_assert(offsetof(packed::implicit_args, local_size_x) == offsetof(implicit_args, local_size_x), "Implicit args struct is not properly aligned for local_size_x member!");
+static_assert(offsetof(packed::implicit_args, local_size_y) == offsetof(implicit_args, local_size_y), "Implicit args struct is not properly aligned for local_size_y member!");
+static_assert(offsetof(packed::implicit_args, local_size_z) == offsetof(implicit_args, local_size_z), "Implicit args struct is not properly aligned for local_size_z member!");
+static_assert(offsetof(packed::implicit_args, global_size_x) == offsetof(implicit_args, global_size_x), "Implicit args struct is not properly aligned for global_size_x member!");
+static_assert(offsetof(packed::implicit_args, global_size_y) == offsetof(implicit_args, global_size_y), "Implicit args struct is not properly aligned for global_size_y member!");
+static_assert(offsetof(packed::implicit_args, global_size_z) == offsetof(implicit_args, global_size_z), "Implicit args struct is not properly aligned for global_size_z member!");
+static_assert(offsetof(packed::implicit_args, printf_buffer_ptr) == offsetof(implicit_args, printf_buffer_ptr), "Implicit args struct is not properly aligned for printf_buffer_ptr member!");
+static_assert(offsetof(packed::implicit_args, global_offset_x) == offsetof(implicit_args, global_offset_x), "Implicit args struct is not properly aligned for global_offset_x member!");
+static_assert(offsetof(packed::implicit_args, global_offset_y) == offsetof(implicit_args, global_offset_y), "Implicit args struct is not properly aligned for global_offset_y member!");
+static_assert(offsetof(packed::implicit_args, global_offset_z) == offsetof(implicit_args, global_offset_z), "Implicit args struct is not properly aligned for global_offset_z member!");
+static_assert(offsetof(packed::implicit_args, local_id_table_ptr) == offsetof(implicit_args, local_id_table_ptr), "Implicit args struct is not properly aligned for local_id_table_ptr member!");
+static_assert(offsetof(packed::implicit_args, group_count_x) == offsetof(implicit_args, group_count_x), "Implicit args struct is not properly aligned for group_count_x member!");
+static_assert(offsetof(packed::implicit_args, group_count_y) == offsetof(implicit_args, group_count_y), "Implicit args struct is not properly aligned for group_count_y member!");
+static_assert(offsetof(packed::implicit_args, group_count_z) == offsetof(implicit_args, group_count_z), "Implicit args struct is not properly aligned for group_count_z member!");
+static_assert(offsetof(packed::implicit_args, padding0) == offsetof(implicit_args, padding0), "Implicit args struct is not properly aligned for padding0 member!");
+static_assert(offsetof(packed::implicit_args, rt_global_buffer_ptr) == offsetof(implicit_args, rt_global_buffer_ptr), "Implicit args struct is not properly aligned for rt_global_buffer_ptr member!");
+static_assert(offsetof(packed::implicit_args, assert_buffer_ptr) == offsetof(implicit_args, assert_buffer_ptr), "Implicit args struct is not properly aligned for assert_buffer_ptr member!");
 
-// Structure of side buffer generated by NEO:
+    // Structure of side buffer generated by NEO:
 //struct implicit_args {
 //    uint8_t struct_size;
 //    uint8_t struct_version;
@@ -336,7 +357,7 @@ llvm::Value* LowerImplicitArgIntrinsics::BuildLoadInst(llvm::CallInst& CI, unsig
     auto LoadType = IGCLLVM::FixedVectorType::get(ElemType, LoadByteSize / ElemByteSize);
     auto PtrType = PointerType::get(LoadType, AddrSpace);
     auto BitCast = Builder.CreateBitCast(IntToPtr, PtrType);
-    auto LoadInst = Builder.CreateLoad(BitCast);
+    auto LoadInst = Builder.CreateLoad(LoadType, BitCast);
     LoadInst->setAlignment(IGCLLVM::getCorrectAlign(ElemByteSize));
     LoadedData = LoadInst;
 
@@ -829,7 +850,7 @@ void LowerImplicitArgIntrinsics::visitCallInst(CallInst& CI)
             // Load data
             auto Int16Ptr = Type::getInt16PtrTy(F->getContext(), ADDRESS_SPACE_GLOBAL);
             auto Addr = Builder.CreateIntToPtr(Result, Int16Ptr);
-            auto LoadInst = Builder.CreateLoad(Addr);
+            auto LoadInst = Builder.CreateLoad(Builder.getInt16Ty(), Addr);
             auto Trunc = Builder.CreateZExtOrBitCast(LoadInst, CI.getType());
             V = Trunc;
             break;
