@@ -36,10 +36,11 @@ namespace IGC
         EMU_I64DIVREM = 0x1,      // bit 0: original emulation lib, mostly i64 div/rem
         EMU_DP = 0x2,             // bit 1: IEEE-compliant double emulation (+-*/,cmp,convert,etc)
         EMU_DP_DIV_SQRT = 0x4,    // bit 2: IEEE-compliant double emulation for div and sqrt (EMU_DP subset)
-        EMU_SP_DIV = 0x8,         // bit 3: IEEE-complaint float div emulation (float)
-        EMU_I32DIVREM = 0x10,     // bit 4: emulation lib for i32 div/rem
-        EMU_I32DIVREM_SP = 0x20,  // bit 5: emulation lib for i32 div/rem using fp32
-        EMU_FP64_FP16_CONV = 0x40 // bit 6: fp64 to fp32 conversion
+        EMU_DP_CONV = 0x8,        // bit 3: IEEE-compliant double emulation for conversions (EMU_DP subset)
+        EMU_SP_DIV = 0x10,        // bit 4: IEEE-complaint float div emulation (float)
+        EMU_I32DIVREM = 0x20,     // bit 5: emulation lib for i32 div/rem
+        EMU_I32DIVREM_SP = 0x40,  // bit 6: emulation lib for i32 div/rem using fp32
+        EMU_FP64_FP16_CONV = 0x80 // bit 7: fp64 to fp32 conversion
     };
 
     class PreCompiledFuncImport : public llvm::ModulePass, public llvm::InstVisitor<PreCompiledFuncImport>
@@ -184,6 +185,42 @@ namespace IGC
         static void checkAndSetEnableSubroutine(CodeGenContext* CGCtx);
 
     private:
+
+        // Data about imported function.
+        struct ImportedFunction
+        {
+        public:
+            enum EmuType
+            {
+                INT64,
+                FASTDP, // faster DP div/rem for platforms without MADM instruction
+                SLOWDP,
+                OTHER
+            };
+
+            ImportedFunction(llvm::Function* F);
+
+            bool isSlowDPEmuFunc() { return type == SLOWDP; }
+
+            void updateUses();
+
+            bool operator==(const llvm::Function* F) {
+                return this->F == F;
+            }
+
+            static ImportedFunction copy(ImportedFunction& other);
+            static bool compare(ImportedFunction& L, ImportedFunction& R);
+
+            llvm::Function* F;
+            EmuType type;
+            unsigned funcInstructions;
+            unsigned totalInstructions;
+
+        private:
+            ImportedFunction(llvm::Function* F, EmuType type, unsigned funcInstructions, unsigned totalInstructions)
+                : F(F), type(type), funcInstructions(funcInstructions), totalInstructions(totalInstructions) {}
+        };
+
         void processDivide(llvm::BinaryOperator& inst, EmulatedFunctions function);
         void getInt64DivideEmuType(EmulatedFunctions function, unsigned int elementIndex, const char*& functionName, LibraryModules& module);
         llvm::BinaryOperator* upcastTo32Bit(llvm::BinaryOperator* I);
@@ -220,16 +257,18 @@ namespace IGC
         // Remove llvm.module.flags metadata before linking
         void removeLLVMModuleFlag(llvm::Module* M);
 
+        ImportedFunction createInlinedCopy(ImportedFunction& IF, unsigned calls);
+
         // Check if subroutine call is needed and set it if so.
         void checkAndSetEnableSubroutine();
-        CodeGenContext* m_pCtx;
-        int m_enableCallForEmulation; // subroutine/stackcall
+        CodeGenContext* m_pCtx = nullptr;
+        int m_enableCallForEmulation{}; // subroutine/stackcall
 
-        IGCMD::MetaDataUtils* m_pMdUtils;
-        llvm::Module* m_pModule;
+        IGCMD::MetaDataUtils* m_pMdUtils = nullptr;
+        llvm::Module* m_pModule = nullptr;
 
         /// @brief  Indicates if the pass changed the processed function
-        bool m_changed;
+        bool m_changed{};
 
         const static char* m_Int64DivRemFunctionNames[NUM_FUNCTIONS][NUM_TYPES];
         const static char* m_Int64SpDivRemFunctionNames[NUM_FUNCTIONS][NUM_TYPES];
@@ -240,12 +279,12 @@ namespace IGC
         uint32_t m_emuKind;
         bool isDPEmu() const { return (m_emuKind & EmuKind::EMU_DP) > 0; }
         bool isDPDivSqrtEmu() const { return (m_emuKind & EmuKind::EMU_DP_DIV_SQRT) > 0; }
+        bool isDPConvEmu() const { return (m_emuKind & EmuKind::EMU_DP_CONV) > 0; }
         bool isI64DivRem() const { return (m_emuKind & EmuKind::EMU_I64DIVREM) > 0; }
         bool isI32DivRem() const { return (m_emuKind & EmuKind::EMU_I32DIVREM) > 0; }
         bool isI32DivRemSP() const { return (m_emuKind & EmuKind::EMU_I32DIVREM_SP) > 0; }
         bool isSPDiv() const { return (m_emuKind & EmuKind::EMU_SP_DIV) > 0; }
         bool isFP64toFP16() const { return (m_emuKind & EmuKind::EMU_FP64_FP16_CONV) > 0; }
-        bool isDPConvFunc(llvm::Function* F) const;
 
         bool m_libModuleToBeImported[NUM_LIBMODS];
         bool m_libModuleAlreadyImported[NUM_LIBMODS];
