@@ -29,7 +29,7 @@ namespace IGC
 
         ~SPIRMetaDataTranslation() {}
 
-        void WarpFunctionMetadata(llvm::Module& M);
+        void translateKernelMetadataIntoOpenCLKernelsMD(llvm::Module& M);
         bool runOnModule(llvm::Module& M) override;
 
         virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const override
