@@ -48,8 +48,10 @@ llvm_config.add_tool_substitutions(tools, tool_dirs)
 # opt CLI changes between LLVM 10 and 11.
 # FIXME: Remove once older-than-11 LLVM versions go out of use.
 if int(config.llvm_version) < 11:
+  config.substitutions.append(('%LLVM_11_CHECK_PREFIX%', 'CHECK-PRE-LLVM-11'))
   config.substitutions.append(('%enable-basic-aa%', '-basicaa'))
 else:
+  config.substitutions.append(('%LLVM_11_CHECK_PREFIX%', 'CHECK-LLVM-11-PLUS'))
   config.substitutions.append(('%enable-basic-aa%', '--basic-aa'))
 # Add LLVM version-dependent check prefixes.
 # FIXME: Remove altogether after unifying all supported LLVM versions at 14+.
@@ -58,11 +60,6 @@ if int(config.llvm_version) < 14:
 else:
   config.substitutions.append(('%LLVM_DEPENDENT_CHECK_PREFIX%', 'CHECK-LLVM-14-PLUS'))
   config.available_features.add('llvm-14-plus')
-if int(config.llvm_version) < 10:
-  config.substitutions.append(('%LLVM_10_CHECK_PREFIX%', 'CHECK-PRE-LLVM-10'))
-else:
-  config.substitutions.append(('%LLVM_10_CHECK_PREFIX%', 'CHECK-LLVM-10-PLUS'))
-  config.available_features.add('llvm-10-plus')
 
 if config.use_khronos_spirv_translator_in_sc == "1":
   config.substitutions.append(('%SPV_CHECK_PREFIX%', 'CHECK-KHR'))
@@ -73,3 +70,5 @@ else:
 if not config.regkeys_disabled:
   config.available_features.add('regkeys')
 
+  config.substitutions.append(('%RT_CHECK_PREFIX%', 'CHECK-RT'))
+
