@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -37,9 +37,9 @@ See LICENSE.TXT for details.
 #include "common/LLVMWarningsPop.hpp"
 // clang-format on
 
+#include "StreamEmitter.hpp"
 #include "DIE.hpp"
 #include "DwarfDebug.hpp"
-#include "StreamEmitter.hpp"
 
 #include "Probe/Assertion.h"
 
@@ -373,10 +373,11 @@ class VISAMCCodeEmitter : public MCCodeEmitter {
     IGC_ASSERT_EXIT_MESSAGE(0, "Unimplemented");
   }
 
-  void operator=(const VISAMCCodeEmitter &) {
-    // TODO: implement this
-    IGC_ASSERT_EXIT_MESSAGE(0, "Unimplemented");
-  }
+public:
+  VISAMCCodeEmitter() = default;
+  ~VISAMCCodeEmitter() = default;
+  VISAMCCodeEmitter(const VISAMCCodeEmitter &) = delete;
+  VISAMCCodeEmitter &operator=(const VISAMCCodeEmitter &) = delete;
 };
 
 } // namespace IGC
@@ -707,7 +708,7 @@ void StreamEmitter::EmitSymbolValue(const MCSymbol *pSym, unsigned size,
 void StreamEmitter::EmitSectionOffset(const MCSymbol *pLabel,
                                       const MCSymbol *pSectionLabel) const {
   // Get the section that we're referring to, based on pSectionLabel.
-  const MCSection &section = pSectionLabel->getSection();
+  [[maybe_unused]] const MCSection &section = pSectionLabel->getSection();
 
   // If pLabel has already been emitted, verify that it is in the same section
   // as section label for sanity.
@@ -801,8 +802,7 @@ void StreamEmitter::verifyRegisterLocationSize(const IGC::DbgVariable &VarVal,
 
   auto *DbgInst = VarVal.getDbgInst();
   IGC_ASSERT(DbgInst);
-  Value *IRLoc = IGCLLVM::getVariableLocation(DbgInst);
-  auto *Ty = IRLoc->getType();
+  auto *Ty = DbgInst->getType();
   IGC_ASSERT(Ty->isSingleValueType());
 
   if (Ty->isPointerTy())
@@ -814,7 +814,7 @@ void StreamEmitter::verifyRegisterLocationSize(const IGC::DbgVariable &VarVal,
     return;
   }
   DiagnosticBuff Diag;
-  auto DwarfTypeSize = VarVal.getBasicSize(&DD);
+  auto DwarfTypeSize = DwarfDebug::getBaseTypeSize(VarVal.getType());
   if (DwarfTypeSize != ExpectedSize) {
     Diag.out() << "ValidationFailure [regLocSize] -- DWARF Type Size: "
                << DwarfTypeSize << ", expected: " << ExpectedSize << "\n";
