@@ -175,13 +175,6 @@ unsigned int VISAModule::GetVisaOffset(const llvm::Instruction *pInst) const {
   return itr->second.m_offset;
 }
 
-unsigned int VISAModule::GetVisaSize(const llvm::Instruction *pInst) const {
-  InstInfoMap::const_iterator itr = m_instInfoMap.find(pInst);
-  IGC_ASSERT_MESSAGE(itr != m_instInfoMap.end(), "Invalid Instruction");
-  IGC_ASSERT_MESSAGE(itr->second.m_size != INVALID_SIZE, "Invalid Size");
-  return itr->second.m_size;
-}
-
 const Module *VISAModule::GetModule() const { return m_Func->getParent(); }
 
 const Function *VISAModule::GetEntryFunction() const { return m_Func; }
@@ -295,7 +288,7 @@ VISAModule::getAllCallerSave(const VISAObjectDebugInfo &VDI,
       if (startRange < callerSaveIp) {
         callerRestoreIp = std::min<uint64_t>(endRange, callerRestoreIp);
         // Variable is live over stack call function.
-        for (auto callerSaveReg : (*callerSaveStartIt).data) {
+        for (const auto &callerSaveReg : (*callerSaveStartIt).data) {
           // startRegNum is saved to caller save area around the stack call.
           if ((callerSaveReg.srcRegOff / getGRFSizeInBytes()) == startRegNum) {
             // Emit caller save/restore only if %ip is within range
@@ -424,11 +417,12 @@ VISAModule::getGenISARange(const VISAObjectDebugInfo &VDI,
       start = getNextInst(start);
       continue;
     }
+    IGC_ASSERT_MESSAGE(itr->second.m_size != INVALID_SIZE, "Invalid Size");
 
     auto startVISAOffset = itr->second.m_offset;
     // VISASize indicated # of VISA insts emitted for this
     // LLVM IR inst
-    auto VISASize = GetVisaSize(start);
+    auto VISASize = itr->second.m_size;
 
     for (unsigned int i = 0; i != VISASize; i++) {
       auto VISAIndex = startVISAOffset + i;
@@ -548,14 +542,3 @@ bool VISAVariableLocation::IsTexture() const {
     return true;
   return false;
 }
-
-bool VISAVariableLocation::IsSLM() const {
-  if (!HasSurface())
-    return false;
-
-  auto surface = GetSurface();
-  if (surface ==
-      VISAModule::LOCAL_SURFACE_BTI + VISAModule::TEXTURE_REGISTER_BEGIN)
-    return true;
-  return false;
-}
