@@ -663,6 +663,41 @@ public:
         return hasImmediateOffsets() ? getOperand(operandCoordIndex) : nullptr;
     }
 
+    bool hasDynamicOffsets() const
+    {
+        for (unsigned int i = 0; i < 3; ++i)
+        {
+            Value* offset = getImmediateOffsetsValue(i);
+            if (offset && !isa<Constant>(offset))
+            {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    bool hasImmediateOffsetsInS3Range() const
+    {
+        constexpr int64_t cMinImmediateOffset = -8;
+        constexpr int64_t cMaxImmediateOffset = 7;
+        for (unsigned int i = 0; i < 3; ++i)
+        {
+            Value* offset = getImmediateOffsetsValue(i);
+            if (!offset)
+            {
+                continue;
+            }
+            ConstantInt* imm = dyn_cast<ConstantInt>(offset);
+            if (!imm ||
+                imm->getSExtValue() < cMinImmediateOffset ||
+                imm->getSExtValue() > cMaxImmediateOffset)
+            {
+                return false;
+            }
+        }
+        return true;
+    }
+
     inline Value* getSamplerValue() const
     {
         unsigned int index = getSamplerIndex();
@@ -1526,6 +1561,19 @@ public:
     }
 };
 
+class GlobalQueuePointerIntrinsic : public GenIntrinsicInst {
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst* I) {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_GlobalQueuePointer;
+    }
+
+    static inline bool classof(const Value* V) {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+};
+
 class SWStackPtrIntrinsic : public GenIntrinsicInst {
 public:
     // Methods for support type inquiry through isa, cast, and dyn_cast:
