@@ -20,14 +20,39 @@ SPDX-License-Identifier: MIT
 namespace IGC
 {
 
-static llvm::GenISAIntrinsic::ID GetID(const llvm::Function& func)
+static llvm::GenISAIntrinsic::ID GetID(const llvm::Function& func, bool useContextWrapper)
 {
-    LLVMContextWrapper& ctxWrapper = static_cast<LLVMContextWrapper&>(func.getContext());
-    LLVMContextWrapper::SafeIntrinsicIDCacheTy& SafeIntrinsicIdCache = ctxWrapper.m_SafeIntrinsicIDCache;
+    if (useContextWrapper)
+    {
+        LLVMContextWrapper& ctxWrapper = static_cast<LLVMContextWrapper&>(func.getContext());
+        LLVMContextWrapper::SafeIntrinsicIDCacheTy& SafeIntrinsicIdCache = ctxWrapper.m_SafeIntrinsicIDCache;
 
-    //If you do not find the function ptr as key corresponding to the GenISAIntrinsic::ID add the new key
-    auto it = SafeIntrinsicIdCache.find(&func);
-    if (it == SafeIntrinsicIdCache.end()) {
+        //If you do not find the function ptr as key corresponding to the GenISAIntrinsic::ID add the new key
+        auto it = SafeIntrinsicIdCache.find(&func);
+        if (it == SafeIntrinsicIdCache.end()) {
+            llvm::GenISAIntrinsic::ID id = llvm::GenISAIntrinsic::ID::no_intrinsic;
+            const llvm::ValueName* const pValueName = func.getValueName();
+            if (nullptr != pValueName)
+            {
+                llvm::StringRef prefix = GetIntrinsicPrefixName();
+                llvm::StringRef Name = pValueName->getKey();
+                if (Name.size() > prefix.size() && Name.startswith(prefix))
+                {
+                    id = static_cast<llvm::GenISAIntrinsic::ID>(LookupIntrinsicId(Name.data()));
+                    SafeIntrinsicIdCache[&func] = static_cast<uint32_t>(id);
+                }
+            }
+            return id;
+        }
+        else
+        {
+            // If you have an entry for the function ptr corresponding to the GenISAIntrinsic::ID return it back,
+            //instead of going through a lengthy look-up.
+            return (static_cast<llvm::GenISAIntrinsic::ID>(it->second));
+        }
+    }
+    else
+    {
         llvm::GenISAIntrinsic::ID id = llvm::GenISAIntrinsic::ID::no_intrinsic;
         const llvm::ValueName* const pValueName = func.getValueName();
         if (nullptr != pValueName)
@@ -37,17 +62,10 @@ static llvm::GenISAIntrinsic::ID GetID(const llvm::Function& func)
             if (Name.size() > prefix.size() && Name.startswith(prefix))
             {
                 id = static_cast<llvm::GenISAIntrinsic::ID>(LookupIntrinsicId(Name.data()));
-                SafeIntrinsicIdCache[&func] = static_cast<uint32_t>(id);
             }
         }
         return id;
     }
-    else
-    {
-        // If you have an entry for the function ptr corresponding to the GenISAIntrinsic::ID return it back,
-        //instead of going through a lengthy look-up.
-        return (static_cast<llvm::GenISAIntrinsic::ID>(it->second));
-    }
 }
 
 } // namespace IGC
@@ -72,11 +90,11 @@ Function* getDeclaration(Module* M, ID id, ArrayRef<Type*> OverloadedTys /*= Non
     return IGC::GetDeclaration(M, id, OverloadedTys);
 }
 
-ID getIntrinsicID(const Function* F)
+ID getIntrinsicID(const Function* F, bool useContextWrapper /*= true*/)
 {
     if (F != nullptr)
     {
-        return IGC::GetID(*F);
+        return IGC::GetID(*F, useContextWrapper);
     }
     return ID::no_intrinsic;
 }
