@@ -114,14 +114,6 @@ static VcPayloadInfo tryExtractPayload(const char *Input, size_t InputSize) {
   return Result;
 }
 
-static std::unique_ptr<llvm::MemoryBuffer>
-getGenericModuleBuffer(int ResourceID) {
-  std::ostringstream SS;
-  SS << '#' << ResourceID;
-  return std::unique_ptr<llvm::MemoryBuffer>{
-      llvm::LoadBufferFromResource(SS.str().c_str(), "BC")};
-}
-
 template <enum vc::bif::RawKind Kind>
 std::unique_ptr<llvm::MemoryBuffer>
 getVCModuleBufferForArch(llvm::StringRef CPUStr = "") {
@@ -314,6 +306,11 @@ static void adjustTransformationsAndOptimizations(vc::CompileOptions &Opts) {
           IGC_GET_FLAG_VALUE(VCDisableExtraCoalescing));
 }
 
+static void adjustKernelMetrics(vc::CompileOptions &Opts) {
+  if (IGC_IS_FLAG_ENABLED(EnableKernelCostInfo))
+    Opts.EnableCostModel = true;
+}
+
 static void adjustDumpOptions(vc::CompileOptions &Opts) {
   if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable)) {
     Opts.DumpIR = true;
@@ -345,6 +342,7 @@ static void adjustOptions(const IGC::CPlatform &IGCPlatform,
   adjustHashOptions(Opts, Hash);
 
   adjustTransformationsAndOptimizations(Opts);
+  adjustKernelMetrics(Opts);
 }
 
 static void setErrorMessage(llvm::StringRef ErrorMessage,
@@ -474,9 +472,6 @@ fillExternalData(vc::BinaryKind Binary, llvm::StringRef CPUStr,
                  llvm::ArrayRef<const char *> VISALTOStrings,
                  llvm::ArrayRef<const char *> DirectCallFunctions) {
   vc::ExternalData ExtData;
-  ExtData.OCLGenericBIFModule = getGenericModuleBuffer(OCL_BC);
-  if (!ExtData.OCLGenericBIFModule)
-    return {};
   switch (Binary) {
   case vc::BinaryKind::CM:
     if (!fillPrintfData<vc::bif::RawKind::PrintfCM32,
