@@ -160,10 +160,11 @@ struct CompileOptions {
   bool EnableHashMovs = false;
   bool EnableHashMovsAtPrologue = false;
   uint64_t AsmHash = 0;
+
+  bool EnableCostModel = false;
 };
 
 struct ExternalData {
-  std::unique_ptr<llvm::MemoryBuffer> OCLGenericBIFModule;
   std::unique_ptr<llvm::MemoryBuffer> VCPrintf32BIFModule;
   std::unique_ptr<llvm::MemoryBuffer> VCPrintf64BIFModule;
   std::unique_ptr<llvm::MemoryBuffer> VCBuiltinsBIFModule;
