@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2020-2021 Intel Corporation
+Copyright (C) 2020-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -93,6 +93,16 @@ public:
   }
 };
 
+class BifLoadingError final : public llvm::ErrorInfo<BifLoadingError> {
+public:
+  static char ID;
+
+  void log(llvm::raw_ostream &OS) const override;
+  std::error_code convertToErrorCode() const override {
+    return make_error_code(errc::bif_load_fail);
+  }
+};
+
 class TargetMachineError final : public llvm::ErrorInfo<TargetMachineError> {
 public:
   static char ID;
