@@ -288,6 +288,8 @@ static GenXBackendOptions createBackendOptions(const vc::CompileOptions &Opts) {
   BackendOpts.EnableHashMovsAtPrologue = Opts.EnableHashMovsAtPrologue;
   BackendOpts.AsmHash = Opts.AsmHash;
 
+  BackendOpts.EnableCostModel = Opts.EnableCostModel;
+
   return BackendOpts;
 }
 
@@ -296,8 +298,6 @@ static GenXBackendData createBackendData(const vc::ExternalData &Data,
   IGC_ASSERT_MESSAGE(PointerSizeInBits == 32 || PointerSizeInBits == 64,
       "only 32 and 64 bit pointers are expected");
   GenXBackendData BackendData;
-  BackendData.BiFModule[BiFKind::OCLGeneric] =
-      llvm::MemoryBufferRef{*Data.OCLGenericBIFModule};
   BackendData.BiFModule[BiFKind::VCBuiltins] =
       llvm::MemoryBufferRef{*Data.VCBuiltinsBIFModule};
   BackendData.BiFModule[BiFKind::VCSPIRVBuiltins] =
