@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2021-2023 Intel Corporation
+Copyright (C) 2021-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -13,7 +13,9 @@ SPDX-License-Identifier: MIT
 /// The pass:
 /// * replaces all LLVM loads and stores, using correct namespace,
 /// * replaces all @llvm.masked.gather and @llvm.masked.scatter intrinsics,
+/// * replaces all @llvm.vc.internal.atomic intrinsics,
 /// * replaces all atomic instructions,
+/// * replaces all fence instructions,
 /// * removes lifetime builtins as we are not sure how to process those.
 ///
 //===----------------------------------------------------------------------===//
@@ -23,6 +25,8 @@ SPDX-License-Identifier: MIT
 #include "GenXUtil.h"
 #include "GenXVisa.h"
 
+#include "BiFModule/Headers/spirv_atomics_common.h"
+
 #include "vc/InternalIntrinsics/InternalIntrinsics.h"
 #include "vc/Support/BackendConfig.h"
 #include "vc/Support/GenXDiagnostic.h"
@@ -84,6 +88,13 @@ enum class HWAddrSpace : char {
   SLM, // Shared local memory.
 };
 
+enum SPIRVAtomicOp {
+  PointerOp = 0,
+  ScopeOp = 1,
+  SemanticsOp = 2,
+  SourceOp = 3
+};
+
 constexpr const char AlignMDName[] = "VCAlignment";
 
 // load and store lowering pass
@@ -119,11 +130,14 @@ private:
   Instruction *switchAddrSpace(MemoryInstT &I) const;
   template <MessageKind MK, Atomicity A>
   Instruction *switchAddrSpace(IntrinsicInst &I) const;
+  template <MessageKind MK, Atomicity A>
+  Instruction *switchAddrSpace(FenceInst &I) const;
 
   // Creates a replacement for \p I instruction. The template parameters
   // describe the provided instruction and how it should be lowered.
   template <HWAddrSpace HWAS, MessageKind MK, Atomicity A, typename MemoryInstT>
   Instruction *createIntrinsic(MemoryInstT &I) const;
+  template <MessageKind MK> Instruction *createIntrinsic(FenceInst &FI) const;
 
   Instruction *createLegacyLoadStore(Instruction &I, unsigned BTI, Value *Ptr,
                                      Value *Data = nullptr) const;
@@ -175,6 +189,9 @@ private:
   Instruction *createLSCAtomicRMW(AtomicRMWInst &I,
                                   vc::InternalIntrinsic::ID IID, Type *AddrTy,
                                   Value *BTI) const;
+  Instruction *createLSCAtomicRMW(IntrinsicInst &I,
+                                  vc::InternalIntrinsic::ID IID, Type *AddrTy,
+                                  Value *BTI) const;
   Instruction *createLSCAtomicCmpXchg(AtomicCmpXchgInst &I,
                                       vc::InternalIntrinsic::ID IID,
                                       Type *AddrTy, Value *BTI) const;
@@ -183,6 +200,7 @@ private:
                                    vc::InternalIntrinsic::ID IID,
                                    LSC_OP AtomicOp, Value *BTI, Value *Addr,
                                    Value *Src0, Value *Src1) const;
+  Instruction *createLSCStandAloneFence(FenceInst &I) const;
   void createLSCAtomicFenceImpl(Instruction &AtomicI, IRBuilder<> &Builder,
                                 bool IsPostFence) const;
 
@@ -194,6 +212,7 @@ private:
   Instruction *createLegacyAtomicImpl(Instruction &I, GenXIntrinsic::ID IID,
                                       Value *BTI, Value *Addr, Value *Src0,
                                       Value *Src1) const;
+  Instruction *createLegacyStandAloneFence(FenceInst &I) const;
   void createLegacyAtomicFenceImpl(Instruction &I, IRBuilder<> &Builder,
                                    bool IsPostFence) const;
 
@@ -232,6 +251,8 @@ public:
   void visitIntrinsicInst(IntrinsicInst &Intrinsic) const;
   void visitLoadInst(LoadInst &LdI) const;
   void visitStoreInst(StoreInst &StI) const;
+  void visitCallInst(CallInst &CI) const;
+  void visitFenceInst(FenceInst &FI) const;
 
 public:
   static char ID;
@@ -258,6 +279,31 @@ FunctionPass *llvm::createGenXLoadStoreLoweringPass() {
   return new GenXLoadStoreLowering;
 }
 
+static bool isSPIRVAtomic(const Value *Val) {
+  unsigned IID = vc::getAnyIntrinsicID(Val);
+  switch (IID) {
+  default:
+    return false;
+  case vc::InternalIntrinsic::atomic_fmin:
+  case vc::InternalIntrinsic::atomic_fmax:
+    return true;
+  }
+}
+
+static LSC_FENCE_OP getLSCFenceOp(AtomicOrdering Ordering) {
+  switch (Ordering) {
+  default:
+    return LSC_FENCE_OP_NONE;
+  case AtomicOrdering::Acquire:
+    return LSC_FENCE_OP_INVALIDATE;
+  case AtomicOrdering::Release:
+    return LSC_FENCE_OP_CLEAN;
+  case AtomicOrdering::AcquireRelease:
+  case AtomicOrdering::SequentiallyConsistent:
+    return LSC_FENCE_OP_EVICT;
+  }
+}
+
 void GenXLoadStoreLowering::getAnalysisUsage(AnalysisUsage &AU) const {
   AU.addRequired<TargetPassConfig>();
   AU.addRequired<GenXBackendConfig>();
@@ -287,6 +333,8 @@ bool GenXLoadStoreLowering::runOnFunction(Function &F) {
   //   * visitIntrinsicInst
   //   * visitLoadInst
   //   * visitStoreInst
+  //   * visitCallInst
+  //   * visitFenceInst
   visit(F);
 
   return true;
@@ -342,6 +390,23 @@ void GenXLoadStoreLowering::visitStoreInst(StoreInst &StI) const {
   StI.eraseFromParent();
 }
 
+void GenXLoadStoreLowering::visitCallInst(CallInst &CI) const {
+  if (!isSPIRVAtomic(&CI))
+    return;
+  LLVM_DEBUG(dbgs() << "Replacing intrinsic " << CI << " ===>\n");
+  auto *Replacement = createMemoryInstReplacement(cast<IntrinsicInst>(CI));
+  CI.replaceAllUsesWith(Replacement);
+  CI.eraseFromParent();
+}
+
+void GenXLoadStoreLowering::visitFenceInst(FenceInst &FI) const {
+  LLVM_DEBUG(dbgs() << "Replacing fence " << FI << " ===>\n");
+  auto *Replacement = createMemoryInstReplacement(FI);
+  if (Replacement)
+    LLVM_DEBUG(dbgs() << *Replacement << "\n");
+  FI.eraseFromParent();
+}
+
 namespace {
 struct GatherScatterOperands {
   bool IsLoad;
@@ -917,7 +982,6 @@ Instruction *GenXLoadStoreLowering::createLSCGatherScatter(
 std::pair<unsigned, AtomicOrdering>
 GenXLoadStoreLowering::getAddressSpaceAndOrderingOfAtomic(
     const Instruction &AtomicI) const {
-  IGC_ASSERT(AtomicI.isAtomic());
   if (auto *ARMW = dyn_cast<AtomicRMWInst>(&AtomicI))
     return {ARMW->getPointerAddressSpace(), ARMW->getOrdering()};
   if (auto *CmpXchg = dyn_cast<AtomicCmpXchgInst>(&AtomicI))
@@ -934,6 +998,37 @@ GenXLoadStoreLowering::getAddressSpaceAndOrderingOfAtomic(
                       ->getAddressSpace();
     return {AS, SI->getOrdering()};
   }
+  if (auto *FI = dyn_cast<FenceInst>(&AtomicI)) {
+    auto ScopeID = FI->getSyncScopeID();
+    auto AS = StringSwitch<unsigned>(SyncScopeNames[ScopeID])
+                  .Case("workitem", vc::AddrSpace::Private)
+                  .Case("subgroup", vc::AddrSpace::Private)
+                  .Case("workgroup", vc::AddrSpace::Local)
+                  .Default(vc::AddrSpace::Global);
+    return {AS, FI->getOrdering()};
+  }
+  if (isSPIRVAtomic(&AtomicI)) {
+    auto *II = cast<IntrinsicInst>(&AtomicI);
+    unsigned AS = vc::getAddrSpace(
+        II->getArgOperand(SPIRVAtomicOp::PointerOp)->getType());
+    unsigned Semantics =
+        cast<ConstantInt>(II->getArgOperand(SPIRVAtomicOp::SemanticsOp))
+            ->getZExtValue();
+    switch (Semantics) {
+    default:
+      LLVM_FALLTHROUGH;
+    case SequentiallyConsistent:
+      return {AS, AtomicOrdering::SequentiallyConsistent};
+    case Relaxed:
+      return {AS, AtomicOrdering::Monotonic};
+    case Acquire:
+      return {AS, AtomicOrdering::Acquire};
+    case Release:
+      return {AS, AtomicOrdering::Release};
+    case AcquireRelease:
+      return {AS, AtomicOrdering::AcquireRelease};
+    }
+  }
   IGC_ASSERT_MESSAGE(false, "Unimplemented atomic inst");
   return {0, AtomicOrdering::Monotonic};
 }
@@ -985,6 +1080,24 @@ LSC_SCOPE GenXLoadStoreLowering::getLSCFenceScope(Instruction *I) const {
     ScopeID = AI->getSyncScopeID();
   else if (auto *FI = dyn_cast<FenceInst>(I))
     ScopeID = FI->getSyncScopeID();
+  else if (isSPIRVAtomic(I)) {
+    auto *II = cast<IntrinsicInst>(I);
+    unsigned Scope =
+        cast<ConstantInt>(II->getArgOperand(SPIRVAtomicOp::ScopeOp))
+            ->getZExtValue();
+    switch (Scope) {
+    default:
+      LLVM_FALLTHROUGH;
+    case CrossDevice:
+      return LSC_SCOPE_GPUS;
+    case Device:
+      return ST->hasMultiTile() ? LSC_SCOPE_GPU : LSC_SCOPE_TILE;
+    case Workgroup:
+    case Subgroup:
+    case Invocation:
+      return LSC_SCOPE_GROUP;
+    }
+  }
 
   switch (ScopeID) {
   case SyncScope::SingleThread:
@@ -995,17 +1108,37 @@ LSC_SCOPE GenXLoadStoreLowering::getLSCFenceScope(Instruction *I) const {
 
   return StringSwitch<LSC_SCOPE>(SyncScopeNames[ScopeID])
       .Case("subgroup", LSC_SCOPE_GROUP)
-      .Case("workgroup", LSC_SCOPE_LOCAL)
+      .Case("workgroup", LSC_SCOPE_GROUP)
       .Case("device", ST->hasMultiTile() ? LSC_SCOPE_GPU : LSC_SCOPE_TILE)
       .Case("all_devices", LSC_SCOPE_GPUS)
       .Default(LSC_SCOPE_GROUP);
 }
 
+Instruction *
+GenXLoadStoreLowering::createLSCStandAloneFence(FenceInst &I) const {
+  IRBuilder<> Builder(&I);
+  auto [AS, Ordering] = getAddressSpaceAndOrderingOfAtomic(I);
+
+  if (AS == vc::AddrSpace::Private ||
+      AS == vc::AddrSpace::Local && ST->hasLocalMemFenceSupress())
+    return nullptr;
+
+  bool IsGlobal = AS == vc::AddrSpace::Global;
+  auto FenceOp = IsGlobal ? getLSCFenceOp(Ordering) : LSC_FENCE_OP_NONE;
+  auto SubFuncID = IsGlobal ? LSC_UGM : LSC_SLM;
+  auto Scope = getLSCFenceScope(&I);
+
+  auto *M = I.getModule();
+  auto *Func = GenXIntrinsic::getAnyDeclaration(
+      M, GenXIntrinsic::genx_lsc_fence, {Builder.getInt1Ty()});
+  return Builder.CreateCall(Func,
+                            {Builder.getTrue(), Builder.getInt8(SubFuncID),
+                             Builder.getInt8(FenceOp), Builder.getInt8(Scope)});
+}
+
 void GenXLoadStoreLowering::createLSCAtomicFenceImpl(Instruction &AtomicI,
                                                      IRBuilder<> &Builder,
                                                      bool IsPostFence) const {
-  IGC_ASSERT_EXIT(AtomicI.isAtomic());
-
   auto [AS, Ordering] = getAddressSpaceAndOrderingOfAtomic(AtomicI);
 
   bool IsGlobal = AS != vc::AddrSpace::Local;
@@ -1037,7 +1170,6 @@ void GenXLoadStoreLowering::createLSCAtomicFenceImpl(Instruction &AtomicI,
 Instruction *GenXLoadStoreLowering::createLSCAtomicImpl(
     Instruction &I, vc::InternalIntrinsic::ID IID, LSC_OP AtomicOp, Value *Base,
     Value *Addr, Value *Src0, Value *Src1) const {
-  IGC_ASSERT_EXIT(I.isAtomic());
   IGC_ASSERT_EXIT(IID == vc::InternalIntrinsic::lsc_atomic_bti ||
                   IID == vc::InternalIntrinsic::lsc_atomic_slm ||
                   IID == vc::InternalIntrinsic::lsc_atomic_ugm);
@@ -1057,10 +1189,19 @@ Instruction *GenXLoadStoreLowering::createLSCAtomicImpl(
                   ElementSize == LSC_DATA_SIZE_64b);
 
   if (ElementSize == LSC_DATA_SIZE_16c32b) {
-    Src0 = Builder.CreateBitCast(Src0, Builder.getInt16Ty());
-    Src0 = Builder.CreateZExt(Src0, Builder.getInt32Ty());
-    Src1 = Builder.CreateBitCast(Src1, Builder.getInt16Ty());
-    Src1 = Builder.CreateZExt(Src1, Builder.getInt32Ty());
+    // We must preserve undef operands but
+    // zext/sext casts make them zero.
+    if (!isa<UndefValue>(Src0)) {
+      Src0 = Builder.CreateBitCast(Src0, Builder.getInt16Ty());
+      Src0 = Builder.CreateZExt(Src0, Builder.getInt32Ty());
+    } else
+      Src0 = UndefValue::get(Builder.getInt32Ty());
+
+    if (!isa<UndefValue>(Src1)) {
+      Src1 = Builder.CreateBitCast(Src1, Builder.getInt16Ty());
+      Src1 = Builder.CreateZExt(Src1, Builder.getInt32Ty());
+    } else
+      Src1 = UndefValue::get(Builder.getInt32Ty());
   }
 
   auto AddrSize = IID == vc::InternalIntrinsic::lsc_atomic_ugm
@@ -1220,6 +1361,37 @@ GenXLoadStoreLowering::createLSCAtomicRMW(AtomicRMWInst &I,
   return createLSCAtomicImpl(I, IID, AtomicOp, Base, Addr, Src, Undef);
 }
 
+Instruction *
+GenXLoadStoreLowering::createLSCAtomicRMW(IntrinsicInst &I,
+                                          vc::InternalIntrinsic::ID IID,
+                                          Type *AddrTy, Value *Base) const {
+  IGC_ASSERT(isSPIRVAtomic(&I));
+
+  IRBuilder<> Builder(&I);
+  auto *Ptr = I.getArgOperand(SPIRVAtomicOp::PointerOp);
+  auto *Addr = Builder.CreatePtrToInt(Ptr, AddrTy);
+
+  auto *Src = I.getArgOperand(SPIRVAtomicOp::SourceOp);
+  auto *DataTy = Src->getType();
+  auto *Undef = UndefValue::get(DataTy);
+
+  auto AtomicOp = LSC_INVALID;
+  auto ID = vc::getAnyIntrinsicID(&I);
+  switch (ID) {
+  case vc::InternalIntrinsic::atomic_fmin:
+    AtomicOp = LSC_ATOMIC_FMIN;
+    break;
+  case vc::InternalIntrinsic::atomic_fmax:
+    AtomicOp = LSC_ATOMIC_FMAX;
+    break;
+  default:
+    IGC_ASSERT_EXIT_MESSAGE(0, "Unexpected atomic operation");
+    break;
+  }
+
+  return createLSCAtomicImpl(I, IID, AtomicOp, Base, Addr, Src, Undef);
+}
+
 Instruction *
 GenXLoadStoreLowering::createLSCAtomicCmpXchg(AtomicCmpXchgInst &I,
                                               vc::InternalIntrinsic::ID IID,
@@ -1245,6 +1417,32 @@ GenXLoadStoreLowering::createLSCAtomicCmpXchg(AtomicCmpXchgInst &I,
   return cast<Instruction>(Res);
 }
 
+Instruction *
+GenXLoadStoreLowering::createLegacyStandAloneFence(FenceInst &I) const {
+  IRBuilder<> Builder(&I);
+  auto [AS, Ordering] = getAddressSpaceAndOrderingOfAtomic(I);
+
+  if (AS == vc::AddrSpace::Private ||
+      AS == vc::AddrSpace::Local && ST->hasLocalMemFenceSupress())
+    return nullptr;
+
+  bool IsGlobal = AS == vc::AddrSpace::Global;
+  bool IsInvalidateL1 = Ordering == AtomicOrdering::SequentiallyConsistent ||
+                        Ordering == AtomicOrdering::AcquireRelease ||
+                        Ordering == AtomicOrdering::Acquire;
+  IsInvalidateL1 &= IsGlobal;
+
+  uint8_t FenceOp = 1;
+  if (!IsGlobal)
+    FenceOp |= 1 << 5;
+  if (IsInvalidateL1)
+    FenceOp |= 1 << 6;
+
+  auto *M = I.getModule();
+  auto *Func = GenXIntrinsic::getAnyDeclaration(M, GenXIntrinsic::genx_fence);
+  return Builder.CreateCall(Func, {Builder.getInt8(FenceOp)});
+}
+
 void GenXLoadStoreLowering::createLegacyAtomicFenceImpl(
     Instruction &AtomicI, IRBuilder<> &Builder, bool IsPostFence) const {
   IGC_ASSERT_EXIT(AtomicI.isAtomic());
@@ -1997,7 +2195,8 @@ template <typename MemoryInstT>
 Instruction *
 GenXLoadStoreLowering::createMemoryInstReplacement(MemoryInstT &I) const {
   auto *Replacement = switchAtomicity(I);
-  Replacement->takeName(&I);
+  if (Replacement)
+    Replacement->takeName(&I);
   return Replacement;
 }
 
@@ -2008,6 +2207,14 @@ Instruction *GenXLoadStoreLowering::switchAtomicity(MemoryInstT &I) const {
   return switchMessage<Atomicity::NonAtomic>(I);
 }
 
+template <>
+Instruction *
+GenXLoadStoreLowering::switchAtomicity<IntrinsicInst>(IntrinsicInst &I) const {
+  if (isSPIRVAtomic(&I))
+    return switchMessage<Atomicity::Atomic>(I);
+  return switchMessage<Atomicity::NonAtomic>(I);
+}
+
 template <Atomicity A, typename MemoryInstT>
 Instruction *GenXLoadStoreLowering::switchMessage(MemoryInstT &I) const {
   if (ST->hasLSCMessages())
@@ -2049,12 +2256,22 @@ Instruction *GenXLoadStoreLowering::switchAddrSpace(IntrinsicInst &I) const {
   case Intrinsic::masked_scatter:
     PointerOperandNum = 1;
     break;
+  case vc::InternalIntrinsic::atomic_fmin:
+  case vc::InternalIntrinsic::atomic_fmax:
+    PointerOperandNum = SPIRVAtomicOp::PointerOp;
+    break;
   }
 
   auto *Ptr = I.getArgOperand(PointerOperandNum);
-  auto *PtrVTy = cast<IGCLLVM::FixedVectorType>(Ptr->getType());
-  auto *PtrTy = cast<PointerType>(PtrVTy->getElementType());
-  return switchAddrSpace<MK, A>(I, PtrTy);
+  auto *PtrTy = Ptr->getType();
+  if (auto *PtrVTy = dyn_cast<IGCLLVM::FixedVectorType>(PtrTy))
+    PtrTy = PtrVTy->getElementType();
+  return switchAddrSpace<MK, A>(I, cast<PointerType>(PtrTy));
+}
+
+template <MessageKind MK, Atomicity A>
+Instruction *GenXLoadStoreLowering::switchAddrSpace(FenceInst &I) const {
+  return createIntrinsic<MK>(I);
 }
 
 template <HWAddrSpace HWAS, MessageKind MK, Atomicity A, typename MemoryInstT>
@@ -2233,6 +2450,12 @@ GenXLoadStoreLowering::createIntrinsic<HWAddrSpace::SLM, MessageKind::Legacy,
   return createLegacyGatherScatter(I, visa::RSI_Slm);
 }
 
+template <>
+Instruction *GenXLoadStoreLowering::createIntrinsic<MessageKind::Legacy>(
+    FenceInst &I) const {
+  return createLegacyStandAloneFence(I);
+}
+
 template <>
 Instruction *
 GenXLoadStoreLowering::createIntrinsic<HWAddrSpace::A64, MessageKind::LSC,
@@ -2349,6 +2572,30 @@ GenXLoadStoreLowering::createIntrinsic<HWAddrSpace::SLM, MessageKind::LSC,
                                 AddrTy);
 }
 
+template <>
+Instruction *
+GenXLoadStoreLowering::createIntrinsic<HWAddrSpace::A64, MessageKind::LSC,
+                                       Atomicity::Atomic, IntrinsicInst>(
+    IntrinsicInst &I) const {
+  IRBuilder<> Builder(&I);
+  auto *Base = Builder.getInt64(0);
+  auto *AddrTy = Builder.getInt64Ty();
+  return createLSCAtomicRMW(I, vc::InternalIntrinsic::lsc_atomic_ugm, AddrTy,
+                            Base);
+}
+
+template <>
+Instruction *
+GenXLoadStoreLowering::createIntrinsic<HWAddrSpace::SLM, MessageKind::LSC,
+                                       Atomicity::Atomic, IntrinsicInst>(
+    IntrinsicInst &I) const {
+  IRBuilder<> Builder(&I);
+  auto *Base = Builder.getInt32(0);
+  auto *AddrTy = Builder.getInt32Ty();
+  return createLSCAtomicRMW(I, vc::InternalIntrinsic::lsc_atomic_slm, AddrTy,
+                            Base);
+}
+
 template <>
 Instruction *
 GenXLoadStoreLowering::createIntrinsic<HWAddrSpace::A64, MessageKind::LSC,
@@ -2492,3 +2739,10 @@ GenXLoadStoreLowering::createIntrinsic<HWAddrSpace::SLM, MessageKind::LSC,
   return createLSCAtomicCmpXchg(I, vc::InternalIntrinsic::lsc_atomic_slm,
                                 AddrTy, Base);
 }
+
+template <>
+Instruction *
+GenXLoadStoreLowering::createIntrinsic<MessageKind::LSC>(FenceInst &I) const {
+  IRBuilder<> Builder(&I);
+  return createLSCStandAloneFence(I);
+}
