@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2019-2023 Intel Corporation
+Copyright (C) 2019-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -159,18 +159,15 @@ static void setMemorySliceWithVecStore(SliceInfo Slice, Value &SetVal,
       "wrong argument: insertion point must be a valid instruction");
   IGC_ASSERT_MESSAGE(Slice.Offset >= 0 && isPowerOf2_32(Slice.Width),
                      "illegal slice is provided");
-  IGC_ASSERT_MESSAGE(SetVal.getType() ==
-                         IGCLLVM::getNonOpaquePtrEltTy(BaseAddr.getType()),
-                     "value and pointer types must correspond");
 
   auto *VecTy = IGCLLVM::FixedVectorType::get(SetVal.getType(), Slice.Width);
   IRBuilder<> IRB(InsertionPt);
   Value *WriteOut = IRB.CreateVectorSplat(Slice.Width, &SetVal);
   auto *DstAddr = &BaseAddr;
   if (Slice.Offset != 0)
-    DstAddr = IRB.CreateGEP(IGCLLVM::getNonOpaquePtrEltTy(BaseAddr.getType()),
-                            &BaseAddr, IRB.getInt32(Slice.Offset),
-                            BaseAddr.getName() + ".addr.offset");
+    DstAddr =
+        IRB.CreateGEP(SetVal.getType(), &BaseAddr, IRB.getInt32(Slice.Offset),
+                      BaseAddr.getName() + ".addr.offset");
   auto DstAS = cast<PointerType>(DstAddr->getType())->getAddressSpace();
   auto *StoreVecPtr = IRB.CreateBitCast(DstAddr, VecTy->getPointerTo(DstAS));
   auto *Store = IRB.CreateStore(WriteOut, StoreVecPtr);
@@ -251,17 +248,14 @@ void GenXLowerAggrCopies::expandMemMov2VecLoadStore(T *MemCall) {
 
   auto *DstAddr = MemCall->getRawDest();
 
-  auto *I8Ty = IGCLLVM::getNonOpaquePtrEltTy(DstAddr->getType());
-  IGC_ASSERT(I8Ty->isIntegerTy(8));
-  auto *VecTy = IGCLLVM::FixedVectorType::get(I8Ty, Len);
+  auto *VecTy = IGCLLVM::FixedVectorType::get(IRB.getInt8Ty(), Len);
 
   auto *SrcAddr = MemCall->getRawSource();
   unsigned SrcAddrSpace =
       cast<PointerType>(SrcAddr->getType())->getAddressSpace();
 
   auto *LoadPtrV = IRB.CreateBitCast(SrcAddr, VecTy->getPointerTo(SrcAddrSpace));
-  Type *Ty = IGCLLVM::getNonOpaquePtrEltTy(LoadPtrV->getType());
-  auto *Load = IRB.CreateLoad(Ty, LoadPtrV);
+  auto *Load = IRB.CreateLoad(VecTy, LoadPtrV);
   Load->setAlignment(IGCLLVM::getSourceAlign(*MemCall));
 
   unsigned DstAddrSpace = cast<PointerType>(DstAddr->getType())->getAddressSpace();
