@@ -28,14 +28,14 @@ SPDX-License-Identifier: MIT
 /// before it for pre-copies, where N is the number of SimpleValues in the
 /// (possibly struct) args, allowing for extra args that might be added later by
 /// GenXArgIndirection.
-/// 
+///
 /// Similarly, a non-intrinsic call has N slots reserved after it for
 /// post-copies, where N is the number of SimpleValues in the (possibly struct)
 /// return value. The definition of each SimpleValue in the result of the call
 /// is considered to be in its slot, and the corresponding SimpleValue in the
 /// unified return value has an extra segment of live range from the call up to
 /// that slot.
-/// 
+///
 /// A return instruction in a subroutine has N slots reserved before it for
 /// pre-copies, where N is the number of SimpleValues in the (possibly struct)
 /// return value. The use of each SimpleValue in the return is considered to be
@@ -58,6 +58,8 @@ SPDX-License-Identifier: MIT
 #include "IgnoreRAUWValueMap.h"
 #include "llvm/IR/Value.h"
 
+#include <unordered_map>
+
 namespace llvm {
 
 class CallInst;
@@ -65,9 +67,8 @@ class GenXBaling;
 class PHINode;
 class ReturnInst;
 
-FunctionGroupPass *createGenXGroupPrinterPass(raw_ostream &O, const std::string &Banner);
-
-class GenXNumbering : public FunctionGroupPass {
+class GenXNumbering : public FGPassImplInterface,
+                      public IDMixin<GenXNumbering> {
   FunctionGroup *FG = nullptr;
   GenXBaling *Baling = nullptr;
   struct BBNumber {
@@ -99,12 +100,10 @@ class GenXNumbering : public FunctionGroupPass {
   unsigned LastNum = 0;
 
 public:
-  static char ID;
-  explicit GenXNumbering() : FunctionGroupPass(ID), Baling(0) { }
-  ~GenXNumbering() { clear(); }
-  virtual StringRef getPassName() const { return "GenX numbering"; }
-  void getAnalysisUsage(AnalysisUsage &AU) const;
-  bool runOnFunctionGroup(FunctionGroup &FG);
+  explicit GenXNumbering() : Baling(0) {}
+  static StringRef getPassName() { return "GenX numbering"; }
+  static void getAnalysisUsage(AnalysisUsage &AU);
+  bool runOnFunctionGroup(FunctionGroup &FG) override;
   // get BBNumber struct for a basic block
   const BBNumber *getBBNumber(BasicBlock *BB) { return &BBNumbers[BB]; }
   // get and set instruction number
@@ -128,25 +127,19 @@ public:
   // getPhiIncomingFromNumber : get the phi incoming for a number returned from getPhiNumber
   std::unordered_map<PHINode *, unsigned>
   getPhiIncomingFromNumber(unsigned Number);
-  // createPrinterPass : get a pass to print the IR, together with the GenX
-  // specific analyses
-  virtual Pass *createPrinterPass(raw_ostream &O,
-                                  const std::string &Banner) const {
-    return createGenXGroupPrinterPass(O, Banner);
-  }
   // Debug dump
+  void print(raw_ostream &OS) const;
   void dump();
-  using llvm::Pass::print; // enables overloading of print in this class rather
-                           // than override (and stops compiler warnings)
-  virtual void print(raw_ostream &OS) const;
+
+  void releaseMemory() override;
 
 private:
-  void clear();
   unsigned numberInstructionsInFunc(Function *Func, unsigned Num);
   unsigned getPhiOffset(PHINode *Phi) const;
 };
 
-void initializeGenXNumberingPass(PassRegistry &);
+void initializeGenXNumberingWrapperPass(PassRegistry &);
+using GenXNumberingWrapper = FunctionGroupWrapperPass<GenXNumbering>;
 
 } // end namespace llvm
 #endif //ndef GENXNUMBERING_H
