@@ -17,8 +17,10 @@ SPDX-License-Identifier: MIT
 #include "OCLRuntimeInfoPrinter.h"
 #include "GenXGlobalUniform.h"
 
+#include "vc/Utils/GenX/CostInfo.h"
 #include "vc/Utils/GenX/GlobalVariable.h"
 #include "vc/Utils/GenX/InternalMetadata.h"
+#include "vc/Utils/GenX/KernelInfo.h"
 #include "vc/Utils/GenX/Printf.h"
 #include "vc/Utils/GenX/RegCategory.h"
 
@@ -54,6 +56,7 @@ SPDX-License-Identifier: MIT
   } while (0);
 
 using namespace llvm;
+using namespace vc;
 
 char GenXOCLRuntimeInfo::ID = 0;
 
@@ -62,33 +65,11 @@ char GenXOCLRuntimeInfo::ID = 0;
 // Kernel argument info implementation.
 //
 //===----------------------------------------------------------------------===//
-// Supported kernel argument attributes.
-struct OCLAttributes {
-  // Type qualifiers for resources.
-  static constexpr auto ReadOnly = "read_only";
-  static constexpr auto WriteOnly = "write_only";
-  static constexpr auto ReadWrite = "read_write";
-
-  // Buffer surface.
-  static constexpr auto Buffer = "buffer_t";
-  // SVM pointer to buffer.
-  static constexpr auto SVM = "svmptr_t";
-  // OpenCL-like types.
-  static constexpr auto Sampler = "sampler_t";
-  static constexpr auto Image1d = "image1d_t";
-  static constexpr auto Image1dArray = "image1d_array_t";
-  // Same as 1D image. Seems that there is no difference in runtime.
-  static constexpr auto Image1dBuffer = "image1d_buffer_t";
-  static constexpr auto Image2d = "image2d_t";
-  static constexpr auto Image2dArray = "image2d_array_t";
-  static constexpr auto Image2dMediaBlock = "image2d_media_block_t";
-  static constexpr auto Image3d = "image3d_t";
-};
-
 namespace llvm {
 class KernelArgBuilder final {
   using ArgKindType = GenXOCLRuntimeInfo::KernelArgInfo::KindType;
   using ArgAccessKindType = GenXOCLRuntimeInfo::KernelArgInfo::AccessKindType;
+  using ArgAddressModeType = GenXOCLRuntimeInfo::KernelArgInfo::AddressModeType;
 
   const vc::KernelMetadata &KM;
   const DataLayout &DL;
@@ -112,8 +93,12 @@ private:
                             const Argument &Arg) const;
   ArgAccessKindType getOCLArgAccessKind(ArrayRef<StringRef> Tokens,
                                         ArgKindType Kind) const;
-  std::pair<ArgKindType, ArgAccessKindType>
+  ArgAddressModeType getOCLArgAddressMode(ArgKindType Kind,
+                                          const Argument &Arg) const;
+
+  std::tuple<ArgKindType, ArgAddressModeType, ArgAccessKindType>
   translateArgDesc(const Argument &Arg) const;
+
   unsigned getArgSizeInBytes(const Argument &Arg) const;
 };
 } // namespace llvm
@@ -143,7 +128,6 @@ KernelArgBuilder::getOCLArgAccessKind(ArrayRef<StringRef> Tokens,
   case ArgKindType::Image2DMediaBlock:
   case ArgKindType::Image3D:
   case ArgKindType::SVM:
-  case ArgKindType::BindlessBuffer:
     if (any_of(Tokens, getStrPred(OCLAttributes::ReadOnly)))
       return ArgAccessKindType::ReadOnly;
     if (any_of(Tokens, getStrPred(OCLAttributes::WriteOnly)))
@@ -155,6 +139,43 @@ KernelArgBuilder::getOCLArgAccessKind(ArrayRef<StringRef> Tokens,
   }
 }
 
+KernelArgBuilder::ArgAddressModeType
+KernelArgBuilder::getOCLArgAddressMode(ArgKindType Kind,
+                                       const Argument &Arg) const {
+  unsigned ArgNo = Arg.getArgNo();
+
+  switch (Kind) {
+  default:
+    break;
+  case ArgKindType::SVM:
+  case ArgKindType::AssertBuffer:
+  case ArgKindType::PrintBuffer:
+  case ArgKindType::SyncBuffer:
+  case ArgKindType::PrivateBase:
+    return ArgAddressModeType::Stateless;
+  case ArgKindType::Sampler:
+    if (KM.getArgCategory(ArgNo) == vc::RegCategory::Sampler)
+      return ArgAddressModeType::Stateful;
+    IGC_ASSERT_MESSAGE(KM.getArgCategory(ArgNo) == vc::RegCategory::General,
+                       "Expected general category");
+    return ArgAddressModeType::Bindless;
+  case ArgKindType::Buffer:
+  case ArgKindType::Image1D:
+  case ArgKindType::Image1DArray:
+  case ArgKindType::Image2D:
+  case ArgKindType::Image2DArray:
+  case ArgKindType::Image2DMediaBlock:
+  case ArgKindType::Image3D:
+    if (KM.getArgCategory(ArgNo) == vc::RegCategory::Surface)
+      return ArgAddressModeType::Stateful;
+    IGC_ASSERT_MESSAGE(KM.getArgCategory(ArgNo) == vc::RegCategory::General,
+                       "Expected general category");
+    return ArgAddressModeType::Bindless;
+  }
+
+  return ArgAddressModeType::None;
+}
+
 KernelArgBuilder::ArgKindType
 KernelArgBuilder::getOCLArgKind(ArrayRef<StringRef> Tokens,
                                 const Argument &Arg) const {
@@ -181,19 +202,15 @@ KernelArgBuilder::getOCLArgKind(ArrayRef<StringRef> Tokens,
     return ArgKindType::ImplicitArgsBuffer;
 
   // Explicit arguments.
-  switch (KM.getArgCategory(ArgNo)) {
+  switch (auto Cat = KM.getArgCategory(ArgNo)) {
   default:
-    return ArgKindType::General;
+    break;
   case vc::RegCategory::General:
     if (any_of(Tokens, getStrPred(OCLAttributes::SVM)))
       return ArgKindType::SVM;
-    // Bindless buffers have general category but buffer annotation.
-    if (any_of(Tokens, getStrPred(OCLAttributes::Buffer)))
-      return ArgKindType::BindlessBuffer;
-    if (ArgTy->isPointerTy())
-      if (vc::getAddrSpace(ArgTy) == vc::AddrSpace::Local)
-        return ArgKindType::SLM;
-    return ArgKindType::General;
+    if (ArgTy->isPointerTy() && vc::getAddrSpace(ArgTy) == vc::AddrSpace::Local)
+      return ArgKindType::SLM;
+    LLVM_FALLTHROUGH;
   case vc::RegCategory::Surface:
     if (any_of(Tokens, getStrPred(OCLAttributes::Image1d)))
       return ArgKindType::Image1D;
@@ -202,9 +219,7 @@ KernelArgBuilder::getOCLArgKind(ArrayRef<StringRef> Tokens,
     if (any_of(Tokens, getStrPred(OCLAttributes::Image1dBuffer)))
       return ArgKindType::Image1D;
     if (any_of(Tokens, getStrPred(OCLAttributes::Image2d))) {
-      if (ST.noLegacyDataport())
-        return ArgKindType::Image2D;
-      if (BC.usePlain2DImages())
+      if (ST.noLegacyDataport() || BC.usePlain2DImages())
         return ArgKindType::Image2D;
       // Legacy behavior to treat all 2d images as media block.
       return ArgKindType::Image2DMediaBlock;
@@ -212,20 +227,25 @@ KernelArgBuilder::getOCLArgKind(ArrayRef<StringRef> Tokens,
     if (any_of(Tokens, getStrPred(OCLAttributes::Image2dArray)))
       return ArgKindType::Image2DArray;
     if (any_of(Tokens, getStrPred(OCLAttributes::Image2dMediaBlock))) {
-      if (ST.translateLegacyMessages())
+      if (ST.translateMediaBlockMessages())
         return ArgKindType::Image2D;
       return ArgKindType::Image2DMediaBlock;
     }
     if (any_of(Tokens, getStrPred(OCLAttributes::Image3d)))
       return ArgKindType::Image3D;
-    return ArgKindType::Buffer;
+    if (Cat == vc::RegCategory::Surface)
+      return ArgKindType::Buffer;
+    break;
   case vc::RegCategory::Sampler:
     return ArgKindType::Sampler;
   }
+
+  return ArgKindType::General;
 }
 
 // Retrieve Kind and AccessKind from given ArgTypeDesc in metadata.
-std::pair<KernelArgBuilder::ArgKindType, KernelArgBuilder::ArgAccessKindType>
+std::tuple<KernelArgBuilder::ArgKindType, KernelArgBuilder::ArgAddressModeType,
+           KernelArgBuilder::ArgAccessKindType>
 KernelArgBuilder::translateArgDesc(const Argument &Arg) const {
   unsigned ArgNo = Arg.getArgNo();
   std::string Translated{KM.getArgTypeDesc(ArgNo)};
@@ -244,9 +264,10 @@ KernelArgBuilder::translateArgDesc(const Argument &Arg) const {
   std::sort(Tokens.begin(), Tokens.end());
   Tokens.erase(std::unique(Tokens.begin(), Tokens.end()), Tokens.end());
 
-  const ArgKindType Kind = getOCLArgKind(Tokens, Arg);
-  const ArgAccessKindType AccessKind = getOCLArgAccessKind(Tokens, Kind);
-  return {Kind, AccessKind};
+  const auto Kind = getOCLArgKind(Tokens, Arg);
+  const auto AddressMode = getOCLArgAddressMode(Kind, Arg);
+  const auto AccessKind = getOCLArgAccessKind(Tokens, Kind);
+  return {Kind, AddressMode, AccessKind};
 }
 
 unsigned KernelArgBuilder::getArgSizeInBytes(const Argument &Arg) const {
@@ -262,12 +283,15 @@ GenXOCLRuntimeInfo::KernelArgInfo
 KernelArgBuilder::translateArgument(const Argument &Arg) const {
   GenXOCLRuntimeInfo::KernelArgInfo Info;
   unsigned ArgNo = Arg.getArgNo();
-  std::tie(Info.Kind, Info.AccessKind) = translateArgDesc(Arg);
+
+  std::tie(Info.Kind, Info.AddrMode, Info.AccessKind) = translateArgDesc(Arg);
+
   Info.Offset = KM.getArgOffset(ArgNo);
   Info.SizeInBytes = getArgSizeInBytes(Arg);
   Info.BTI = KM.getBTI(ArgNo);
   // For implicit arguments that are byval argument linearization, index !=
   // ArgNo in the IR function.
+  Info.ArgNo = ArgNo;
   Info.Index = KM.getArgIndex(ArgNo);
   // Linearization arguments have a non-zero offset in the original explicit
   // byval arg.
@@ -330,9 +354,9 @@ void GenXOCLRuntimeInfo::FunctionInfo::initInstructionLevelProperties(
       NumBarriers = 1;
       LLVM_DEBUG(dbgs() << ">> NumBarriers: " << NumBarriers << "\n");
       break;
-    case GenXIntrinsic::genx_3d_sample:
-    case GenXIntrinsic::genx_sample:
     case GenXIntrinsic::genx_sample_unorm:
+    case vc::InternalIntrinsic::sample_bti:
+    case vc::InternalIntrinsic::sampler_load_bti:
       UsesSample = true;
       LLVM_DEBUG(dbgs() << ">> UsesSample: true\n");
       break;
@@ -428,14 +452,11 @@ GenXOCLRuntimeInfo::KernelInfo::KernelInfo(const FunctionGroup &FG,
 // Compiled kernel implementation.
 //
 //===----------------------------------------------------------------------===//
-GenXOCLRuntimeInfo::CompiledKernel::CompiledKernel(KernelInfo &&KI,
-                                                   const vISA::FINALIZER_INFO &JI,
-                                                   const GTPinInfo &GI,
-                                                   std::vector<char> DbgInfoIn)
-    : CompilerInfo(std::move(KI)), JitterInfo(JI),
-      GtpinInfo(GI),
-      DebugInfo{std::move(DbgInfoIn)} {
-}
+GenXOCLRuntimeInfo::CompiledKernel::CompiledKernel(
+    KernelInfo &&KI, const CostInfoT &CI, const vISA::FINALIZER_INFO &JI,
+    const GTPinInfo &GI, std::vector<char> DbgInfoIn)
+    : CompilerInfo(std::move(KI)), CostInfo(CI), JitterInfo(JI),
+      GtpinInfo(GI), DebugInfo{std::move(DbgInfoIn)} {}
 
 //===----------------------------------------------------------------------===//
 //
@@ -831,6 +852,7 @@ RuntimeInfoCollector::CompiledModuleT RuntimeInfoCollector::run() {
 RuntimeInfoCollector::CompiledKernel
 RuntimeInfoCollector::collectFunctionGroupInfo(const FunctionGroup &FG) const {
   using KernelInfo = GenXOCLRuntimeInfo::KernelInfo;
+  using CostInfoT = GenXOCLRuntimeInfo::CostInfoT;
   using GTPinInfo = GenXOCLRuntimeInfo::GTPinInfo;
   using CompiledKernel = GenXOCLRuntimeInfo::CompiledKernel;
 
@@ -885,8 +907,52 @@ RuntimeInfoCollector::collectFunctionGroupInfo(const FunctionGroup &FG) const {
   GTPinInfo gtpin{GTPinBytes, GTPinBytes + GTPinBufferSize};
 
   Info.Func.Data.Buffer = std::move(TextSection.Data).emitConsolidatedData();
-  return CompiledKernel{std::move(Info), *JitInfo, std::move(gtpin),
-                        std::move(DebugData)};
+
+  CostInfoT CInfo{};
+  if (BC.isCostModelEnabled()) {
+    // Get cost metrics from finalizer.
+    const vISA::KernelCostInfo *KernelCostInfo = nullptr;
+    VISAKernel *VF = VB.GetVISAKernel(FG.getName().str());
+    VF->getKernelCostInfo(KernelCostInfo);
+    if (KernelCostInfo) {
+      auto &VISAKernelMetric = KernelCostInfo->kernelCost.C;
+      CInfo.Costs.push_back({(int)VISAKernelMetric.cycles,
+                             (int)VISAKernelMetric.loadBytes,
+                             (int)VISAKernelMetric.storeBytes, 0});
+      for (auto &VISACost : KernelCostInfo->allLoopCosts) {
+        auto &VISAMetric = VISACost.loopBodyCost.C;
+        CInfo.Costs.push_back(
+            {(int)VISAMetric.cycles, (int)VISAMetric.loadBytes,
+             (int)VISAMetric.storeBytes, VISACost.numChildLoops});
+        // Count the number of top-level loops for the kernel.
+        if (VISACost.nestingLevel == 1)
+          CInfo.Costs[0].NumLoops += 1;
+      }
+    } else
+      IGC_ASSERT_MESSAGE(0, "Failed to get cost metrics from finalizer");
+
+    // Finalize cost expressions.
+    auto &LI = RI.getAnalysis<LoopInfoWrapperPass>(*FG.getHead()).getLoopInfo();
+    SmallVector<Loop *, 4> Loops = LI.getLoopsInPreorder();
+    for (auto *L : Loops) {
+      auto Expression = vc::restoreLCEFromMetadata(*L);
+      if (Expression.IsUndef || Expression.Factor == 0.0f) {
+        CInfo.Expressions.push_back({Expression.Factor, -1, Expression.Addend});
+        continue;
+      }
+      CostInfoT::ArgSymInfo Symbol{
+          Expression.Symbol.Num, Expression.Symbol.Offset,
+          Expression.Symbol.Size, Expression.Symbol.IsIndirect};
+      auto SymIt =
+          std::find(CInfo.Symbols.begin(), CInfo.Symbols.end(), Symbol);
+      int Idx = std::distance(CInfo.Symbols.begin(), SymIt);
+      if (SymIt == CInfo.Symbols.end())
+        CInfo.Symbols.push_back(Symbol);
+      CInfo.Expressions.push_back({Expression.Factor, Idx, Expression.Addend});
+    }
+  }
+  return CompiledKernel{std::move(Info), std::move(CInfo), *JitInfo,
+                        std::move(gtpin), std::move(DebugData)};
 }
 
 // Goes through function groups in FGRange and collects their vISA asms into a
@@ -940,7 +1006,8 @@ RuntimeInfoCollector::collectFunctionSubgroupsInfo(
                                    Decl.getName().str());
   Info.Func.Data.Buffer = TextSection.Data.emitConsolidatedData();
 
-  return CompiledKernel{std::move(Info), vISA::FINALIZER_INFO{},
+  return CompiledKernel{std::move(Info), /*CostInfo*/ {},
+                        vISA::FINALIZER_INFO{},
                         /*GtpinInfo*/ {}, std::move(DebugInfo)};
 }
 
@@ -951,6 +1018,7 @@ void GenXOCLRuntimeInfo::getAnalysisUsage(AnalysisUsage &AU) const {
   AU.addRequired<GenXDebugInfo>();
   AU.addRequired<TargetPassConfig>();
   AU.addRequired<GenXGlobalUniformAnalysis>();
+  AU.addRequired<LoopInfoWrapperPass>();
   AU.setPreservesAll();
 }
 
@@ -983,5 +1051,6 @@ INITIALIZE_PASS_DEPENDENCY(GenXModule);
 INITIALIZE_PASS_DEPENDENCY(GenXDebugInfo);
 INITIALIZE_PASS_DEPENDENCY(TargetPassConfig);
 INITIALIZE_PASS_DEPENDENCY(GenXGlobalUniformAnalysis);
+INITIALIZE_PASS_DEPENDENCY(LoopInfoWrapperPass);
 INITIALIZE_PASS_END(GenXOCLRuntimeInfo, "GenXOCLRuntimeInfo",
                     "GenXOCLRuntimeInfo", false, true)
