@@ -170,7 +170,7 @@ void StackAnalysis::visitFunction(Function &F) {
 llvm::Optional<std::pair<uint64_t, alignment_t>>
 StackAnalysis::checkFunction(Function &F) {
   auto pOnF = m_ProcessedFs.find(&F);
-  IGC_ASSERT_MESSAGE(pOnF != m_ProcessedFs.end(),
+  IGC_ASSERT_EXIT_MESSAGE(pOnF != m_ProcessedFs.end(),
                      "Function must be inserted before checking");
 
   auto &StateOfF = pOnF->second;
@@ -304,16 +304,11 @@ void StackAnalysis::doAnalysis(Module &M) {
 bool GenXStackUsage::runOnModule(Module &M) {
   if (!PerformStackAnalysis)
     return false;
-  auto ST = &getAnalysis<TargetPassConfig>()
-                 .getTM<GenXTargetMachine>()
-                 .getGenXSubtarget();
   auto BEConf = &getAnalysis<GenXBackendConfig>();
 
   bool ModuleModified = false;
 
   auto MemSize = BEConf->getStatelessPrivateMemSize();
-  if (!ST->isOCLRuntime())
-    MemSize = visa::StackPerThreadScratch;
 
   const DataLayout &DL = M.getDataLayout();
   CallGraph CG(M);
