@@ -26,12 +26,15 @@ SPDX-License-Identifier: MIT
 #ifndef GENXSUBTARGET_H
 #define GENXSUBTARGET_H
 
+#include "GenX.h"
 #include "visa_igc_common_header.h"
+
 #include "llvm/ADT/StringSwitch.h"
 #include "llvm/ADT/Triple.h"
 #include "llvm/CodeGen/TargetSubtargetInfo.h"
 #include "llvm/IR/CallingConv.h"
 #include "llvm/Pass.h"
+
 #include <string>
 
 #define GET_SUBTARGETINFO_HEADER
@@ -171,6 +174,9 @@ private:
   /// True if subtarget supports half SIMD LSC messages
   bool HasHalfSIMDLSC = false;
 
+  /// True if subtarget supports sampler messages
+  bool HasSampler = false;
+
   /// Has multi-tile.
   bool HasMultiTile = false;
 
@@ -263,6 +269,9 @@ public:
   // can take up.
   unsigned getLSCMaxDataRegisters() const { return 8; }
 
+  unsigned getSamplerMinWidth() const { return GRFByteSize / genx::DWordBytes; }
+  unsigned getSamplerMaxWidth() const { return 2 * getSamplerMinWidth(); }
+
   // ParseSubtargetFeatures - Parses features string setting specified
   // subtarget options.  Definition of function is auto generated by tblgen.
   void ParseSubtargetFeatures(StringRef CPU, StringRef TuneCPU, StringRef FS);
@@ -304,6 +313,8 @@ public:
 
   bool noLegacyDataport() const { return NoLegacyDataport; }
 
+  bool hasSampler() const { return HasSampler; }
+
   /// * hasPackedFloat - true if packed float immediate vector operands are
   /// supported
   bool hasPackedFloat() const { return HasPackedFloat; }
