@@ -247,6 +247,8 @@ void initializeGenXFinalizerPass(PassRegistry &);
 void initializeGenXBuiltinFunctionsPass(PassRegistry &);
 void initializeGenXSLMResolutionPass(PassRegistry &);
 void initializeGenXLscAddrCalcFoldingPass(PassRegistry &);
+void initializeGenXDetectPointerArgPass(PassRegistry &);
+void initializeGenXLCECalculationPass(PassRegistry &);
 } // End llvm namespace
 
 #endif
