@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2020-2023 Intel Corporation
+Copyright (C) 2020-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -32,7 +32,6 @@ SPDX-License-Identifier: MIT
 #include <algorithm>
 #include <cstdint>
 #include <iterator>
-#include <unordered_map>
 #include <vector>
 
 namespace llvm {
@@ -129,8 +128,48 @@ Instruction *convertShlShr(Instruction *Inst);
 bool splitStructPhis(Function *F);
 bool splitStructPhi(PHINode *Phi);
 
-// Get original value before bit-casting chain.
-Value *getBitCastedValue(Value *V);
+// Is an instruction or contant expression bitcast.
+inline bool isABitCast(const Value *const V) {
+  return isa<BitCastInst>(V) ||
+         (isa<ConstantExpr>(V) &&
+          cast<ConstantExpr>(V)->getOpcode() == CastInst::BitCast);
+}
+
+// Get the first User found after traversing possible single-user chain of
+// bitcasts.
+inline const User *peelBitCastsWhileSingleUserChain(const User *U) {
+  while (isABitCast(U) && std::distance(U->user_begin(), U->user_end()) == 1)
+    U = *U->user_begin();
+  return U;
+}
+
+// Get the first User found after traversing possible single-user chain of
+// bitcasts.
+inline User *peelBitCastsWhileSingleUserChain(User *U) {
+  return const_cast<User *>(
+      peelBitCastsWhileSingleUserChain(const_cast<const User *>(U)));
+}
+
+// Get Users set ignoring possible intermediate bitcast chains.
+llvm::SmallPtrSet<const User *, 4> peelBitCastsGetUsers(const Value *const V);
+// Get Users set ignoring possible intermediate bitcast chains.
+llvm::SmallPtrSet<User *, 4> peelBitCastsGetUsers(Value *const V);
+
+// Get Value def ignoring possible bitcasts chain.
+inline const Value *getBitCastedValue(const Value *V) {
+  IGC_ASSERT_MESSAGE(V, "non-null value expected");
+  while (isa<BitCastInst>(V) ||
+         (isa<ConstantExpr>(V) &&
+          cast<ConstantExpr>(V)->getOpcode() == CastInst::BitCast))
+    V = isa<BitCastInst>(V) ? cast<Instruction>(V)->getOperand(0)
+                            : cast<ConstantExpr>(V)->getOperand(0);
+  return V;
+}
+
+// Get Value def ignoring possible bitcasts chain.
+inline Value *getBitCastedValue(Value *V) {
+  return const_cast<Value *>(getBitCastedValue(const_cast<const Value *>(V)));
+}
 
 // normalize g_load with bitcasts.
 //
@@ -145,11 +184,34 @@ class Bale;
 
 bool isGlobalStore(Instruction *I);
 bool isGlobalStore(StoreInst *ST);
-bool isGlobalVolatileStore(Instruction *I, Value *GV_ = nullptr);
-
 bool isGlobalLoad(Instruction *I);
 bool isGlobalLoad(LoadInst* LI);
-bool isGlobalVolatileLoad(Instruction *I, Value *GV_ = nullptr);
+
+const Value *getAVLoadSrcOrNull(const Instruction *const I,
+                                const Value *const CmpSrc = nullptr);
+Value *getAVLoadSrcOrNull(Instruction *const I,
+                          const Value *const CmpSrc = nullptr);
+const Value *getAGVLoadSrcOrNull(const Instruction *const I,
+                                 const Value *const CmpSrc = nullptr);
+Value *getAGVLoadSrcOrNull(Instruction *const I,
+                           const Value *const CmpSrc = nullptr);
+bool isAVLoad(const Instruction *const I);
+bool isAVLoad(const Instruction *const I, const Value *const CmpSrc);
+bool isAGVLoad(const Instruction *const I,
+               const Value *const CmpGvSrc = nullptr);
+
+const Value *getAVStoreDstOrNull(const Instruction *const I,
+                                 const Value *const CmpDst = nullptr);
+Value *getAVStoreDstOrNull(Instruction *const I,
+                           const Value *const CmpDst = nullptr);
+const Value *getAGVStoreDstOrNull(const Instruction *const I,
+                                  const Value *const CmpGvDst = nullptr);
+Value *getAGVStoreDstOrNull(Instruction *const I,
+                            const Value *const CmpGvDst = nullptr);
+bool isAVStore(const Instruction *const I);
+bool isAVStore(const Instruction *const I, const Value *const CmpDst);
+bool isAGVStore(const Instruction *const I,
+                const Value *const CmpGvDst = nullptr);
 
 // Check that V is correct as value for global store to StorePtr.
 // This implies:
@@ -228,6 +290,9 @@ bool isFuncPointerVec(Value *V);
 // of value (in other words, it can be copy-coalesced).
 bool isNoopCast(const CastInst *CI);
 
+// isBFloat16Cast : test if cast operation extends bfloat or truncates to bfloat
+bool isBFloat16Cast(const Instruction *I);
+
 // ShuffleVectorAnalyzer : class to analyze a shufflevector
 class ShuffleVectorAnalyzer {
   ShuffleVectorInst *SI;
@@ -742,59 +807,129 @@ public:
 std::size_t getStructElementPaddedSize(unsigned ElemIdx, unsigned NumOperands,
                                        const StructLayout &Layout);
 
-// Determine if there is a store to global variable Addr in between of L1 and
-// L2. L1 and L2 can be either vloads or regular stores.
-// TODO: replace with getInterveningGVStoreOrNull.
-bool hasMemoryDeps(Instruction *L1, Instruction *L2, Value *Addr,
-                   const DominatorTree *DT);
-
 // Return true if V is rdregion from a load result.
-bool isRdRFromGlobalLoad(Value *V);
+bool isRdRWithOldValueVLoadSrc(Value *V);
 
 // Return true if wrregion has result of load as old value.
-bool isWrRToGlobalLoad(Value *V);
+bool isWrRWithOldValueVLoadSrc(Value *V);
 
 //----------------------------------------------------------
-// Check that a Store potentially clobbers Load's Use.
-//----------------------------------------------------------
-// G = global value, L = load(G), S = store(G), U = use(L)
-// S clobbers L if there's a path from S to U not through
-// L: { S -> U } != { S -> L -> U }
-//----------------------------------------------------------
-// This helper is similar to but not the same as hasMemoryDeps.
-// It also properly works with loops (e.g. in case SI follows UI in a loop
-// body).
-// NB: This function may return a call to a user function as a potential
+// If there's a genx.vload kill on the path (if this path exists) from
+// LI to PosI, return it, otherwize returns nullptr.
+// + NB: In VC BE semantics genx.vload value is not translated to any VISA
+// instruction, rather it signifies either a reference to an object pinned in
+// the register file (if genx.vload loads from a global value with genx_volatile
+// attribute) or a kernel parameter passed by reference (in which case
+// genx.vload is lowered early in pipeline to simple load and later converted to
+// ssa form by mem2reg).
+// + NB: This function may return a call to a user function as a potential
 //     intervening store. If CallSites is supplied, we'll limit our lookup to
 //     the call instructions mentioned in this list.
-// NB: There may be multiple intervening stores, for now we don't have
+// + NB/TBD: There may be multiple intervening stores, for now we don't have
 //     a use case where we want the whole list, hence this variant
-//     only returns the first found.
-Instruction *getInterveningGVStoreOrNull(
-    Instruction *LI, Instruction *UIOrPos,
-    llvm::SetVector<Instruction *> *CallSites = nullptr);
+//     only returns the first one found.
+//----------------------------------------------------------
+// P = value pointer, L = genx.vload(P), S = genx.vstore(P), U = use(L)
+// S clobbers L if there's a path from S to U not through L:
+// { S -> U } != { S -> L -> U }
+//----------------------------------------------------------
+const Instruction *getAVLoadKillOrNull(
+    const Instruction *FromVLoad, const Instruction *PosI,
+    const bool ChangeSearchDirectionBasedOnDominance = false,
+    bool PosProvedReachableFromVLoad = false,
+    const DominatorTree *const DT = nullptr,
+    const SmallPtrSet<const BasicBlock *, 2>
+        *const ExcludeBlocksOnCfgTraversal = nullptr,
+    const llvm::SmallVector<const Instruction *, 8> *const KillCallSites =
+        nullptr);
 
-bool checkGVClobberingByInterveningStore(
-    Instruction *LI, llvm::GenXBaling *Baling, llvm::GenXLiveness *Liveness,
-    const llvm::StringRef PassName,
-    llvm::SetVector<Instruction *> *SIs = nullptr);
+llvm::SmallPtrSet<const Instruction *, 1> getSrcVLoads(const Instruction *I);
+llvm::SmallPtrSet<Instruction *, 1> getSrcVLoads(Instruction *I);
+const Instruction *getSrcVLoadOrNull(const Instruction *const I);
+Instruction *getSrcVLoadOrNull(Instruction *const I);
+bool hasVLoadSource(const Instruction *const I);
 
-void collectRelatedCallSitesPerFunction(
-    Instruction *SI, class FunctionGroup *FG,
-    std::unordered_map<Function *, llvm::SetVector<Instruction *>>
-        &InstAndRelatedCallSitesPerFunction);
+// genx_volatile genx.vload's users sanity check.
+// + A genx_volatile genx.vload's "forbidden use" is defined as a use on IR
+// instruction whos optimization can not be controlled by VC backend-owned
+// codebase and can result in clobbering after standard LLVM optimizations.
+// Example:
+// ------ mem2reg can optimize this sequence ------
+// L=genx.vload(@genx_volatile_global*)
+// store(L,%simple_global.localized*)
+// <... no stores into %simple_global.localized* ...>
+// <... possible genx.vstore(Y, @genx_volatile_global*) ...>
+// L2 = load(%simple_global.localized*)
+// use(L2)
+//-------- into --------------
+// L=genx.vload(@genx_volatile_global*)
+// <... possible genx.vstore(Y, @genx_volatile_global*) ...>
+// use(L)
+// ---------------------------
+// Which is clobbering situation, since L is not a VISA instruction in VC BE
+// semantics, but only signifies a pointer to an object pinned in the register
+// file.
+inline bool isAGVLoadForbiddenUser(const User *U) {
+  U = genx::peelBitCastsWhileSingleUserChain(
+      U); // For simplicity a bitcast having multiple users is considered
+          // forbidden, but bitcasts in single-use chain are not.
+  return !vc::isAnyVcIntrinsic(U) &&
+         !isa<PHINode>(U); // Currently PHI nodes are handeled by GenXCoalescing
+                           // by inserting phicopy
+}
 
-llvm::SetVector<Instruction *> getAncestorGVLoads(Instruction *I,
-                                                  bool OneLevel = false);
-Instruction *getAncestorGVLoadOrNull(Instruction *I, bool OneLevel = false);
-bool hasGVLoadSource(Instruction *I);
+// Safety checks for vloads clobbering when sinking a bale of instructions.
+bool isSafeToSink_CheckAVLoadKill(const Bale &B, const Instruction *const To,
+                                  const DominatorTree *const DT = nullptr);
+// Safety checks for vloads clobbering when sinking a bale of instructions..
+bool isSafeToSink_CheckAVLoadKill(const Instruction *const I,
+                                  const Instruction *const To,
+                                  const GenXBaling *const Baling,
+                                  const DominatorTree *const DT = nullptr);
 
-bool isSafeToMoveBaleCheckGVLoadClobber(const Bale &B, Instruction *To);
-bool isSafeToMoveInstCheckGVLoadClobber(
-    Instruction *I, Instruction *To,
-    bool OnlyImmediateGVLoadPredecessors = true);
-bool isSafeToMoveInstCheckGVLoadClobber(Instruction *I, Instruction *To,
-                                        GenXBaling *Baling_);
+// Safety checks for vloads clobbering when looking to hoist or sink an
+// instruction.
+// + NB: Is also reused by isSafeToReplace_CheckAVLoadKillOrForbiddenUser(...).
+// + NB: Besides checking for a vload kill, take into account that instruction
+// "I" may not dominate "To" but if it has any VLoad sources - we'll not move
+// them by ourselves even if there'd be no clobbering. Hence make sure that if
+// any, all of them dominate "To" position by the time this check is called.
+// + NB: If const DominatorTree * is not passed we assume that both "I" and
+// potential VLoad sources of "I" dominate "To".
+bool isSafeToMove_CheckAVLoadKill(const Instruction *const I,
+                                  const Instruction *const To,
+                                  const DominatorTree *const DT);
+
+// Safety checks for vloads clobbering when replacing an instruction.
+// + NB: When used for checks on hoisting, const DominatorTree* is mandatory and
+// must not be nullptr.
+bool isSafeToReplace_CheckAVLoadKillOrForbiddenUser(
+    const Instruction *const OldI, const Instruction *const NewI,
+    const DominatorTree *const DT);
+
+// Safety checks for genx_volatile genx.vloads clobbering when
+// targeting to modify a use on a UseTarget instruction.
+// + NB: UseSrc must dominate UseTarget.
+// + NB: This API can be augmented to optionally avoid the "forbidden user"
+// check on certain pipeline stages when we know that no further standard LLVM
+// optimizations shall be run.
+// + NB: This API can be augmented by returning metadata
+// differentiating between "kill" and "forbidden user" cases,
+// returning "forbidden user" to make further legalization on the caller side.
+bool isSafeToUse_CheckAVLoadKillOrForbiddenUser(
+    const Instruction *const UseSrc, const Instruction *const UseTarget,
+    const DominatorTree *const DT);
+
+// See isAGVLoadForbiddenUser(...) description.
+bool legalizeGVLoadForbiddenUsers(Instruction *GVLoad);
+
+bool vloadsReadSameValue(const Instruction *L1, const Instruction *L2,
+                         const DominatorTree *const DT);
+
+// TBD: there's another, a bit different, isBitwiseIdentical in CMABI, make it
+// common.
+bool isBitwiseIdentical(const Value *V1, const Value *V2,
+                        const DominatorTree *const DT = nullptr);
 
 } // namespace genx
 } // namespace llvm
