@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2022 Intel Corporation
+Copyright (C) 2022-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -104,7 +104,7 @@ void GenXCloneIndirectFunctions::visitFunction(Function &F) {
     return;
   if (vc::InternalIntrinsic::isInternalNonTrivialIntrinsic(&F))
     return;
-  if (vc::isEmulationFunction(F))
+  if (vc::isBuiltinFunction(F))
     return;
   if (vc::isKernel(&F))
     return;
@@ -146,10 +146,13 @@ bool GenXCloneIndirectFunctions::runOnModule(Module &M) {
 
   auto &&BECfg = getAnalysis<GenXBackendConfig>();
   IGC_ASSERT_MESSAGE(
-    llvm::none_of(M.functions(),
-      [&](const Function& F) { return F.hasAddressTaken() && BECfg.directCallsOnly(F.getName()); }),
-    "A function has address taken inside the module that contradicts "
-    "DirectCallsOnly option");
+      llvm::none_of(M.functions(),
+                    [&](const Function &F) {
+                      return F.hasAddressTaken() &&
+                             BECfg.directCallsOnly(F.getName());
+                    }),
+      "A function has address taken inside the module that contradicts "
+      "DirectCallsOnly option");
 
   // If direct calls are forced for all functions.
   if (BECfg.directCallsOnly()) {
@@ -160,8 +163,9 @@ bool GenXCloneIndirectFunctions::runOnModule(Module &M) {
 
   bool Modified = false;
 
-  for (auto [F, IsExternal] : IndirectFuncs) {
-    if (BECfg.directCallsOnly(F->getName())) continue;
+  for (auto &[F, IsExternal] : IndirectFuncs) {
+    if (BECfg.directCallsOnly(F->getName()))
+      continue;
 
     auto CheckDirectCall = [Func = F](User *U) {
       auto *CI = dyn_cast<CallInst>(U);
