@@ -59,6 +59,18 @@ enum {
     // attribute and section changes. The content is stored in a nul-terminated
     // string and the format is "<Major number>.<Minor number>".
     NT_INTELGT_ZEBIN_VERSION = 4,
+    // The description represents VISA ABI version used in generated code.
+    // Note that VISA ABI is valid only when stack calls are used. Without
+    // stack calls, VISA ABI field may be absent.
+    NT_INTELGT_VISA_ABI_VERSION = 5,
+    // The description is stored in a 4-byte ELF word and is used instead
+    // of NT_INTELGT_PRODUCT_FAMILY, NT_INTELGT_GFXCORE_FAMILY and
+    // NT_INTELGT_TARGET_METADATA because it contains all required info to
+    // validate program for a target device
+    NT_INTELGT_PRODUCT_CONFIG = 6,
+    // The description is the version of Indirect Access Detection implementation
+    // stored in a 4-byte ELF word.
+    NT_INTELGT_INDIRECT_ACCESS_DETECTION_VERSION = 7,
 };
 
 struct TargetMetadata {
@@ -69,7 +81,8 @@ struct TargetMetadata {
     // bit[23:21]: generator of this device binary
     enum GeneratorId : uint8_t {
         UNREGISTERED = 0,
-        IGC          = 1
+        IGC          = 1,
+        NGEN         = 2
     };
 
     union {
