@@ -426,6 +426,10 @@ public:
     // createKernelMiscInfo - create a zeKernelMiscInfo and add it into zeInfoContainer
     zeInfoKernelMiscInfo& createKernelMiscInfo(const std::string& name);
 
+    // createKernelCostInfo - create a zeKernelCostInfo and add it into zeInfoContainer
+    // zeKernelCostInfo: experimental kernel cost expressions.
+    zeInfoKernelCostInfo& createKernelCostInfo(const std::string &name);
+
     // addGlobalHostAccessSymbol - create a zeInfo global_host_access_table section
     // which is used by Runtime to identify a global variable based on host name
     void addGlobalHostAccessSymbol(const std::string& device_name, const std::string& host_name);
