@@ -36,7 +36,7 @@ namespace IGC
     {
         PRECISION_UNUSED, U8, U4, U2, S8, S4, S2,
         BF8,
-        TF32,
+        TF32 = 10,
         BF16, FP16
     };
 
@@ -132,7 +132,6 @@ enum class ShaderType
     COMPUTE_SHADER,
     OPENCL_SHADER,
     RAYTRACING_SHADER,
-    BINDLESS_SHADER,
     END,
     BEGIN = 0
 };
@@ -160,7 +159,6 @@ static const char *ShaderTypeString[] = {
     "CS",
     "OCL",
     "RAYDISPATCH",
-    "BINDLESS",
     "ERROR"
 };
 
