@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -196,7 +196,7 @@ DumpName DumpName::Pass(std::string const& name, llvm::Optional<unsigned int> in
         [](const char s) { return s == '/' || s == '\\'; }, '_');
     IGC_ASSERT_MESSAGE(newName.find(' ') == std::string::npos, "Pass name must not contain spaces");
     DumpName copy(*this);
-    CPassDescriptor pd = { newName, index };
+    CPassDescriptor pd = { std::move(newName), index };
     copy.m_pass = pd;
     return copy;
 }
@@ -545,6 +545,8 @@ namespace
                 delete m_pUnder;
             }
         }
+        PrefixStream(const PrefixStream&) = delete;
+        PrefixStream& operator=(const PrefixStream&) = delete;
 
         virtual uint64_t current_pos() const override
         {
