@@ -30,8 +30,9 @@ SPDX-License-Identifier: MIT
 ///
 //===----------------------------------------------------------------------===//
 
+#define LSC_CACHE_CTRL_NUM 12
 #ifdef LSC_CACHE_CTRL_OPTION
-    #define LSC_CACHE_CTRL_OPTIONS                                                                                                      \
+    #define LSC_CACHE_CTRL_OPTIONS_WITH_CTRL_NUM(ctrl_num)                                                                                                      \
         LSC_CACHE_CTRL_OPTION(LSC_L1DEF_L3DEF,     0,  "default")                                                                       \
         LSC_CACHE_CTRL_OPTION(LSC_L1UC_L3UC,       1,  "Load: L1 uncached   L3 uncached # Store: L1 uncached      L3 uncached")         \
         LSC_CACHE_CTRL_OPTION(LSC_L1UC_L3C_WB,     2,  "Load: L1 uncached   L3 cached   # Store: L1 uncached      L3 write-back")       \
@@ -40,8 +41,14 @@ SPDX-License-Identifier: MIT
         LSC_CACHE_CTRL_OPTION(LSC_L1S_L3UC,        5,  "Load: L1 streaming  L3 uncached # Store: L1 streaming     L3 uncached")         \
         LSC_CACHE_CTRL_OPTION(LSC_L1S_L3C_WB,      6,  "Load: L1 streaming  L3 cached   # Store: L1 streaming     L3 write-back")       \
         LSC_CACHE_CTRL_OPTION(LSC_L1IAR_WB_L3C_WB, 7,  "Load: L1 invalidate after read L3 cached # Store: L1 write-back L3 write-back") \
+        LSC_CACHE_CTRL_OPTION(LSC_L1UC_L3CC,       8,  "Load: L1 uncached   L3 const-cached # Store: N/A")                              \
+        LSC_CACHE_CTRL_OPTION(LSC_L1C_L3CC,        9,  "Load: L1 cached     L3 const-cached # Store: N/A")                              \
+        LSC_CACHE_CTRL_OPTION(LSC_L1IAR_L3IAR,     10, "Load: L1 invalidate after read L3 invalidate after read # Store: N/A")          \
                                                                                                                                         \
-        LSC_CACHE_CTRL_OPTION(LSC_CC_INVALID,      8,  "Invalid")
+        LSC_CACHE_CTRL_OPTION(LSC_CC_INVALID,      11, "Invalid") \
+        LSC_CACHE_CTRL_OPTION(LSC_CC_NUM,          ctrl_num, "Invalid")
+
+    #define LSC_CACHE_CTRL_OPTIONS LSC_CACHE_CTRL_OPTIONS_WITH_CTRL_NUM(LSC_CACHE_CTRL_NUM)
 #endif // LSC_CACHE_CTRL_OPTION
 
 #ifdef EARLY_OUT_CS_PATTERN
@@ -71,10 +78,11 @@ SPDX-License-Identifier: MIT
             SHADER_TYPE_MASK(HS,       0x4) \
             SHADER_TYPE_MASK(DS,       0x8) \
             SHADER_TYPE_MASK(GS,       0x10)\
+            SHADER_TYPE_MASK(TS,       0x20)\
+            SHADER_TYPE_MASK(MS,       0x40)\
             SHADER_TYPE_MASK(PS,       0x80)\
             SHADER_TYPE_MASK(CS,       0x100)   \
-            SHADER_TYPE_MASK(MS,       0x40)\
-            SHADER_TYPE_MASK(TS,       0x20)\
+            SHADER_TYPE_MASK(OCL,      0x200)   \
             SHADER_TYPE_MASK(RT,       0x400)
 #endif // SHADER_TYPE_MASKS
 
@@ -93,8 +101,3 @@ SPDX-License-Identifier: MIT
     TRIBOOL_OPTION(Enabled,  1)
 #endif // TRIBOOL_OPTION
 
-#ifdef RTMEMORY_STYLE_OPTION
-#define RTMEMORY_STYLE_OPTIONS      \
-    RTMEMORY_STYLE_OPTION(Auto, -1) \
-    RTMEMORY_STYLE_OPTION(Xe,    0)
-#endif // RTMEMORY_STYLE_OPTION
