@@ -38,6 +38,8 @@ typedef enum {
     IGFX_PVC = 1271,
     IGFX_METEORLAKE = 1272,
     IGFX_ARROWLAKE = 1273,
+    IGFX_BMG = 1274,
+    IGFX_LUNARLAKE = 1275,
     IGFX_MAX_PRODUCT,
 
 
@@ -89,7 +91,7 @@ typedef enum {
     IGFX_XE_HP_CORE   = 0x0c05, // XeHP Family
     IGFX_XE_HPG_CORE = 0x0c07,  // XE_HPG Family
     IGFX_XE_HPC_CORE = 0x0c08,  // XE_HPC Family
-    IGFX_XE2_LPG_CORE = 0x0c09, // XE2_LPG Family
+    IGFX_XE2_HPG_CORE = 0x0c09, // XE2_HPG Family
     IGFX_MAX_CORE,              // Max Family, for lookup table
 
     IGFX_GENNEXT_CORE          = 0x7ffffffe,  //GenNext
@@ -147,6 +149,10 @@ typedef struct GFX_GMD_ID_DEF
 
 #define GFX_GMD_ARCH_12_RELEASE_XE_LP_MD                 (70)
 #define GFX_GMD_ARCH_12_RELEASE_XE_LP_LG                 (71)
+#define GFX_GMD_ARCH_12_RELEASE_XE_LPG_PLUS_1274         (74)
+
+#define GFX_GMD_ARCH_20_RELEASE_XE2_HPG_X2               (1)
+#define GFX_GMD_ARCH_20_RELEASE_XE2_LPG                  (4)
 
 #define GFX_GET_GMD_RELEASE_VERSION_RENDER(p)             ((p).sRenderBlockID.GmdID.GMDRelease)
 #define GFX_GET_GMD_RELEASE_VERSION_DISPLAY(p)            ((p).sDisplayBlockID.GmdID.GMDRelease)
@@ -716,8 +722,25 @@ typedef enum __NATIVEGTTYPE
 #define DEV_ID_56B3                             0x56B3
 #define DEV_ID_56C0                             0x56C0
 #define DEV_ID_56C1                             0x56C1
+#define DEV_ID_56C2                             0x56C2
 #define DEV_ID_56CF                             0x56CF
 
+// ARL
+#define DEV_ID_7D67                             0x7D67
+
+// PVC
+#define DEV_ID_0BD0                            0x0BD0
+#define DEV_ID_0BD4                            0x0BD4
+#define DEV_ID_0BD5                            0x0BD5
+#define DEV_ID_0BD6                            0x0BD6
+#define DEV_ID_0BD7                            0x0BD7
+#define DEV_ID_0BD8                            0x0BD8
+#define DEV_ID_0BD9                            0x0BD9
+#define DEV_ID_0BDA                            0x0BDA
+#define DEV_ID_0BDB                            0x0BDB
+#define DEV_ID_0B69                            0x0B69
+#define DEV_ID_0B6E                            0x0B6E
+
 #define GFX_IS_DG2_G11_CONFIG(d) ( ( d == DEV_ID_56A5 )             ||   \
                                  ( d == DEV_ID_56A6 )             ||   \
                                  ( d == DEV_ID_5693 )             ||   \
@@ -729,13 +752,14 @@ typedef enum __NATIVEGTTYPE
                                  ( d == DEV_ID_4F87 )             ||   \
                                  ( d == DEV_ID_4F88 ))
 
-#define GFX_IS_DG2_G10_CONFIG(d) ( ( d == DEV_ID_56A0 )                              ||   \
+#define GFX_IS_DG2_G10_CONFIG(d) (    ( d == DEV_ID_56A0 )                              ||   \
                                       ( d == DEV_ID_56A1 )                              ||   \
                                       ( d == DEV_ID_56A2 )                              ||   \
                                       ( d == DEV_ID_5690 )                              ||   \
                                       ( d == DEV_ID_5691 )                              ||   \
                                       ( d == DEV_ID_5692 )                              ||   \
                                       ( d == DEV_ID_56C0 )                              ||   \
+                                      ( d == DEV_ID_56C2 )                              ||   \
                                       ( d == DEV_ID_4F80 )                              ||   \
                                       ( d == DEV_ID_4F81 )                              ||   \
                                       ( d == DEV_ID_4F82 )                              ||   \
@@ -751,4 +775,17 @@ typedef enum __NATIVEGTTYPE
                                       ( d == DEV_ID_56B2 )                              ||   \
                                       ( d == DEV_ID_56B3 ))
 
+#define GFX_IS_ARL_S(d)  ( ( d == DEV_ID_7D67 ) )
+
+#define GFX_IS_XT_CONFIG(d) ((d == DEV_ID_0BD5) || \
+                             (d == DEV_ID_0BD6) || \
+                             (d == DEV_ID_0BD7) || \
+                             (d == DEV_ID_0BD8) || \
+                             (d == DEV_ID_0BD9) || \
+                             (d == DEV_ID_0BDA) || \
+                             (d == DEV_ID_0BDB) || \
+                             (d == DEV_ID_0B69) || \
+                             (d == DEV_ID_0B6E))
+#define GFX_IS_VG_CONFIG(d) ((d == DEV_ID_0BD4))
+
 #endif
