@@ -83,6 +83,9 @@ private:
     _nextArena = 0;
   }
 
+  ArenaHeader(const ArenaHeader&) = delete;
+  ArenaHeader& operator=(const ArenaHeader&) = delete;
+
   void *AllocSpace(size_t size, size_t align);
 
   // Data
@@ -105,6 +108,8 @@ private:
   }
 
   ~ArenaManager() { FreeArenas(); }
+  ArenaManager(const ArenaManager&) = delete;
+  ArenaManager& operator=(const ArenaManager&) = delete;
 
   void *AllocDataSpace(size_t size, size_t al) {
     // Do separate memory allocations of debugMemAlloc is set, to allow
