@@ -128,7 +128,7 @@ public:
   uint32_t instNumber;      // Global instruction number
   bool compacted = false;
 
-  uint64_t genOffset;
+  uint64_t genOffset = 0;
 
   BinInst() {
     DWords[0] = 0;
@@ -1156,7 +1156,7 @@ public:
   _CompactSourceTable3Src_ CompactSourceTable3Src;
   _CompactSourceTable3SrcCHV_ CompactSourceTable3SrcCHV;
 
-  BinaryEncodingBase(Mem_Manager &m, G4_Kernel &k, std::string fname)
+  BinaryEncodingBase(Mem_Manager &m, G4_Kernel &k, const std::string& fname)
       : BDWCompactControlTable(m), BDWCompactSourceTable(m),
         BDWCompactSubRegTable(m), BDWCompactDataTypeTableStr(m),
         CompactControlTable3Src(k.getPlatform()),
@@ -2402,7 +2402,7 @@ inline bool BinaryEncodingBase::uncompactOneInstruction(G4_INST *inst) {
   uint32_t subRegIndex100 = CompactSubRegTable.GetBits_100_096(subRegIndex);
   uint32_t subRegIndex68 = CompactSubRegTable.GetBits_068_064(subRegIndex);
   uint32_t subRegIndex52 = CompactSubRegTable.GetBits_052_048(subRegIndex);
-  uint32_t condModifer = GetCondModifier(mybin);
+  uint32_t condModifier = GetCondModifier(mybin);
   uint32_t accWrCtrl = GetCmpAccWrCtrl(mybin);
   uint32_t flagSubRegNum = GetCmpFlagSubRegNum(mybin);
   uint32_t bits88 = CompactSourceTable.GetBits_088_077(src0Index);
@@ -2420,7 +2420,7 @@ inline bool BinaryEncodingBase::uncompactOneInstruction(G4_INST *inst) {
   mybin->SetBits(100, 96, subRegIndex100);
   mybin->SetBits(68, 64, subRegIndex68);
   mybin->SetBits(52, 48, subRegIndex52);
-  SetCondModifier(mybin, condModifer);
+  SetCondModifier(mybin, condModifier);
   SetAccWrCtrl(mybin, accWrCtrl);
   SetFlagRegNum(mybin, flagSubRegNum);
   SetCompactCtrl(mybin, 0); // uncompaction
