@@ -32,7 +32,17 @@ bool G4_BB::isLastInstEOT() const {
     return false;
   }
 
-  G4_INST *i = instList.back();
+  // Scan backward because some pass may insert nop after EOT.
+  auto iter =
+      std::find_if(instList.rbegin(), instList.rend(), [](G4_INST *inst) {
+        return inst->opcode() != G4_nop && inst->opcode() != G4_mov;
+      });
+
+  if (iter == instList.rend()) { //There is only nop instructions
+    return false;
+  }
+
+  G4_INST *i = *iter;
 
   if (parent->builder->hasSendShootdown()) {
     // due to send shootdown, a predicated send may not actually be an EOT
@@ -198,7 +208,7 @@ void G4_BB::emitBasicInstruction(std::ostream &output, INST_LIST_ITER &it) {
 }
 void G4_BB::emitBasicInstructionComment(std::ostream &output,
                                         INST_LIST_ITER &it, int *suppressRegs,
-                                        int *lastRegs) {
+                                        int *lastRegs, int32_t pc) {
   const G4_INST *inst = *it;
 
   auto platform = inst->getPlatform();
@@ -249,6 +259,9 @@ void G4_BB::emitBasicInstructionComment(std::ostream &output,
       jitInfo->statsVerbose.BCNum += BCNum;
       jitInfo->statsVerbose.numByteRMWs += countReadModifyWrite(output, inst);
     }
+
+    if (getParent().getKernel()->getOption(vISA_PrintInstOffsetInAsm))
+      output << " inst_offset=" << pc;
   }
 }
 
@@ -578,6 +591,10 @@ int G4_BB::getConflictTimesForTGL(std::ostream &output, int *firstRegCandidate,
 
       if (reducedBundles) {
         bundleID = (firstRegCandidate[i] % 16) / 2;
+      }
+      if (parent->builder->has64bundleSize2GRFPerBank()) {
+        bankID = (firstRegCandidate[i] % 4) / 2;
+        bundleID = (firstRegCandidate[i] % 32) / 4;
       }
        // Same bank and same bundle
       if (bundles[bankID][bundleID] != -1) {
@@ -885,6 +902,13 @@ static bool hasInternalConflict(IR_Builder *builder, int reg1, int reg2) {
     bankID2 = (reg2 % 4) / 2;
   }
 
+  if (builder->has64bundleSize2GRFPerBank()) {
+    bundleID1 = (reg1 % 32) / 4;
+    bankID1 = (reg1 % 4) / 2;
+    bundleID2 = (reg2 % 32) / 4;
+    bankID2 = (reg2 % 4) / 2;
+  }
+
   if (builder->hasOneGRFBank16Bundles()) {
     bundleID1 = (reg1 % 64) / 4;
     bankID1 = reg1 % 2;
@@ -892,6 +916,13 @@ static bool hasInternalConflict(IR_Builder *builder, int reg1, int reg2) {
     bankID2 = reg2 % 2;
   }
 
+  if (builder->has64bundleSize()) {
+    bundleID1 = (reg1 % 16) / 2;
+    bankID1 = reg1 % 2;
+    bundleID2 = (reg2 % 16) / 2;
+    bankID2 = reg2 % 2;
+  }
+
   return ((bankID1 == bankID2) && (bundleID1 == bundleID2));
 }
 
