@@ -118,6 +118,8 @@ private:
 class G4_BB;
 class KernelDebugInfo;
 class VarSplitPass;
+struct KernelCostInfo;
+class KernelCost;
 
 
 // Handles information for GRF selection
@@ -187,6 +189,16 @@ public:
     return found->numGRF;
   }
 
+  // Get GRF number for initial kernel creation
+  unsigned getInitalGRFNum() const {
+    // Max GRF number is used when GRF selection is enabled.
+    // This allows input payload to be allocated to GRFs
+    // beyond 128 if needed.
+    if (options->getOption(vISA_AutoGRFSelection))
+      return configs.back().numGRF;
+    return configs[currentMode].numGRF;
+  }
+
   // Get the next larger GRF available
   unsigned getLargerGRF() const {
     // find the first larger mode that's available for VRT
@@ -546,6 +558,9 @@ private:
   std::unordered_map<G4_INST *, G4_SrcRegRegion *> instImplicitAccSrc;
   std::unordered_map<G4_INST *, G4_DstRegRegion *> instImplicitAccDef;
 
+  // Kernel cost model
+  std::unique_ptr<KernelCostInfo> m_kernelCost;
+
   // Internal use of updating shared_ptr KernelDebugInfo
   // To access the pointer, use getKernelDebugInfo() instead
   void setKernelDebugInfoSharedPtr(std::shared_ptr<KernelDebugInfo>& k) {
@@ -714,6 +729,10 @@ public:
     return gtPinInfo.get();
   }
 
+  // Kernel cost model
+  void createKernelCostInfo(KernelCost *KCA);
+  KernelCostInfo *getKernelCostInfo() { return m_kernelCost.get(); }
+
   G4_INST *setupBE_FP = nullptr;
   G4_INST *setupBE_SP = nullptr;
 
