@@ -428,7 +428,6 @@ private:
   std::unordered_map<FuncInfo *, std::unordered_set<G4_Declare *>> retValPerSub;
   std::unordered_map<G4_Declare *, std::unordered_set<FuncInfo *>> unknownArgRetvalRefs;
   std::unordered_map<G4_Declare *, std::unordered_set<FuncInfo *>> nonGRFRefs;
-  std::unordered_map<G4_BB *, FuncInfo *> bbToFunc;
   // Store home function for given variable. Home function is defined as
   // function that contains explicit def or use of the variable. Each regular
   // variable has a unique home function. Arg/retval don't have a unique home
@@ -903,6 +902,10 @@ class Interference {
 
   void markInterferenceForSend(G4_BB *bb, G4_INST *inst, G4_DstRegRegion *dst);
 
+  void setOutOfBoundForbidden(G4_Operand *opnd);
+
+  void setForbiddenGRFNumForSVMScatter(G4_INST *inst);
+
   void buildInterferenceWithLocalRA(G4_BB *bb);
 
   void buildInterferenceAmongLiveOuts();
@@ -1348,6 +1351,23 @@ public:
 
   const bool use4GRFAlign = false;
 
+  // [-2^16...2^16) in bytes
+  //   (frame pointer is biased 2^16 so that -2^16 references scratch[0x0])
+  static constexpr unsigned SPILL_FILL_IMMOFF_MAX = 0x10000; // 64k
+
+  static bool LSCUsesImmOff(IR_Builder &builder) {
+    const auto scratchAddrType = VISA_LSC_IMMOFF_ADDR_TYPE_SS;
+    const uint32_t immOffOpts =
+        builder.getuint32Option(vISA_lscEnableImmOffsFor);
+    return
+        // HW supports it
+        builder.getPlatform() >= Xe2 &&
+        // the spill/fill is enabled in options
+        (immOffOpts & (1 << VISA_LSC_IMMOFF_SPILL_FILL)) != 0 &&
+        // address type is also enabled in options
+        (immOffOpts & (1 << scratchAddrType)) != 0;
+  }
+
 private:
   template <class REGION_TYPE>
   static unsigned getRegionDisp(REGION_TYPE *region, const IR_Builder &irb);
@@ -1392,10 +1412,6 @@ private:
   // store instructions that shouldnt be rematerialized.
   std::unordered_set<G4_INST *> dontRemat;
 
-  // [-2^16...2^16) in bytes
-  //   (frame pointer is biased 2^16 so that -2^16 references scratch[0x0])
-  static constexpr unsigned SPILL_FILL_IMMOFF_MAX = 0x10000; // 64k
-
   // map each BB to its local RA GRF usage summary, populated in local RA.
   std::map<G4_BB *, PhyRegSummary *> bbLocalRAMap;
   llvm::SpecificBumpPtrAllocator<PhyRegSummary> PRSAlloc;
