@@ -419,7 +419,7 @@ void LinearScanRA::linearScanMarkReferencesInOpnd(G4_Operand *opnd, bool isEOT,
       LSLiveRange *lr = GetOrCreateLocalLiveRange(topdcl);
 
       lr->recordRef(curBB_, false);
-      if (isEOT) {
+      if (isEOT && kernel.fg.builder->hasEOTGRFBinding()) {
         lr->markEOT();
       }
       if (topdcl->getRegVar() && topdcl->getRegVar()->isPhyRegAssigned() &&
@@ -788,12 +788,6 @@ void LinearScanRA::calculateFuncLastID() {
 }
 
 int LinearScanRA::linearScanRA() {
-  if (kernel.fg.getIsStackCallFunc()) {
-    // Allocate space to store Frame Descriptor
-    nextSpillOffset += 32;
-    scratchOffset += 32;
-  }
-
   std::list<LSLiveRange *> spillLRs;
   int iterator = 0;
   uint32_t GRFSpillFillCount = 0;
@@ -916,24 +910,25 @@ int LinearScanRA::linearScanRA() {
           kernel.getInt32KernelAttr(Attributes::ATTR_Target) == VISA_3D &&
           !hasStackCall) {
         enableSpillSpaceCompression = gra.spillSpaceCompression(
-            gra.computeSpillSize(spillLRs), nextSpillOffset);
+            gra.computeSpillSize(spillLRs), gra.nextSpillOffset);
       }
 
-      SpillManagerGRF spillGRF(gra, nextSpillOffset, &l, &spillLRs,
+      SpillManagerGRF spillGRF(gra, gra.nextSpillOffset, &l, &spillLRs,
                                enableSpillSpaceCompression,
                                useScratchMsgForSpill);
 
       spillGRF.spillLiveRanges(&kernel);
-      nextSpillOffset = spillGRF.getNextOffset();
-      ;
-      scratchOffset = std::max(scratchOffset, spillGRF.getNextScratchOffset());
+      gra.nextSpillOffset = spillGRF.getNextOffset();
+      gra.scratchOffset = std::max(gra.scratchOffset, spillGRF.getNextScratchOffset());
+
 #ifdef DEBUG_VERBOSE_ON
       COUT_ERROR << "===== printSpillLiveIntervals============"
                  << "\n";
       printSpillLiveIntervals(spillLRs);
 #endif
+
       if (builder.hasScratchSurface() && !hasStackCall &&
-          (nextSpillOffset + globalScratchOffset) > SCRATCH_MSG_LIMIT) {
+          (gra.nextSpillOffset + globalScratchOffset) > SCRATCH_MSG_LIMIT) {
         // create temp variable to store old a0.2 - this is marked as live-in
         // and live-out. because the variable is emitted only post RA to
         // preserve old value of a0.2.
@@ -945,6 +940,7 @@ int LinearScanRA::linearScanRA() {
         }
       } else if (gra.useLscForNonStackCallSpillFill ||
                  gra.useLscForScatterSpill) {
+        // Xe2+ LSC-based spill/fill needs the same as above
         G4_Declare *a0Dcl = kernel.fg.builder->getOldA0Dot2Temp();
         LSLiveRange *lr = gra.getSafeLSLR(a0Dcl);
         if (!lr) {
@@ -981,8 +977,8 @@ int LinearScanRA::linearScanRA() {
           kernel.getGTPinData()->setScratchNextFree(
               8 * 1024 - kernel.getGTPinData()->getNumBytesScratchUse());
         } else {
-          jitInfo->stats.spillMemUsed = nextSpillOffset;
-          kernel.getGTPinData()->setScratchNextFree(nextSpillOffset);
+          jitInfo->stats.spillMemUsed = gra.nextSpillOffset;
+          kernel.getGTPinData()->setScratchNextFree(gra.nextSpillOffset);
         }
         jitInfo->stats.numGRFSpillFillWeighted = GRFSpillFillCount;
       }
@@ -991,7 +987,7 @@ int LinearScanRA::linearScanRA() {
 
     RA_TRACE({
       std::cout << "\titeration: " << iterator << "\n";
-      std::cout << "\t\tnextSpillOffset: " << nextSpillOffset << "\n";
+      std::cout << "\t\tnextSpillOffset: " << gra.nextSpillOffset << "\n";
       std::cout << "\t\tGRFSpillFillCount: " << GRFSpillFillCount << "\n";
     });
 
@@ -1015,7 +1011,7 @@ int LinearScanRA::linearScanRA() {
 
   if (kernel.fg.getHasStackCalls() || kernel.fg.getIsStackCallFunc()) {
     getSaveRestoreRegister();
-    unsigned localSpillAreaOwordSize = ROUND(scratchOffset, 64) / 16;
+    unsigned localSpillAreaOwordSize = ROUND(gra.scratchOffset, 64) / 16;
     gra.addSaveRestoreCode(localSpillAreaOwordSize);
   }
   return VISA_SUCCESS;
@@ -1165,7 +1161,11 @@ void LinearScanRA::setDstReferences(
     if (lr->getFirstRef(startIdx) == NULL && startIdx == 0) {
       lr->setFirstRef(curInst, curInst->getLexicalId() * 2);
     }
-    lr->setLastRef(curInst, curInst->getLexicalId() * 2);
+
+    // For dst, we set the last ref = lexcial_ID * 2 + 1.
+    // So that the dst will not be released immediately if it's defined only.
+    // This is to handle some special workload. Such as for LIT test
+    lr->setLastRef(curInst, curInst->getLexicalId() * 2 + 1);
   } else if (dcl->getRegVar()
                  ->getPhyReg()
                  ->isGreg()) // Assigned already and is GRF
@@ -1184,7 +1184,9 @@ void LinearScanRA::setDstReferences(
     if (lr->getFirstRef(startIdx) == NULL && startIdx == 0) {
       lr->setFirstRef(curInst, curInst->getLexicalId() * 2);
     }
-    lr->setLastRef(curInst, curInst->getLexicalId() * 2);
+    // For dst, we set the last ref = lexcial_ID * 2 + 1.
+    // So that the dst will not be released immediately if it's defined only.
+    lr->setLastRef(curInst, curInst->getLexicalId() * 2 + 1);
   }
 
   if (lr->isEOT() && std::find(eotLiveIntervals.begin(), eotLiveIntervals.end(),
@@ -1281,7 +1283,10 @@ void LinearScanRA::generateInputIntervals(
   unsigned int idx = regNum * builder.numEltPerGRF<Type_UW>() +
                      (regOff * topdcl->getElemSize()) / G4_WSIZE +
                      topdcl->getWordSize() - 1;
-
+  //Variable size may be larger than occupied GRF
+  if (idx > (kernel.getNumRegTotal() * builder.numEltPerGRF<Type_UW>() - 1)) {
+    idx = kernel.getNumRegTotal() * builder.numEltPerGRF<Type_UW>() - 1;
+  }
   unsigned int numWords = topdcl->getWordSize();
   for (int i = numWords - 1; i >= 0; --i, --idx) {
     if ((inputRegLastRef[idx] == UINT_MAX || inputRegLastRef[idx] < instID) &&
@@ -1542,8 +1547,7 @@ void LinearScanRA::calculateCurrentBBLiveIntervals(
         liveIntervals.push_back(stackCallRetLR);
         stackCallRetLR->setPushed(true);
       }
-      if (arg && argSize > 0 && arg->getRegVar()) {
-        vASSERT(stackCallArgLR);
+      if (stackCallArgLR && arg && argSize > 0 && arg->getRegVar()) {
         stackCallArgLR->setLastRef(
             curInst, curInst->getLexicalId() * 2 -
                          1); // Minus one so that arguments will not be spilled
@@ -1570,7 +1574,7 @@ void LinearScanRA::calculateCurrentBBLiveIntervals(
         if (src->asSrcRegRegion()->isIndirect()) {
           auto pointsToSet = l.getPointsToAnalysis().getAllInPointsTo(
               src->getBase()->asRegVar());
-          for (auto pt : *pointsToSet) {
+          for (const auto& pt : *pointsToSet) {
             G4_Declare *dcl = pt.var->getDeclare()->getRootDeclare();
 
             setSrcReferences(bb, inst_it, i, dcl, liveIntervals,
@@ -1593,7 +1597,7 @@ void LinearScanRA::calculateCurrentBBLiveIntervals(
       if (dst->isIndirect()) {
         auto pointsToSet = l.getPointsToAnalysis().getAllInPointsTo(
             dst->getBase()->asRegVar());
-        for (auto pt : *pointsToSet) {
+        for (const auto& pt : *pointsToSet) {
           G4_Declare *dcl = pt.var->getDeclare()->getRootDeclare();
 
           setDstReferences(bb, inst_it, dcl, liveIntervals, eotLiveIntervals);
