@@ -319,7 +319,8 @@ bool LiveNode::alignedWithChannelMask(G4_INST *DefInst,
   }
 
   // Do not analyze instructions that may exceed two GRF boundary
-  if (DefInst->mayExceedTwoGRF() || UseInst->mayExceedTwoGRF())
+  if (DefInst->isSend() || DefInst->isDpas() || UseInst->isSend() ||
+      UseInst->isDpas())
     return true;
 
   // Check that all uses are defined under the righ emask, if defined.
@@ -348,7 +349,7 @@ bool LiveNode::alignedWithChannelMask(G4_INST *DefInst,
   // Other cases.
   //
   // Initial value -1 means byte not defined, for [DefLB, DefRB].
-  std::array<int, 128> DefByteMask;
+  std::array<int, 256> DefByteMask;
   DefByteMask.fill(-1);
 
   // Set channel value for each defining byte.
@@ -605,7 +606,7 @@ void DefEscapeBBAnalysis::analyzeBB(G4_BB *bb) {
       escapedDefs.push_back(I);
     }
   }
-  escapedInsts[bb] = escapedDefs;
+  escapedInsts[bb] = std::move(escapedDefs);
 }
 
 void DefEscapeBBAnalysis::print(std::ostream &OS) const {
