@@ -126,6 +126,7 @@ class Optimizer {
   bool foldPseudoNot(G4_BB *bb, INST_LIST_ITER &iter);
   bool createSmov(G4_BB *bb, G4_INST *flagMove, G4_INST *nextInst);
   bool foldCmpToCondMod(G4_BB *BB, INST_LIST_ITER &iter);
+  void sinkBarrierWait();
   void HWWorkaround();
   void preRA_HWWorkaround();
   void postRA_HWWorkaround();
@@ -134,7 +135,6 @@ class Optimizer {
   void dumpPayload();
   void collectStats();
   void createR0Copy();
-
   void fixEndIfWhileLabels();
   void mergeScalarInst();
   void EmulateInt64Add() { ::EmulateInt64Add(builder, kernel); }
@@ -189,6 +189,8 @@ class Optimizer {
 
   void accSubBeforeRA();
 
+  void HWDebug();
+
 
   // return true if BuiltInR0 gets a different allocation than r0
   bool R0CopyNeeded();
@@ -210,6 +212,7 @@ private:
   void hoistBarrierHeaderToTop(G4_SrcRegRegion *);
   /* end of member functions for message header opt */
   void cleanupBindless();
+  void cleanupA0Movs();
   G4_Operand *updateSendsHeaderReuse(std::vector<std::vector<G4_INST *>> &,
                                      std::vector<G4_INST *> &, INST_LIST_ITER);
   void countGRFUsage();
@@ -237,6 +240,7 @@ private:
   void linePlaneWA(G4_INST *inst);
   void fixSendSrcRegion(G4_INST *inst);
   void clearARFDependencies();
+  void mulMacRSWA();
   void clearSendDependencies();
   void loadThreadPayload();
   void addFFIDProlog();
@@ -282,6 +286,8 @@ private:
   // In both case, the dst of inst_with_ip is used to save IP.
   void replaceIPWithCall(InstListType &insts, G4_INST *inst_with_ip);
 
+  void insertDummyAdd(G4_BB *bb, INST_LIST_ITER inst_it, int imm = 0);
+
   void insertDummyMad(G4_BB *bb, INST_LIST_ITER inst_it);
 
   void insertDummyCsel(G4_BB *bb, INST_LIST_ITER inst_it, bool newBB);
@@ -361,7 +367,7 @@ public:
     PI_localSchedule,
     PI_HWWorkaround,        // always
     PI_fixEndIfWhileLabels, // always
-    PI_insertHashMovs,
+    PI_HWDebug,
     PI_insertDummyMovForHWRSWA,
     PI_insertDummyCompactInst,
     PI_mergeScalarInst,
@@ -374,6 +380,7 @@ public:
     PI_createR0Copy,
     PI_initializePayload,
     PI_cleanupBindless,
+    PI_cleanupA0Movs,
     PI_countGRFUsage,
     PI_changeMoveType,
     PI_accSubBeforeRA,
@@ -396,6 +403,7 @@ public:
     PI_expandMadwPostSchedule,
     PI_ACCSchedule,
     PI_staticProfiling,
+    PI_sinkBarrierWait,
     PI_NUM_PASSES
   };
 
@@ -439,6 +447,8 @@ public:
 #endif // DLL_MODE
     initOptimizations();
   }
+  Optimizer(const Optimizer&) = delete;
+  Optimizer& operator=(const Optimizer&) = delete;
   ~Optimizer() {
     // Normally, noMask Info will be freed in postRA workaround.
     // But in case RA fails, postRA will not be invoked. Thus, need
