@@ -59,8 +59,9 @@ void Optimizer::createR0Copy() {
       builder.createDst(builder.getBuiltinR0()->getRegVar(), 0, 0, 1, Type_UD);
 
   unsigned int options = InstOpt_WriteEnable;
-  G4_INST *movInst =
-      builder.createMov(g4::SIMD8, R0CopyOpnd, R0Opnd, options, false);
+  unsigned numElt = kernel.getGRFSize() / TypeSize(Type_UD);
+  G4_INST *movInst = builder.createMov(G4_ExecSize(numElt),
+                                       R0CopyOpnd, R0Opnd, options, false);
 
   for (G4_BB *bb : kernel.fg) {
     INST_LIST_ITER ii = bb->begin();
@@ -362,7 +363,6 @@ public:
         AlignUp(k.getInt32KernelAttr(Attributes::ATTR_PerThreadInputSize),
                 k.numEltPerGRF<Type_UB>()) / k.numEltPerGRF<Type_UB>())
   {
-
     auto rtmpRegNum = k.getNumRegTotal() - 1;
     rtmp = b.createHardwiredDeclare(k.numEltPerGRF<Type_UD>(), Type_UD, rtmpRegNum, 0);
 
