@@ -62,7 +62,7 @@ void RPE::runBB(G4_BB *bb) {
     if (dst && (topdcl = dst->getTopDcl())) {
       if (topdcl->getRegVar()->isRegAllocPartaker()) {
         // Check if dst is killed
-        if (liveAnalysis->writeWholeRegion(bb, inst, dst, options) ||
+        if (liveAnalysis->writeWholeRegion(bb, inst, dst) ||
             inst->isPseudoKill()) {
           id = topdcl->getRegVar()->getId();
           updateLiveness(live, id, false);
@@ -70,7 +70,7 @@ void RPE::runBB(G4_BB *bb) {
       } else if ((LLR = gra.getLocalLR(topdcl)) && LLR->getAssigned()) {
         uint32_t firstRefIdx;
         if (LLR->getFirstRef(firstRefIdx) == inst ||
-            liveAnalysis->writeWholeRegion(bb, inst, dst, options)) {
+            liveAnalysis->writeWholeRegion(bb, inst, dst)) {
           updateLivenessForLLR(LLR, false);
         }
       }
@@ -100,7 +100,7 @@ void RPE::runBB(G4_BB *bb) {
         const REGVAR_VECTOR &pointsToSet =
             liveAnalysis->getPointsToAnalysis().getAllInPointsToOrIndrUse(src,
                                                                           bb);
-        for (auto pt : pointsToSet) {
+        for (const auto &pt : pointsToSet) {
           if (pt.var->isRegAllocPartaker()) {
             updateLiveness(live, pt.var->getId(), true);
           }
@@ -201,7 +201,7 @@ void RPE::updateRegisterPressure(bool change, bool clean,
 void RPE::recomputeMaxRP() {
   maxRP = 0;
   // Find max register pressure over all entries in map
-  for (auto item : rp) {
+  for (const auto &item : rp) {
     maxRP = std::max(maxRP, item.second);
   }
 }
