@@ -33,7 +33,8 @@ void HWConformity::fixOpndTypeAlign(G4_BB *bb) {
   for (auto iEnd = bb->end(); i != iEnd; i = next_iter) {
     G4_INST *inst = *i;
     G4_opcode opcode = inst->opcode();
-    if (opcode == G4_nop || opcode == G4_label || inst->mayExceedTwoGRF()) {
+    if (opcode == G4_nop || opcode == G4_label || inst->isSend() ||
+        inst->isDpas()) {
       next_iter++;
     } else if (fixInstOpndTypeAlign(i, bb)) {
       needSplit = true;
@@ -64,6 +65,8 @@ void HWConformity::fixOpndTypeAlign(G4_BB *bb) {
 // be 128-bit aligned with destination horizontal stride equivalent to a
 // word for an immediate integer vector (v) and equivalent to a DWord for an
 // immediate float vector (vf).
+// For Xe2+, :vf datatype is not supported and :v and :uv must not be used
+// when destination is any of the float datatypes.
 bool HWConformity::fixDstAlignmentWithVectorImm(INST_LIST_ITER iter,
                                                 G4_BB *bb) {
   bool changed = false;
@@ -86,7 +89,8 @@ bool HWConformity::fixDstAlignmentWithVectorImm(INST_LIST_ITER iter,
     G4_Type moveTy = (ty == Type_V)    ? Type_W
                      : (ty == Type_UV) ? Type_UW
                                        : Type_F;
-    if (!dstAligned) {
+    if (!dstAligned || (builder.getPlatform() >= Xe2 &&
+                        IS_TYPE_FLOAT_ALL(reg->getType()))) {
       inst->setSrc(insertMovBefore(iter, k, moveTy, bb), k);
       changed = true;
     } else if (hsInBytes != TypeSize(moveTy)) {
@@ -95,10 +99,10 @@ bool HWConformity::fixDstAlignmentWithVectorImm(INST_LIST_ITER iter,
         // we can interleave the vector to avoid a move
         // e.g., mov (2) r1.0<1>:d 0x21:uv  -->
         //       mov (2) r1.0<1>:d 0x0201:uv
-        uint32_t bitValue = 0;
-        uint16_t immBits = (uint16_t)src->asImm()->getImm();
+        uint64_t bitValue = 0;
+        uint32_t immBits = static_cast<uint32_t>(src->asImm()->getImm());
         for (int i = 0; i < execSize; ++i) {
-          int val = (immBits >> (i * 4)) & 0xF;
+          uint64_t val = (immBits >> (i * 4)) & 0xF;
           bitValue |= val << (i * 8);
         }
         inst->setSrc(builder.createImm(bitValue, ty), k);
@@ -255,7 +259,8 @@ void HWConformity::fixInstExecSize(G4_BB *bb) {
     next_iter++;
     G4_INST *inst = *i;
     G4_opcode opcode = inst->opcode();
-    if (opcode == G4_nop || opcode == G4_label || inst->mayExceedTwoGRF()) {
+    if (opcode == G4_nop || opcode == G4_label || inst->isSend() ||
+        inst->isDpas()) {
       continue;
     }
 
