@@ -40,7 +40,7 @@ G4_Declare *SpillManager::createNewSpillLocDeclare(G4_Declare *dcl) {
     vISA_ASSERT(type == Type_UW || type == Type_W || type == Type_UD ||
                     type == Type_D,
                 "addr reg's type should be UW or UD");
-    vISA_ASSERT(dcl->getNumElems() <= getNumAddrRegisters(),
+    vISA_ASSERT(dcl->getNumElems() <= builder.getNumAddrRegisters(),
                 "Addr reg Spill size exceeds limit");
   }
   G4_Declare *sp = dcl->getSpilledDeclare();
@@ -62,8 +62,8 @@ G4_Declare *SpillManager::createNewTempAddrDeclare(G4_Declare *dcl) {
   vISA_ASSERT(dcl->getElemType() == Type_UW || dcl->getElemType() == Type_W,
               "addr reg's type should be UW");
   vISA_ASSERT(dcl->getNumRows() == 1, "Temp_ADDR should be only 1 row");
-  vISA_ASSERT(dcl->getNumElems() <= getNumAddrRegisters(),
-              "Temp_ADDR exceeds 16 bytes");
+  vISA_ASSERT(dcl->getNumElems() <= builder.getNumAddrRegisters(),
+              "Temp_ADDR element number exceeds maximum value");
   G4_Declare *sp = builder.createDeclare(name, G4_ADDRESS, dcl->getNumElems(),
                                          1, // 1 row
                                          Type_UW);
@@ -101,7 +101,7 @@ G4_Declare *SpillManager::createNewTempAddrDeclare(G4_Declare *dcl,
                   type == Type_D,
               "addr reg's type should be UW or UD");
   vISA_ASSERT(dcl->getNumRows() == 1, "Temp_ADDR should be only 1 row");
-  vISA_ASSERT(dcl->getNumElems() <= getNumAddrRegisters(),
+  vISA_ASSERT(dcl->getNumElems() <= builder.getNumAddrRegisters(),
               "Temp_ADDR exceeds 16 bytes");
   G4_Declare *sp = builder.createDeclare(name, G4_ADDRESS, num_reg,
                                          1, // 1 row
@@ -126,7 +126,8 @@ void SpillManager::genRegMov(G4_BB *bb, INST_LIST_ITER it, G4_VarBase *src,
   builder.instList.clear();
 
   uint16_t dSubRegOff = 0;
-  for (uint16_t i = 16; i != 0 && nRegs != 0; i >>= 1) // 16, 8, 4, 2, 1
+  for (uint16_t i = builder.getNumAddrRegisters(); i != 0 && nRegs != 0;
+       i >>= 1) // 16, 8, 4, 2, 1
   {
     if (nRegs >= i) {
       //
@@ -195,8 +196,8 @@ void SpillManager::genRegMov(G4_BB *bb, INST_LIST_ITER it, G4_VarBase *src,
 void SpillManager::replaceSpilledDst(
     G4_BB *bb,
     INST_LIST_ITER it, // where new insts will be inserted
-    G4_INST *inst, G4_Operand **operands_analyzed,
-    G4_Declare **declares_created) {
+    G4_INST *inst, std::vector<G4_Operand *> &operands_analyzed,
+    std::vector<G4_Declare *> &declares_created) {
   G4_DstRegRegion *dst = inst->getDst();
   if (dst == NULL)
     return;
@@ -233,7 +234,7 @@ void SpillManager::replaceSpilledDst(
       G4_Declare *tmpDcl = NULL;
       bool match_found = false;
 
-      for (unsigned int j = 0; j < G4_MAX_SRCS; j++) {
+      for (unsigned int j = 0, je = operands_analyzed.size(); j < je; j++) {
         G4_SrcRegRegion *analyzed_src =
             static_cast<G4_SrcRegRegion *>(operands_analyzed[j]);
         if (analyzed_src != NULL &&
@@ -276,8 +277,8 @@ void SpillManager::replaceSpilledDst(
 void SpillManager::replaceSpilledSrc(
     G4_BB *bb,
     INST_LIST_ITER it, // where new insts will be inserted
-    G4_INST *inst, unsigned i, G4_Operand **operands_analyzed,
-    G4_Declare **declares_created) {
+    G4_INST *inst, unsigned i, std::vector<G4_Operand *> &operands_analyzed,
+    std::vector<G4_Declare *> &declares_created) {
   G4_Operand *src = inst->getSrc(i);
   if (src == NULL)
     return;
@@ -367,6 +368,9 @@ void SpillManager::replaceSpilledSrc(
         genRegMov(bb, it, spDcl->getRegVar(), ss->getSubRegOff(),
                   tmpDcl->getRegVar(), tmpDcl->getNumElems(),
                   isNoMask);
+        auto pseudoKill =
+          builder.createPseudoKill(tmpDcl, PseudoKillType::Other, false);
+        bb->insertBefore(std::prev(it), pseudoKill);
       }
 
       // create new src from the temp address variable, with offset 0
@@ -668,10 +672,11 @@ void SpillManager::insertSpillCode() {
 
       currCISAOffset = inst->getVISAId();
 
-      G4_Operand *operands_analyzed[G4_MAX_SRCS] = {};
-      G4_Declare *declares_created[G4_MAX_SRCS] = {};
+      const unsigned numSrc = inst->getNumSrc();
+      std::vector<G4_Operand *> operands_analyzed(numSrc, nullptr);
+      std::vector<G4_Declare *> declares_created(numSrc, nullptr);
       // insert spill inst for spilled srcs
-      for (unsigned i = 0; i < G4_MAX_SRCS; i++) {
+      for (unsigned i = 0; i < numSrc; i++) {
         replaceSpilledSrc(bb, inst_it, inst, i, operands_analyzed,
                           declares_created);
       }
