@@ -37,12 +37,13 @@ class LSLiveRange;
 class PointsToAnalysis;
 class GlobalRA;
 class GraphColor;
+class G4_INST;
 } // namespace vISA
 struct RegionDesc;
 using LiveRangeVec = std::vector<vISA::LiveRange*>;
+
 // Class definitions
 namespace vISA {
-
 // New fail safeRA implemenetation
 class BoundedRA {
 public:
@@ -263,8 +264,6 @@ public:
 
   bool spillLiveRanges(G4_Kernel *kernel);
 
-  unsigned getNumGRFSpill() const { return numGRFSpill; }
-  unsigned getNumGRFFill() const { return numGRFFill; }
   // return the next cumulative logical offset. This does not non-spilled stuff
   // like private variables placed by IGC (marked by spill_mem_offset) this
   // should only be called after insertSpillFillCode()
@@ -603,7 +602,7 @@ private:
   unsigned latestImplicitVarIdCount_;
   const LivenessAnalysis *lvInfo_;
   const LiveRangeVec *lrInfo_;
-  const LR_LIST *spilledLRs_;
+  const LR_LIST *spilledLRs_ = nullptr;
   LSLR_LIST *spilledLSLRs_;
   unsigned spillRangeCount_ = 0;
   unsigned fillRangeCount_ = 0;
@@ -617,19 +616,12 @@ private:
   bool doSpillSpaceCompression;
 
   bool failSafeSpill_;
-  unsigned spillRegStart_;
-  unsigned indrSpillRegStart_;
-  unsigned spillRegOffset_;
-  LSLR_LIST activeLR_;
+  unsigned spillRegStart_ = 0;
+  unsigned indrSpillRegStart_ = 0;
+  unsigned spillRegOffset_ = 0;
   std::unordered_set<G4_DstRegRegion *> noRMWNeeded;
 
-  const Interference *spillIntf_;
-
-  // The number of GRF spill.
-  unsigned numGRFSpill = 0;
-
-  // The number of GRF fill.
-  unsigned numGRFFill = 0;
+  const Interference *spillIntf_ = nullptr;
 
   // CISA instruction id of current instruction
   G4_INST *curInst;
@@ -637,7 +629,6 @@ private:
   int globalScratchOffset;
 
   const bool useScratchMsg_;
-  bool avoidDstSrcOverlap_;
   // spilled declares that represent a scalar immediate (created due to encoding
   // restrictions) We rematerialize the immediate value instead of spill/fill
   // them. Map stores type used on dst that defined the immediate.
@@ -731,45 +722,6 @@ static bool immFillCandidate(G4_INST *inst) {
 
 G4_SrcRegRegion *getSpillFillHeader(IR_Builder &builder, G4_Declare *decl);
 
-// Class used to analyze spill/fill decisions
-class SpillAnalysis {
-public:
-  SpillAnalysis() = default;
-  ~SpillAnalysis();
-
-  void Dump(std::ostream &OS = std::cerr);
-  void DumpHistogram(std::ostream &OS = std::cerr);
-
-  unsigned int GetDistance(G4_Declare *Dcl);
-  void LoadAugIntervals(DECLARE_LIST &, GlobalRA &);
-  void LoadDegree(G4_Declare *Dcl, unsigned int degree);
-
-  void SetLivenessAnalysis(LivenessAnalysis *L) { LA = L; }
-  void SetGraphColor(GraphColor *C) { GC = C; }
-  void SetSpillManager(SpillManagerGRF *S) { SM = S; }
-
-  void Clear();
-
-  void Do(LivenessAnalysis *L, GraphColor *C, SpillManagerGRF *S);
-
-private:
-  VarReferences *Refs = nullptr;
-
-  LivenessAnalysis *LA = nullptr;
-  GraphColor *GC = nullptr;
-  SpillManagerGRF *SM = nullptr;
-
-  std::unordered_map<G4_Declare *, std::pair<G4_INST *, G4_INST *>>
-      AugIntervals;
-  std::unordered_map<G4_Declare *, unsigned int> DclDegree;
-
-  std::vector<G4_BB *> GetLiveBBs(G4_Declare *,
-                                  std::unordered_map<G4_INST *, G4_BB *> &);
-  std::vector<G4_BB *>
-  GetIntervalBBs(G4_INST *Start, G4_INST *End,
-                 std::unordered_map<G4_INST *, G4_BB *> &InstBBMap);
-};
-
 bool isEOTSpillWithFailSafeRA(const IR_Builder &builder, const LiveRange *lr,
                               bool isFailSafeIter);
 
