@@ -71,7 +71,7 @@ int IR_Builder::translateVISAArithmeticInst(
       predOpnd = nullptr;
     }
 
-    if (opcode == ISA_MADW) {
+    if (opcode == ISA_MADW && !supportNativeSIMD32()) {
       vISA_ASSERT_INPUT(exsize <= 16,
                         "Unsupported execution size for madw");
     }
@@ -124,7 +124,8 @@ int IR_Builder::translateVISADpasInst(VISA_Exec_Size executionSize,
                                       G4_SrcRegRegion *src0Opnd,
                                       G4_SrcRegRegion *src1Opnd,
                                       G4_SrcRegRegion *src2Opnd,
-                                      G4_SrcRegRegion *src3Opnd, GenPrecision A,
+                                      G4_SrcRegRegion *src3Opnd,
+                                      G4_SrcRegRegion *src4Opnd, GenPrecision A,
                                       GenPrecision W, uint8_t D, uint8_t C) {
   TIME_SCOPE(VISA_BUILDER_IR_CONSTRUCTION);
 
@@ -150,7 +151,7 @@ int IR_Builder::translateVISADpasInst(VISA_Exec_Size executionSize,
   }
 
   createDpasInst(opc, exsize, dstOpnd, src0Opnd, src1Opnd, src2Opnd, src3Opnd,
-                 instOpt, A, W, D, C, true);
+                 src4Opnd, instOpt, A, W, D, C, true);
 
   return VISA_SUCCESS;
 }
@@ -193,7 +194,7 @@ int IR_Builder::translateVISACompareInst(
   G4_CondMod *condMod = NULL;
   G4_ExecSize exsize = toExecSize(execsize);
   G4_InstOpts inst_opt = Get_Gen4_Emask(emask, exsize);
-  const char *varName = getNameString(50, "PTemp_%d", kernel.Declares.size());
+  const char *varName = getNameString(50, "PTemp_%" PRIu64, kernel.Declares.size());
 
   uint8_t numWords = (exsize + 15) / 16;
   if (needs32BitFlag(inst_opt)) {
