@@ -153,7 +153,6 @@ static void Copy_Source_To_Payload(IR_Builder *IRB, G4_ExecSize batchExSize,
   }
 }
 
-
 void IR_Builder::preparePayload(G4_SrcRegRegion *msgs[2], unsigned sizes[2],
                                 G4_ExecSize batchExSize, bool splitSendEnabled,
                                 PayloadSource srcs[], unsigned len) {
@@ -290,6 +289,7 @@ void IR_Builder::preparePayload(G4_SrcRegRegion *msgs[2], unsigned sizes[2],
 }
 
 G4_SrcRegRegion *IR_Builder::coalescePayload(
+    G4_Predicate *pred,
     unsigned sourceAlignment, unsigned payloadAlignment,
     uint32_t payloadWidth, // number of elements for one payload in the send.
     uint32_t srcSize,      // number of elements provided by src
@@ -371,7 +371,8 @@ G4_SrcRegRegion *IR_Builder::coalescePayload(
           G4_SrcRegRegion *srcRegion = createSrc(src->getTopDcl()->getRegVar(),
                                                  src->getRegOff() + rowOffset,
                                                  0, getRegionStride1(), type);
-          createMov(MAX_SIMD, dstRegion, srcRegion, instOpt, true);
+
+          createMov(duplicateOperand(pred), MAX_SIMD, dstRegion, srcRegion, instOpt, true);
           moveMask = Get_Next_EMask(moveMask, MAX_SIMD);
         }
       };
@@ -468,6 +469,13 @@ int IR_Builder::generateDebugInfoPlaceholder() {
   return VISA_SUCCESS;
 }
 
+int IR_Builder::translateBreakpointInstruction() {
+  // create breakpoint intrinsic
+  createIntrinsicInst(nullptr, Intrinsic::Breakpoint, g4::SIMD1, nullptr,
+      nullptr, nullptr, nullptr, InstOpt_BreakPoint, true);
+  return VISA_SUCCESS;
+}
+
 int IR_Builder::translateVISALifetimeInst(bool isStart, G4_Operand *var) {
   // Lifetime.start/end are two variants of this instruction
   if (isStart) {
