@@ -8,8 +8,4 @@ SPDX-License-Identifier: MIT
 
 extern "C" int iga_main(int argc, const char **argv);
 
-int main(int argc, const char **argv)
-{
-    return iga_main(argc, argv);
-}
-
+int main(int argc, const char **argv) { return iga_main(argc, argv); }
