@@ -806,6 +806,7 @@ static inline SFMessageType translate(GED_MESSAGE_TYPE fc) {
   }
 }
 
+
 static inline MathFC translate(GED_MATH_FC fc) {
   switch (fc) {
   case GED_MATH_FC_INV:
@@ -950,6 +951,8 @@ static inline SyncFC translate(GED_SYNC_FC fc) {
     return SyncFC::ALLWR;
   case GED_SYNC_FC_bar:
     return SyncFC::BAR;
+  case GED_SYNC_FC_flush:
+    return SyncFC::FLUSH;
   case GED_SYNC_FC_fence:
     return SyncFC::FENCE;
   case GED_SYNC_FC_host:
