@@ -327,6 +327,7 @@ public:
     const auto exDesc = i.getExtMsgDescriptor();
     const auto sfid = i.getSendFc();
     const auto di = tryDecode(platform(), sfid, i.getExecSize(),
+                              i.getExtImmOffDescriptor(),
                               exDesc, desc, nullptr);
     if (!di) {
       // if message decode failed fallback to the canonical send syntax
@@ -393,15 +394,15 @@ public:
   /////////////////////////////////////////////////////////////////////////
   // op+subfunc+exec size and exec offset
   void emitPredOpSubfuncExecInfoFlagReg(const Instruction &i,
-                                        std::string mnemonic,
-                                        std::string subop) {
+                                        const std::string& mnemonic,
+                                        const std::string& subop) {
     emitPred(i);
 
     emit(", \"op\":\"", mnemonic, "\"");
     if (subop.empty()) {
       emit(", \"subop\":null");
     } else {
-      auto sf = subop[0] == '.' ? subop.substr(1) : subop;
+      const auto& sf = subop[0] == '.' ? subop.substr(1) : subop;
       emit(", \"subop\":\"", sf, "\"");
     }
 
