@@ -126,6 +126,7 @@ class InstBuilder {
   OperandInfo m_srcs[3];
   int m_nSrcs = 0;
 
+  uint32_t m_exImmOffDesc;
   SendDesc m_exDesc;
   SendDesc m_desc;
 
@@ -224,6 +225,7 @@ private:
     }
     m_nSrcs = 0;
 
+    m_exImmOffDesc = 0;
     m_exDesc.imm = 0;
     m_desc.imm = 0;
 
@@ -343,6 +345,7 @@ public:
       inst = m_kernel->createSendInstruction(*m_opSpec, m_subfunc.send,
                                              m_predication, m_flagReg,
                                              m_execSize, m_chOff, m_maskCtrl,
+                                             m_exImmOffDesc,
                                              m_exDesc, m_desc);
       if (m_sendSrc0Len >= 0)
         inst->setSrc0Length(m_sendSrc0Len);
@@ -446,7 +449,7 @@ public:
 
     SWSB::InstType instType = inst->getSWSBInstType(m_swsbEncodeMode);
     if (!swInfo.verify(m_swsbEncodeMode, instType)) {
-      m_errorHandler.reportError(inst->getLoc(), "invalid SWSB mode");
+      m_errorHandler.reportError(inst->getLoc(), "invalid SWSB");
     }
 
     inst->setSWSB(swInfo);
@@ -505,7 +508,7 @@ public:
   void InstDstOpRegDirect(const Loc &loc, RegName rn, int reg,
                           Region::Horz rgnHorz, Type ty) {
     RegRef rr;
-    rr.regNum = (uint8_t)reg;
+    rr.regNum = (uint16_t)reg;
     InstDstOpRegDirect(loc, rn, rr, rgnHorz, ty);
   }
   void InstDstOpRegDirect(const Loc &loc, RegName rn, RegRef reg,
@@ -530,8 +533,8 @@ public:
 
     m_dst.regOpDstMod = m_dstModifier;
     m_dst.regOpName = rnm;
-    m_dst.regOpReg = RegRef(static_cast<uint8_t>(regNum), 0);
-    m_dst.regOpReg.regNum = (uint8_t)regNum;
+    m_dst.regOpReg = RegRef((uint16_t)regNum, (uint16_t)0);
+    m_dst.regOpReg.regNum = (uint16_t)regNum;
     m_dst.regOpRgn.setDstHz(rgnH);
     m_dst.regOpMathMacroExtReg = mme;
     m_dst.type = ty;
@@ -574,7 +577,7 @@ public:
                           int reg,     // register/subregister
                           Region rgn,  // region parameters
                           Type ty) {
-    RegRef rr((uint8_t)reg, 0);
+    RegRef rr((uint16_t)reg, (uint16_t)0);
     InstSrcOpRegDirect(srcOpIx, loc, SrcModifier::NONE, rnm, rr, rgn, ty);
   }
   void
@@ -610,7 +613,7 @@ public:
     src.kind = Operand::Kind::MACRO;
     src.regOpSrcMod = srcMod;
     src.regOpName = rnm;
-    src.regOpReg.regNum = (uint8_t)regNum;
+    src.regOpReg.regNum = (uint16_t)regNum;
     src.regOpReg.subRegNum = 0;
     src.regOpRgn = rgn;
     src.regOpMathMacroExtReg = MathMacroExt;
@@ -730,11 +733,19 @@ public:
   // E.g. "send ... 0xC  a0.0"
   // (we translate  this to:  a0.0<0;1,0>:ud)
   void InstSendDescs(const Loc &,
+                     uint32_t exImmOffDesc,
                      const SendDesc &exDesc, const Loc &,
                      const SendDesc &desc) {
+    m_exImmOffDesc = exImmOffDesc;
     m_exDesc = exDesc;
     m_desc = desc;
   }
+
+  void InstSendDescs(const Loc &exDescLoc, const SendDesc &exDesc,
+                     const Loc &descLoc, const SendDesc &desc) {
+    InstSendDescs(exDescLoc, 0, exDesc, descLoc, desc);
+  }
+
   void InstSendSrc0Length(int src0Length) { m_sendSrc0Len = src0Length; }
   void InstSendSrc1Length(int src1Length) { m_sendSrc1Len = src1Length; }
 
@@ -832,7 +843,7 @@ public:
 
 
   // sets Instruction::setComment(...)
-  void InstComment(std::string comment) { m_comment = comment; }
+  void InstComment(const std::string& comment) { m_comment = comment; }
 
   // set inline binary instruction
   void InstInlineBinary(const Instruction::InlineBinaryType &binary) {
