@@ -86,7 +86,7 @@ decodeFieldsWithWarnings(const Model &model, std::ostream &os, Loc loc,
   FragmentList fields;
   ErrorHandler errHandler;
   native::DecodeFields(loc, model, (const void *)mi, fields, errHandler);
-  for (auto &e : errHandler.getErrors()) {
+  for (const auto &e : errHandler.getErrors()) {
     std::stringstream ss;
     ss << "PC" << e.at.offset << ". " << e.message << "\n";
     emitYellowText(os, ss.str());
@@ -677,6 +677,8 @@ iga_status_t iga::DebugCompaction(Platform p, int /* verbosity */,
 
   CompactionStats cs;
 
+  std::ios_base::fmtflags osFlags = os.flags();
+
   bool success = true;
   size_t pc = 0;
   while (pc < bitsLen) {
@@ -729,7 +731,7 @@ iga_status_t iga::DebugCompaction(Platform p, int /* verbosity */,
     const CompactionMapping *cf = missEntry.first;
     os << std::setw(24) << cf->index.name;
     int64_t misses = 0;
-    for (const auto ms : missEntry.second) {
+    for (const auto& ms : missEntry.second) {
       misses += ms.second.misses.size();
     }
     totalMisses += misses;
@@ -747,7 +749,7 @@ iga_status_t iga::DebugCompaction(Platform p, int /* verbosity */,
 
     // order the misses by the total number
     std::vector<std::pair<Mapping, MappingStats>> orderedEntries;
-    for (const auto ms : missEntry.second) {
+    for (const auto& ms : missEntry.second) {
       orderedEntries.push_back(ms);
     }
     std::sort(orderedEntries.begin(), orderedEntries.end(),
@@ -798,5 +800,7 @@ iga_status_t iga::DebugCompaction(Platform p, int /* verbosity */,
     }
   }
 
+  os.flags(osFlags);
+
   return IGA_SUCCESS;
 }
