@@ -138,7 +138,7 @@ struct PlatformNameMap {
           break;
         nmlist.push_back(str);
       }
-      names[me.platform] = nmlist;
+      names[me.platform] = std::move(nmlist);
       exts[me.platform] = me.extension.str();
     }
   }
@@ -276,6 +276,8 @@ public:
     memset(m_empty_string, 0, sizeof(m_empty_string));
   }
 
+  IGAContext(const IGAContext&) = delete;
+  IGAContext& operator=(const IGAContext&) = delete;
   ~IGAContext() {
     m_validToken = 0xDEADDEADDEADDEADull;
     clearDiagnostics(m_warnings);
@@ -336,8 +338,6 @@ public:
     } else if (pKernel == nullptr) {
       // parser returned nullptr for kernel, but with no errors
       // shouldn't be reachable; implies we have a missing diagnostic
-      if (pKernel)
-        delete pKernel;
       return IGA_ERROR;
     }
 
@@ -356,7 +356,8 @@ public:
     eopts.autoDepSet =
         (aopts.encoder_opts & IGA_ENCODER_OPT_AUTO_DEPENDENCIES) != 0;
     eopts.sbidCount = aopts.sbid_count;
-    eopts.swsbEncodeMode = aopts.swsb_encode_mode;
+    eopts.swsbEncodeMode =
+        static_cast<iga::SWSB_ENCODE_MODE>(aopts.swsb_encode_mode);
 
     if ((aopts.encoder_opts & IGA_ENCODER_OPT_USE_NATIVE) == 0) {
       if (!iga::ged::IsEncodeSupported(m_model, eopts)) {
@@ -873,12 +874,12 @@ iga_status_t iga_diagnostic_get_text_extent(const iga_diagnostic_t *d,
 // internal data structures.
 static iga_opspec_t opspec_to_handle(const OpSpec *os) {
   const uintptr_t TOP_BIT =
-      (sizeof(void *) == 4) ? 0xC0000000 : 0x8000000000000000;
+      (sizeof(void *) == 4) ? (uintptr_t)0xC0000000 : (uintptr_t)0x8000000000000000;
   return (iga_opspec_t)((uintptr_t)os ^ TOP_BIT);
 }
 static const OpSpec *opspec_from_handle(iga_opspec_t os) {
   const uintptr_t TOP_BIT =
-      (sizeof(void *) == 4) ? 0xC0000000 : 0x8000000000000000;
+      (sizeof(void *) == 4) ? (uintptr_t)0xC0000000 : (uintptr_t)0x8000000000000000;
   return (const OpSpec *)((uintptr_t)os ^ TOP_BIT);
 }
 
