@@ -41,7 +41,7 @@ enum GenSymType {
 /// GenSymEntry - An symbol table entry
 typedef struct {
   uint32_t s_type;   // The symbol's type
-  uint32_t s_offset; // The binary offset of this symbol. This field is ignored
+  size_t s_offset; // The binary offset of this symbol. This field is ignored
                      // if s_type is S_UNDEF
   uint32_t s_size;   // The size in bytes of the function binary
   char s_name[MAX_SYMBOL_NAME_LENGTH]; // The symbol's name
@@ -65,17 +65,6 @@ typedef struct {
   char r_symbol[MAX_SYMBOL_NAME_LENGTH]; // The relocation target symbol's name
 } GenRelocEntry;
 
-/// GenFuncAttribEntry - Per-function attribute entry
-typedef struct {
-  uint8_t f_isKernel;             // Is the function a kernel
-  uint8_t f_hasBarrier;           // Does the function use barriers
-  uint32_t f_privateMemPerThread; // Total private memory (in bytes) used by
-                                  // this function per thread
-  uint32_t f_spillMemPerThread;   // Spill mem used (in bytes) in scratch space
-                                  // for this function
-  char f_name[MAX_SYMBOL_NAME_LENGTH]; // The function's name
-} GenFuncAttribEntry;
-
 // HostAccessEntry - per global variable host access entry
 typedef struct {
   char device_name[MAX_SYMBOL_NAME_LENGTH];
@@ -91,14 +80,14 @@ typedef struct {
 /// FIXME: s_type should be standard ELF symbol type instead of GenSymType
 struct ZESymEntry {
   GenSymType s_type;  // The symbol's type
-  uint32_t s_offset;  // The binary offset of this symbol. This field is ignored
+  size_t s_offset;  // The binary offset of this symbol. This field is ignored
                       // if s_type is S_UNDEF
   uint32_t s_size;    // The size in bytes of the function binary
   std::string s_name; // The symbol's name
 
   ZESymEntry() = default;
-  ZESymEntry(GenSymType type, uint32_t offset, uint32_t size, std::string name)
-      : s_type(type), s_offset(offset), s_size(size), s_name(name) {}
+  ZESymEntry(GenSymType type, size_t offset, uint32_t size, std::string name)
+      : s_type(type), s_offset(offset), s_size(size), s_name(std::move(name)) {}
 };
 
 /// ZERelocEntry - A relocation entry that will later be transformed to ZE
@@ -112,7 +101,7 @@ struct ZERelocEntry {
 
   ZERelocEntry() = default;
   ZERelocEntry(GenRelocType type, uint32_t offset, std::string targetSymName)
-      : r_type(type), r_offset(offset), r_symbol(targetSymName) {}
+      : r_type(type), r_offset(offset), r_symbol(std::move(targetSymName)) {}
 };
 
 /// ZEFuncAttribEntry - A function attribute entry that will later be
@@ -134,7 +123,7 @@ struct ZEFuncAttribEntry {
       : f_isKernel(isKernel), f_isExternal(isExternal),
         f_BarrierCount(barrierCount),
         f_privateMemPerThread(privateMemPerThread),
-        f_spillMemPerThread(spillMemPerThread), f_name(funcName), f_hasRTCalls(hasRTCalls) {}
+        f_spillMemPerThread(spillMemPerThread), f_name(std::move(funcName)), f_hasRTCalls(hasRTCalls) {}
 };
 
 /// ZEHostAccessEntry - A host access entry that will later be transformed to ZE
