@@ -460,6 +460,8 @@ DEF_VISA_OPTION(vISA_LocalSchedulingStartBB, ET_INT32, "-scheduleStartBB",
 DEF_VISA_OPTION(vISA_LocalSchedulingEndBB, ET_INT32, "-scheduleEndBB", UNUSED,
                 UINT_MAX)
 DEF_VISA_OPTION(vISA_assumeL1Hit, ET_BOOL, "-assumeL1Hit", UNUSED, false)
+DEF_VISA_OPTION(vISA_ignoreL1Hit, ET_BOOL, "-ignoreL1Hit",
+                "Ignore LSC L1Hit cache option when calculating latency in scheduling", false)
 DEF_VISA_OPTION(vISA_writeCombine, ET_BOOL, "-writeCombine", UNUSED, true)
 DEF_VISA_OPTION(vISA_Q2FInIntegerPipe, ET_BOOL, "-Q2FInteger", UNUSED, false)
 DEF_VISA_OPTION(vISA_LocalScheduleingStartKernel, ET_INT32,
@@ -526,6 +528,8 @@ DEF_VISA_OPTION(vISA_SplitMov64, ET_INT32, "-SplitMov64",
 DEF_VISA_OPTION(vISA_has4DeepSystolic, ET_BOOL, "-has4DeepSystolic", UNUSED,
                 false)
 
+DEF_VISA_OPTION(vISA_PVCSendWARWA, ET_BOOL_TRUE, "-PVCSendWARWA",
+                "enable Send WAR walkaround for PVC", false)
 DEF_VISA_OPTION(vISA_SWSBMakeLocalWAR, ET_BOOL_TRUE, "-SWSBMakeLocalWAR",
                 "Enable WAR Sync at the end of BB", false)
 DEF_VISA_OPTION(vISA_SWSBReplaceARWithAW, ET_BOOL_TRUE, "-SWSBReplaceARWithAW",
@@ -646,6 +650,8 @@ DEF_VISA_OPTION(vISA_dumpKCI, ET_BOOL, "-dumpKCI",
                 "Dump info collected by kernelCostInfo.", false)
 DEF_VISA_OPTION(vISA_dumpDetailKCI, ET_BOOL, "-dumpDetailKCI",
                 "Dump detailed info collected by kernelCostInfo.", false)
+DEF_VISA_OPTION(vISA_dumpKCIForLit, ET_BOOL, "-dumpKCIForLit",
+                "Dump kernel cost info for lit testing only", false)
 // Corresponds to something slightly different in IGC than vISA_HasInt64Add
 // (C.f. Platform.hpp:hasPartialInt64Support)
 DEF_VISA_OPTION(vISA_HasPartialInt64, ET_BOOL, "-partialInt64", UNUSED, false)
@@ -739,6 +745,10 @@ DEF_VISA_OPTION(vISA_noIndirectSrcForCompressedInstWA, ET_BOOL,
                 "Disable WA for fixing compressed instructions with indirect "
                 "src0 and cross-grf dst",
                 false)
+DEF_VISA_OPTION(vISA_TGMDoubleFenceWA, ET_BOOL, "-tgmDoubleFenceWA",
+                "enable the WA to double any TGM fence instruction with "
+                "flush-type is not FLUSH_NONE",
+                false)
 
 //=== HW debugging options ===
 DEF_VISA_OPTION(vISA_GenerateDebugInfo, ET_BOOL, "-generateDebugInfo", UNUSED,
